@@ -1,4 +1,4 @@
-/* $ANTLR 2.7.7 (20120518): "format.g" -> "FMTLexer.cpp"$ */
+/* $ANTLR 2.7.7 (20130428): "format.g" -> "FMTLexer.cpp"$ */
 
 #include "includefirst.hpp"
 
@@ -31,14 +31,8 @@ FMTLexer::FMTLexer(const antlr::LexerSharedInputState& state)
 
 void FMTLexer::initLiterals()
 {
-	literals["cyi"] = 64;
-	literals["csi"] = 66;
-	literals["cmi"] = 65;
 	literals["tl"] = 33;
 	literals["tr"] = 34;
-	literals["cdi"] = 63;
-	literals["cmoi"] = 62;
-	literals["csf"] = 67;
 }
 
 antlr::RefToken FMTLexer::nextToken()
@@ -216,44 +210,48 @@ antlr::RefToken FMTLexer::nextToken()
 				theRetToken=_returnToken;
 				break;
 			}
-			case 0x2d /* '-' */ :
-			case 0x30 /* '0' */ :
-			case 0x31 /* '1' */ :
-			case 0x32 /* '2' */ :
-			case 0x33 /* '3' */ :
-			case 0x34 /* '4' */ :
-			case 0x35 /* '5' */ :
-			case 0x36 /* '6' */ :
-			case 0x37 /* '7' */ :
-			case 0x38 /* '8' */ :
-			case 0x39 /* '9' */ :
-			{
-				mNUMBER(true);
-				theRetToken=_returnToken;
-				break;
-			}
 			default:
-				if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x4d /* 'M' */ ) && (LA(3) == 0x4f /* 'O' */ )) {
+				if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x4d /* 'M' */ ) && (LA(3) == 0x4f /* 'O' */ ) && (LA(4) == 0x41 /* 'A' */ )) {
 					mCMOA(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x4d /* 'M' */ ) && (LA(3) == 0x6f /* 'o' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x4d /* 'M' */ ) && (LA(3) == 0x4f /* 'O' */ ) && (LA(4) == 0x49 /* 'I' */ )) {
+					mCMOI(true);
+					theRetToken=_returnToken;
+				}
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x4d /* 'M' */ ) && (LA(3) == 0x6f /* 'o' */ )) {
 					mCMoA(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x44 /* 'D' */ ) && (LA(3) == 0x57 /* 'W' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x44 /* 'D' */ ) && (LA(3) == 0x49 /* 'I' */ )) {
+					mCDI(true);
+					theRetToken=_returnToken;
+				}
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x4d /* 'M' */ ) && (LA(3) == 0x49 /* 'I' */ )) {
+					mCMI(true);
+					theRetToken=_returnToken;
+				}
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x53 /* 'S' */ ) && (LA(3) == 0x49 /* 'I' */ )) {
+					mCSI(true);
+					theRetToken=_returnToken;
+				}
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x53 /* 'S' */ ) && (LA(3) == 0x46 /* 'F' */ )) {
+					mCSF(true);
+					theRetToken=_returnToken;
+				}
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x44 /* 'D' */ ) && (LA(3) == 0x57 /* 'W' */ )) {
 					mCDWA(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x44 /* 'D' */ ) && (LA(3) == 0x77 /* 'w' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x44 /* 'D' */ ) && (LA(3) == 0x77 /* 'w' */ )) {
 					mCDwA(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x41 /* 'A' */ ) && (LA(3) == 0x50 /* 'P' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x41 /* 'A' */ ) && (LA(3) == 0x50 /* 'P' */ )) {
 					mCAPA(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x41 /* 'A' */ ) && (LA(3) == 0x70 /* 'p' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x41 /* 'A' */ ) && (LA(3) == 0x70 /* 'p' */ )) {
 					mCApA(true);
 					theRetToken=_returnToken;
 				}
@@ -261,26 +259,38 @@ antlr::RefToken FMTLexer::nextToken()
 					mCSTRING(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x6d /* 'm' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x6d /* 'm' */ )) {
 					mCmoA(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x68 /* 'h' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x59 /* 'Y' */ )) {
+					mCYI(true);
+					theRetToken=_returnToken;
+				}
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x48 /* 'H' */ )) {
 					mCHI(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x48 /* 'H' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x68 /* 'h' */ )) {
 					mChI(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x64 /* 'd' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x64 /* 'd' */ )) {
 					mCdwA(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (LA(2) == 0x61 /* 'a' */ )) {
+				else if ((LA(1) == 0x43 /* 'C' */ ) && (LA(2) == 0x61 /* 'a' */ )) {
 					mCapA(true);
 					theRetToken=_returnToken;
 				}
+				else if ((LA(1) == 0x2b /* '+' */ ) && (LA(2) == 0x2d /* '-' */ )) {
+					mPM(true);
+					theRetToken=_returnToken;
+				}
+				else if ((LA(1) == 0x2d /* '-' */ ) && (LA(2) == 0x2b /* '+' */ )) {
+					mMP(true);
+					theRetToken=_returnToken;
+				}
 				else if ((LA(1) == 0x43 /* 'C' */  || LA(1) == 0x63 /* 'c' */ ) && (true)) {
 					mC(true);
 					theRetToken=_returnToken;
@@ -289,6 +299,10 @@ antlr::RefToken FMTLexer::nextToken()
 					mPERCENT(true);
 					theRetToken=_returnToken;
 				}
+				else if ((_tokenSet_0.member(LA(1))) && (true)) {
+					mNUMBER(true);
+					theRetToken=_returnToken;
+				}
 			else {
 				if (LA(1)==EOF_CHAR)
 				{
@@ -338,17 +352,17 @@ void FMTLexer::mSTRING(bool _createToken) {
 				match('\"' /* charlit */ );
 				text.erase(_saveIndex);
 			}
-			else if ((_tokenSet_0.member(LA(1)))) {
+			else if ((_tokenSet_1.member(LA(1)))) {
 				{
-				match(_tokenSet_0);
+				match(_tokenSet_1);
 				}
 			}
 			else {
-				goto _loop47;
+				goto _loop54;
 			}
 			
 		}
-		_loop47:;
+		_loop54:;
 		} // ( ... )*
 		_saveIndex = text.length();
 		match('\"' /* charlit */ );
@@ -368,17 +382,17 @@ void FMTLexer::mSTRING(bool _createToken) {
 				match('\'' /* charlit */ );
 				text.erase(_saveIndex);
 			}
-			else if ((_tokenSet_1.member(LA(1)))) {
+			else if ((_tokenSet_2.member(LA(1)))) {
 				{
-				match(_tokenSet_1);
+				match(_tokenSet_2);
 				}
 			}
 			else {
-				goto _loop50;
+				goto _loop57;
 			}
 			
 		}
-		_loop50:;
+		_loop57:;
 		} // ( ... )*
 		_saveIndex = text.length();
 		match('\'' /* charlit */ );
@@ -1020,10 +1034,10 @@ void FMTLexer::mCMOA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('M' /* charlit */ );
 	match('O' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1039,10 +1053,10 @@ void FMTLexer::mCMoA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('M' /* charlit */ );
 	match('o' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1058,10 +1072,119 @@ void FMTLexer::mCmoA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('m' /* charlit */ );
 	match('o' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
+	}
+	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
+void FMTLexer::mCMOI(bool _createToken) {
+	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
+	_ttype = CMOI;
+	std::string::size_type _saveIndex;
+	
+	{
+	match('C' /* charlit */ );
+	match('M' /* charlit */ );
+	match('O' /* charlit */ );
+	match('I' /* charlit */ );
+	}
+	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
+void FMTLexer::mCDI(bool _createToken) {
+	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
+	_ttype = CDI;
+	std::string::size_type _saveIndex;
+	
+	{
+	match('C' /* charlit */ );
+	match('D' /* charlit */ );
+	match('I' /* charlit */ );
+	}
+	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
+void FMTLexer::mCMI(bool _createToken) {
+	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
+	_ttype = CMI;
+	std::string::size_type _saveIndex;
+	
+	{
+	match('C' /* charlit */ );
+	match('M' /* charlit */ );
+	match('I' /* charlit */ );
+	}
+	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
+void FMTLexer::mCYI(bool _createToken) {
+	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
+	_ttype = CYI;
+	std::string::size_type _saveIndex;
+	
+	{
+	match('C' /* charlit */ );
+	match('Y' /* charlit */ );
+	match('I' /* charlit */ );
+	}
+	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
+void FMTLexer::mCSI(bool _createToken) {
+	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
+	_ttype = CSI;
+	std::string::size_type _saveIndex;
+	
+	{
+	match('C' /* charlit */ );
+	match('S' /* charlit */ );
+	match('I' /* charlit */ );
+	}
+	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
+void FMTLexer::mCSF(bool _createToken) {
+	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
+	_ttype = CSF;
+	std::string::size_type _saveIndex;
+	
+	{
+	match('C' /* charlit */ );
+	match('S' /* charlit */ );
+	match('F' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1077,9 +1200,9 @@ void FMTLexer::mCHI(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
-	match('h' /* charlit */ );
-	mI(false);
+	match('C' /* charlit */ );
+	match('H' /* charlit */ );
+	match('I' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1095,9 +1218,9 @@ void FMTLexer::mChI(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
-	match('H' /* charlit */ );
-	mI(false);
+	match('C' /* charlit */ );
+	match('h' /* charlit */ );
+	match('I' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1113,10 +1236,10 @@ void FMTLexer::mCDWA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('D' /* charlit */ );
 	match('W' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1132,10 +1255,10 @@ void FMTLexer::mCDwA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('D' /* charlit */ );
 	match('w' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1151,10 +1274,10 @@ void FMTLexer::mCdwA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('d' /* charlit */ );
 	match('w' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1170,10 +1293,10 @@ void FMTLexer::mCAPA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('A' /* charlit */ );
 	match('P' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1189,10 +1312,10 @@ void FMTLexer::mCApA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('A' /* charlit */ );
 	match('p' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1208,10 +1331,10 @@ void FMTLexer::mCapA(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{
-	mC(false);
+	match('C' /* charlit */ );
 	match('a' /* charlit */ );
 	match('p' /* charlit */ );
-	mA(false);
+	match('A' /* charlit */ );
 	}
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1249,6 +1372,40 @@ void FMTLexer::mDOT(bool _createToken) {
 	_saveIndex=0;
 }
 
+void FMTLexer::mPM(bool _createToken) {
+	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
+	_ttype = PM;
+	std::string::size_type _saveIndex;
+	
+	{
+	match('+' /* charlit */ );
+	match('-' /* charlit */ );
+	}
+	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
+void FMTLexer::mMP(bool _createToken) {
+	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
+	_ttype = MP;
+	std::string::size_type _saveIndex;
+	
+	{
+	match('-' /* charlit */ );
+	match('+' /* charlit */ );
+	}
+	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
 void FMTLexer::mW(bool _createToken) {
 	int _ttype; antlr::RefToken _token; std::string::size_type _begin = text.length();
 	_ttype = W;
@@ -1286,18 +1443,18 @@ void FMTLexer::mWHITESPACE(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{ // ( ... )+
-	int _cnt120=0;
+	int _cnt143=0;
 	for (;;) {
 		if ((LA(1) == 0x9 /* '\t' */  || LA(1) == 0x20 /* ' ' */ )) {
 			mW(false);
 		}
 		else {
-			if ( _cnt120>=1 ) { goto _loop120; } else {throw antlr::NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
+			if ( _cnt143>=1 ) { goto _loop143; } else {throw antlr::NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
 		}
 		
-		_cnt120++;
+		_cnt143++;
 	}
-	_loop120:;
+	_loop143:;
 	}  // ( ... )+
 	_ttype=antlr::Token::SKIP;
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
@@ -1314,18 +1471,18 @@ void FMTLexer::mDIGITS(bool _createToken) {
 	std::string::size_type _saveIndex;
 	
 	{ // ( ... )+
-	int _cnt123=0;
+	int _cnt146=0;
 	for (;;) {
 		if (((LA(1) >= 0x30 /* '0' */  && LA(1) <= 0x39 /* '9' */ ))) {
 			matchRange('0','9');
 		}
 		else {
-			if ( _cnt123>=1 ) { goto _loop123; } else {throw antlr::NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
+			if ( _cnt146>=1 ) { goto _loop146; } else {throw antlr::NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
 		}
 		
-		_cnt123++;
+		_cnt146++;
 	}
-	_loop123:;
+	_loop146:;
 	}  // ( ... )+
 	if ( _createToken && _token==antlr::nullToken && _ttype!=antlr::Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1359,6 +1516,7 @@ void FMTLexer::mNUMBER(bool _createToken) {
 	
 	SizeT n;
 	SizeT i = 0;
+	bool uPlus = false;
 	bool uMinus = false;
 	
 	
@@ -1372,6 +1530,14 @@ void FMTLexer::mNUMBER(bool _createToken) {
 		
 		break;
 	}
+	case 0x2b /* '+' */ :
+	{
+		match('+' /* charlit */ );
+		
+		uPlus = true;
+		
+		break;
+	}
 	case 0x30 /* '0' */ :
 	case 0x31 /* '1' */ :
 	case 0x32 /* '2' */ :
@@ -1395,6 +1561,7 @@ void FMTLexer::mNUMBER(bool _createToken) {
 	num=_returnToken;
 	
 	if( uMinus) num->setText( "-" + num->getText());
+	if( uPlus) num->setText( "+" + num->getText());
 	
 	{
 	if ((LA(1) == 0x48 /* 'H' */ )) {
@@ -1408,7 +1575,7 @@ void FMTLexer::mNUMBER(bool _createToken) {
 		match('H' /* charlit */ );
 		text.erase(_saveIndex);
 		{ // ( ... )+
-		int _cnt130=0;
+		int _cnt153=0;
 		for (;;) {
 			// init action gets executed even in guessing mode
 			if( i == n )
@@ -1419,12 +1586,12 @@ void FMTLexer::mNUMBER(bool _createToken) {
 				mCHAR(false);
 			}
 			else {
-				if ( _cnt130>=1 ) { goto _loop130; } else {throw antlr::NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
+				if ( _cnt153>=1 ) { goto _loop153; } else {throw antlr::NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
 			}
 			
-			_cnt130++;
+			_cnt153++;
 		}
-		_loop130:;
+		_loop153:;
 		}  // ( ... )+
 	}
 	else {
@@ -1440,16 +1607,19 @@ void FMTLexer::mNUMBER(bool _createToken) {
 }
 
 
-const unsigned long FMTLexer::_tokenSet_0_data_[] = { 4294967288UL, 4294967291UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
+const unsigned long FMTLexer::_tokenSet_0_data_[] = { 0UL, 67053568UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
+// + - 0 1 2 3 4 5 6 7 8 9 
+const antlr::BitSet FMTLexer::_tokenSet_0(_tokenSet_0_data_,10);
+const unsigned long FMTLexer::_tokenSet_1_data_[] = { 4294967288UL, 4294967291UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
 // 0x3 0x4 0x5 0x6 0x7 0x8 0x9 0xa 0xb 0xc 0xd 0xe 0xf 0x10 0x11 0x12 0x13 
 // 0x14 0x15 0x16 0x17 0x18 0x19 0x1a 0x1b 0x1c 0x1d 0x1e 0x1f   ! # $ 
 // % & \' ( ) * + , - . / 0 1 2 3 4 5 6 7 8 9 : ; < = > ? @ A B C D E F 
-// G H I J K L M 
-const antlr::BitSet FMTLexer::_tokenSet_0(_tokenSet_0_data_,16);
-const unsigned long FMTLexer::_tokenSet_1_data_[] = { 4294967288UL, 4294967167UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
+// G H I J K L M N O 
+const antlr::BitSet FMTLexer::_tokenSet_1(_tokenSet_1_data_,16);
+const unsigned long FMTLexer::_tokenSet_2_data_[] = { 4294967288UL, 4294967167UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
 // 0x3 0x4 0x5 0x6 0x7 0x8 0x9 0xa 0xb 0xc 0xd 0xe 0xf 0x10 0x11 0x12 0x13 
 // 0x14 0x15 0x16 0x17 0x18 0x19 0x1a 0x1b 0x1c 0x1d 0x1e 0x1f   ! \" # 
 // $ % & ( ) * + , - . / 0 1 2 3 4 5 6 7 8 9 : ; < = > ? @ A B C D E F 
-// G H I J K L M 
-const antlr::BitSet FMTLexer::_tokenSet_1(_tokenSet_1_data_,16);
+// G H I J K L M N O 
+const antlr::BitSet FMTLexer::_tokenSet_2(_tokenSet_2_data_,16);
 
