@@ -2,7 +2,7 @@
 #define INC_FMTLexer_hpp_
 
 #include <antlr/config.hpp>
-/* $ANTLR 2.7.7 (20120518): "format.g" -> "FMTLexer.hpp"$ */
+/* $ANTLR 2.7.7 (20130428): "format.g" -> "FMTLexer.hpp"$ */
 #include <antlr/CommonToken.hpp>
 #include <antlr/InputBuffer.hpp>
 #include <antlr/BitSet.hpp>
@@ -17,7 +17,7 @@
 
 #include "CFMTLexer.hpp"
 
-#include "antlr/TokenStreamSelector.hpp"
+#include <antlr/TokenStreamSelector.hpp>
 
 //using namespace antlr;
 
@@ -77,6 +77,12 @@ public:
 	public: void mCMOA(bool _createToken);
 	public: void mCMoA(bool _createToken);
 	public: void mCmoA(bool _createToken);
+	public: void mCMOI(bool _createToken);
+	public: void mCDI(bool _createToken);
+	public: void mCMI(bool _createToken);
+	public: void mCYI(bool _createToken);
+	public: void mCSI(bool _createToken);
+	public: void mCSF(bool _createToken);
 	public: void mCHI(bool _createToken);
 	public: void mChI(bool _createToken);
 	public: void mCDWA(bool _createToken);
@@ -87,6 +93,8 @@ public:
 	public: void mCapA(bool _createToken);
 	public: void mPERCENT(bool _createToken);
 	public: void mDOT(bool _createToken);
+	public: void mPM(bool _createToken);
+	public: void mMP(bool _createToken);
 	protected: void mW(bool _createToken);
 	public: void mWHITESPACE(bool _createToken);
 	protected: void mDIGITS(bool _createToken);
@@ -98,6 +106,8 @@ private:
 	static const antlr::BitSet _tokenSet_0;
 	static const unsigned long _tokenSet_1_data_[];
 	static const antlr::BitSet _tokenSet_1;
+	static const unsigned long _tokenSet_2_data_[];
+	static const antlr::BitSet _tokenSet_2;
 };
 
 #endif /*INC_FMTLexer_hpp_*/
