@@ -3,7 +3,7 @@
 
 #include <antlr/config.hpp>
 #include "FMTOutTokenTypes.hpp"
-/* $ANTLR 2.7.7 (20120518): "format.out.g" -> "FMTOut.hpp"$ */
+/* $ANTLR 2.7.7 (2006-11-01): "format.out.g" -> "FMTOut.hpp"$ */
 #include <antlr/TreeParser.hpp>
 
 
@@ -30,15 +30,22 @@ public:
 	valIx(0), termFlag(false), nonlFlag(false), nElements(0)
     {
         std::ostringstream* osLocal;
+#if (__cplusplus >= 201103L)
+std::unique_ptr<std::ostream> osLocalGuard;
+#else
         std::auto_ptr<std::ostream> osLocalGuard;
-        if( *os_ == std::cout)
+#endif
+        //if( *os_ == std::cout) // SA: this did not work with win32
+        if( os_->rdbuf() == std::cout.rdbuf())
             {
+                // e.g. print, 1, f='(A)'
                 osLocal = new std::ostringstream();
                 osLocalGuard.reset( osLocal);
                 os = osLocal;
             }
         else
             {
+                // e.g. print, string(1, f='(A)')
                 os = os_;
             }
 
@@ -48,7 +55,7 @@ public:
     
         GDLStream* j = lib::get_journal();
 
-        if( j != NULL && j->OStream() == (*os)) 
+        if( j != NULL && j->OStream().rdbuf() == os->rdbuf()) 
             (*os) << lib::JOURNALCOMMENT;
 
         format( fmt);
@@ -61,7 +68,7 @@ public:
         {
            (*os) << '\n';
             
-            if( j != NULL && j->OStream() == (*os)) 
+            if( j != NULL && j->OStream().rdbuf() == os->rdbuf()) 
                 (*os) << lib::JOURNALCOMMENT;
 
             format_reversion( reversionAnker);            
@@ -78,7 +85,7 @@ public:
             }
         (*os) << std::flush;
 
-        if( *os_ == std::cout)
+        if( os_->rdbuf() == std::cout.rdbuf()) // SA: see note above
             {
                 os = os_;
                 (*os) << osLocal->str();
@@ -148,7 +155,9 @@ public:
 	public: void format_reversion(RefFMTNode _t);
 	public: void f_csubcode(RefFMTNode _t);
 	public: void x(RefFMTNode _t);
-	public: void csubcode(RefFMTNode _t);
+	public: void csubcode(RefFMTNode _t,
+		SizeT r
+	);
 public:
 	antlr::RefAST getAST()
 	{
