@@ -2,7 +2,7 @@
 #define INC_GDLLexer_hpp_
 
 #include <antlr/config.hpp>
-/* $ANTLR 2.7.7 (20120518): "gdlc.g" -> "GDLLexer.hpp"$ */
+/* $ANTLR 2.7.7 (2006-11-01): "gdlc.g" -> "GDLLexer.hpp"$ */
 #include <antlr/CommonToken.hpp>
 #include <antlr/InputBuffer.hpp>
 #include <antlr/BitSet.hpp>
@@ -18,12 +18,12 @@
 #include "objects.hpp"
 #include "initsysvar.hpp"
 
-#include "antlr/TokenStreamSelector.hpp"
-
-#include "antlr/SemanticException.hpp"
-#include "antlr/NoViableAltForCharException.hpp"
-#include "antlr/TokenStreamIOException.hpp"
-#include "antlr/CharInputBuffer.hpp"
+#include <antlr/TokenStreamSelector.hpp>
+	 
+#include <antlr/SemanticException.hpp>
+#include <antlr/NoViableAltForCharException.hpp>
+#include <antlr/TokenStreamIOException.hpp>
+#include <antlr/CharInputBuffer.hpp>
 
 //#include "dinterpreter.hpp"
 
@@ -35,7 +35,11 @@ class CUSTOM_API GDLLexer : public antlr::CharScanner, public GDLTokenTypes
 
   // Stuff for include files (@filename)
   private:
-    std::auto_ptr<std::ifstream>    inputFile; // stores ifsteam* and deletes 
+#if (__cplusplus >= 201103L)
+    std::unique_ptr<std::ifstream>    inputFile; // stores ifstream* and deletes 
+#else
+     std::auto_ptr<std::ifstream>    inputFile; // stores ifstream* and deletes 
+#endif
                                      // it when it is deleted itself
   
     antlr::TokenStreamSelector*     selector; 
