@@ -2,7 +2,7 @@
  * Project led by Terence Parr at http://www.jGuru.com
  * Software rights: http://www.antlr.org/license.html
  *
- * $Id: LLkParser.cpp,v 1.6 2012/11/13 14:30:14 m_schellens Exp $
+ * $Id: LLkParser.cpp,v 1.8 2013/05/07 13:13:52 m_schellens Exp $
  */
 
 #include "antlr/LLkParser.hpp"
@@ -40,7 +40,8 @@ LLkParser::LLkParser(TokenStream& lexer, int k_)
 
 void LLkParser::trace(const char* ee, const char* rname)
 {
-  //if(inputState->guessing>0) return;
+  // only show non-guessing (production) calls
+  if(inputState->guessing>0) return;
 	
   traceIndent();
   
