@@ -511,9 +511,12 @@ SizeT BaseGDL::OFmtI( std::ostream* os, SizeT offs, SizeT num, int width,
 		      int minN, char fill, BaseGDL::IOMode oM)
 {throw GDLException("BaseGDL::OFmtI(...) called.");}
 
+SizeT BaseGDL::IFmtCal( std::istream* is, SizeT offs, SizeT r, int width, BaseGDL::Cal_IOMode cMode)
+{throw GDLException("BaseGDL::IFmtCal(...) called.");}
+
 SizeT BaseGDL::OFmtCal( std::ostream* os, SizeT offs, SizeT num, int width, 
-		      int minN, char fill, BaseGDL::Cal_IOMode oM)
-{throw GDLException("BaseGDL::OFmtC(...) called.");}
+			int minN, char *f, BaseGDL::Cal_IOMode oM)
+{throw GDLException("BaseGDL::OFmtCal(...) called.");}
 
 SizeT BaseGDL::IFmtA( std::istream* is, SizeT offset, SizeT num, int width)
 {throw GDLException("BaseGDL::IFmtA(...) called.");}
