@@ -344,7 +344,17 @@ public:
     // Linux case for example
     static struct mallinfo mi;
     mi = mallinfo();
-    Current = mi.uordblks;
+//         printf("Total non-mmapped bytes (arena):       %d\n", mi.arena);
+//           printf("# of free chunks (ordblks):            %d\n", mi.ordblks);
+//           printf("# of free fastbin blocks (smblks):     %d\n", mi.smblks);
+//           printf("# of mapped regions (hblks):           %d\n", mi.hblks);
+//           printf("Bytes in mapped regions (hblkhd):      %d\n", mi.hblkhd);
+//           printf("Max. total allocated space (usmblks):  %d\n", mi.usmblks);
+//           printf("Free bytes held in fastbins (fsmblks): %d\n", mi.fsmblks);
+//           printf("Total allocated space (uordblks):      %d\n", mi.uordblks);
+//           printf("Total free space (fordblks):           %d\n", mi.fordblks);
+//           printf("Topmost releasable block (keepcost):   %d\n", mi.keepcost);
+      Current = mi.arena+mi.hblkhd; //was mi.uordblks;
 #elif defined(HAVE_MALLOC_ZONE_STATISTICS) && defined(HAVE_MALLOC_MALLOC_H)
     // Mac OS X case for example
     static malloc_statistics_t stats;
@@ -413,6 +423,8 @@ public:
   };
 
   enum Cal_IOMode {
+      WRITE=-2,
+      COMPUTE=-1,
       DEFAULT=0,
       CMOA,
       CMoA,
@@ -430,7 +442,8 @@ public:
       CdwA,
       CAPA,
       CApA,
-      CapA
+      CapA,
+      STRING
   };
   // FIRST VIRTUAL FUNCTION'S GDL_OBJ FILE CONTAINS ALSO THE VTABLE
   // therefore it must be defined non-inline (g++)
@@ -701,11 +714,12 @@ public:
   virtual SizeT OFmtI( std::ostream* os, SizeT offs, SizeT num, int width, 
 			int minN, char fill, BaseGDL::IOMode oM = DEC);
   virtual SizeT OFmtCal( std::ostream* os, SizeT offs, SizeT num, int width, 
-			int minN, char fill, BaseGDL::Cal_IOMode oM = DEFAULT);
+			 int minN, char *fill, BaseGDL::Cal_IOMode oM = DEFAULT);
   virtual SizeT IFmtA( std::istream* is, SizeT offset, SizeT num, int width);
   virtual SizeT IFmtF( std::istream* is, SizeT offs, SizeT num, int width);
   virtual SizeT IFmtI( std::istream* is, SizeT offs, SizeT num, int width, 
 			BaseGDL::IOMode oM = DEC);
+  virtual SizeT IFmtCal( std::istream* is, SizeT offs, SizeT r, int width, BaseGDL::Cal_IOMode cMode);
 
 #if defined(USE_PYTHON) || defined(PYTHON_MODULE)
 
