@@ -64,7 +64,7 @@ namespace lib {
 
   BaseGDL* n_elements( EnvT* e);
 
-  BaseGDL* execute( EnvT* e);
+  BaseGDL* execute_fun( EnvT* e);
 
   BaseGDL* assoc( EnvT* e);
   
@@ -115,6 +115,8 @@ namespace lib {
 
   BaseGDL* shift_fun( EnvT* e);
 
+  BaseGDL* ishft_fun( EnvT* e);
+
   BaseGDL* arg_present( EnvT* e);
 
   BaseGDL* eof_fun( EnvT* e);
@@ -161,8 +163,9 @@ namespace lib {
   BaseGDL* idl_base64(EnvT* e);
   BaseGDL* get_drive_list(EnvT* e);
 
+  BaseGDL* scope_level( EnvT* e);
+  BaseGDL* scope_traceback( EnvT* e);
   BaseGDL* scope_varfetch_value( EnvT* e); // regular library function
-
   BaseGDL** scope_varfetch_reference( EnvT* e); // special version for LEval()
 
 } // namespace
