@@ -30,8 +30,8 @@
 //#define GDL_DEBUG
 #undef GDL_DEBUG
 
-#ifdef _MSC_VER
-#include "gtdhelper.hpp"
+#if defined (_WIN32) && !defined(__CYGWIN__)
+#include "gtdhelper.hpp" // just a workaround, using QueryPerformanceCounter is better
 #endif
 
 namespace lib {
@@ -41,7 +41,6 @@ namespace lib {
   void (*dynPro[MAXNDLL/2])( EnvT* e);
   BaseGDL*(*dynFun[MAXNDLL/2])( EnvT* e);
 
-
   void point_lun( EnvT* e) 
   { 
     e->NParam( 1);//, "POINT_LUN");
@@ -77,7 +76,7 @@ namespace lib {
   void linkimage( EnvT* e) 
   {
 
-#ifdef _MSC_VER
+#if defined(_WIN32) && !defined(__CYGWIN__)
     HMODULE module[MAXNDLL];
 #else
     const char *error;
@@ -117,14 +116,10 @@ namespace lib {
     e->AssureScalarPar<DStringGDL>( 3, entryName);
 
     /* Load dynamically loaded library */
-#ifdef _MSC_VER
-    #ifdef _UNICODE
-    TCHAR u_shrdimgName[255];
+#if defined(_WIN32) && !defined(__CYGWIN__)
+    WCHAR u_shrdimgName[255];
     MultiByteToWideChar(CP_ACP, 0, shrdimgName.c_str(), shrdimgName.length(), u_shrdimgName, 255);
-    module[count] = LoadLibrary(u_shrdimgName);
-    #else
-    module[count] = LoadLibrary(shrdimgName.c_str());
-    #endif
+    module[count] = LoadLibraryW(u_shrdimgName);
 
     if (!module[count]) {
       fprintf(stderr, "Couldn't open %s\n", 
@@ -144,7 +139,7 @@ namespace lib {
     dlerror();
 #endif
 
-#ifdef _MSC_VER
+#if defined(_WIN32) && !defined(__CYGWIN__)
     if (funcType == 0) {
       (void* &) dynPro[count_pro] = 
 	(void *) GetProcAddress(module[count], entryName.c_str());
@@ -183,7 +178,7 @@ namespace lib {
     }
   }
 
-  void wait( EnvT* e) 
+  void wait_pro( EnvT* e) 
   { 
     e->NParam( 1);//, "WAIT");
 
@@ -192,8 +187,27 @@ namespace lib {
 
     if( waittime < 0)
       throw GDLException( e->CallingNode(), 
-  			  "WAIT:  Argument must be non-negative"
+  			  "WAIT: Argument must be non-negative"
 			  +e->GetParString( 0));
+
+#ifdef _WIN32
+	LARGE_INTEGER Frequency;
+	LARGE_INTEGER BeginTime;
+	LARGE_INTEGER Endtime;
+	LARGE_INTEGER elapsed;
+	LARGE_INTEGER waittime_us;
+	waittime_us.QuadPart = waittime * 1e6;
+
+	QueryPerformanceFrequency(&Frequency);
+	QueryPerformanceCounter(&BeginTime);
+
+	while (1) {
+		QueryPerformanceCounter(&Endtime);
+		elapsed.QuadPart = (Endtime.QuadPart - BeginTime.QuadPart)/(Frequency.QuadPart/1000000);
+		if (elapsed.QuadPart >= waittime_us.QuadPart) break;
+		else if (elapsed.QuadPart > 100) Sleep(80);
+	}
+#else
     int old_version=0;
 
     if (waittime <= 0.005) old_version=1;
@@ -202,37 +216,33 @@ namespace lib {
     // this version is OK and very accurate for small durations
     // but used 100% of one CPU :((
     if (old_version == 1) {
-
-      struct timeval tval;
+	  struct timeval tval;
       struct timezone tzone;
       
       // derivated from the current version of SYSTIME()
-      
       gettimeofday(&tval,&tzone);
       double t_start = tval.tv_sec+tval.tv_usec/1e+6; // time in UTC seconds
       double t_current=0.0;
       
-      double diff=0.0;
+	  double diff = 0.0;
       while (diff < waittime ) {      
-	gettimeofday(&tval,&tzone);
+
+		  gettimeofday(&tval,&tzone);
 	t_current= tval.tv_sec+tval.tv_usec/1e+6;
 	diff=t_current - t_start;
-      }
-    }
+	  }
+	}
 
     // AC 2010-09-16 this version should used much less CPU !
     if (old_version == 0) {
       //cout << floor(waittime) << " " <<  waittime-floor(waittime) << endl;
-#ifdef _MSC_VER
-      Sleep(floor(waittime*1e3));
-#else
       struct timespec tv;
       tv.tv_sec = floor(waittime);
       tv.tv_nsec = (waittime-floor(waittime))*1e9;
       int retval;
       retval=nanosleep(&tv,NULL);
-#endif
     }
+#endif
   }
 
 //   void kwtest( EnvT* e)
@@ -299,17 +309,34 @@ namespace lib {
     if (KWReturnType != NULL && (*KWReturnType)[0] != 0) {
 	myReturnType = (DType) (*KWReturnType)[0];
     }
-
-    string TypeKW[16] = {
+    
+    static int B_VALUEIx = e->KeywordIx("B_VALUE");
+    static int I_VALUEIx = e->KeywordIx("I_VALUE");
+    static int L_VALUEIx = e->KeywordIx("L_VALUE");
+    static int F_VALUEIx = e->KeywordIx("F_VALUE");
+    static int D_VALUEIx = e->KeywordIx("D_VALUE");
+	static int S_VALUEIx = e->KeywordIx("S_VALUE");
+	static int UI_VALUEIx = e->KeywordIx("UI_VALUE");
+    static int UL_VALUEIx = e->KeywordIx("UL_VALUE");
+    static int L64_VALUEIx = e->KeywordIx("L64_VALUE");
+    static int UL64_VALUEIx = e->KeywordIx("UL64_VALUE");
+      
+    static string TypeKW[16] = {
 	"", "B_VALUE", "I_VALUE", "L_VALUE", "F_VALUE", "D_VALUE",
 	"", "S_VALUE", "", "", "", "",
 	"UI_VALUE", "UL_VALUE", "L64_VALUE", "UL64_VALUE"
     };
-
+    static int TypeKWIx[16] = {
+	-1, B_VALUEIx, I_VALUEIx, L_VALUEIx, F_VALUEIx, D_VALUEIx,
+	-1, S_VALUEIx, -1 , -1, -1, -1,
+	UI_VALUEIx, UL_VALUEIx, L64_VALUEIx, UL64_VALUEIx
+    };
     for (int i=0; i<16; i++) {
-	if (strcmp(TypeKW[i].c_str(), "") == 0) {continue;}
-	int kwIxType = e->KeywordIx(TypeKW[i]);
-	if (e->KeywordSet(kwIxType)) {
+//	if (strcmp(TypeKW[i].c_str(), "") == 0) {continue;}
+	if (TypeKWIx[i] == -1) {continue;}
+//	int kwIxType = e->KeywordIx(TypeKW[i]); //should be made static to speed up
+//	if (e->KeywordSet(kwIxType)) {
+	if (e->KeywordSet(TypeKWIx[i])) {
 	    if (myReturnType != GDL_UNDEF) {
 		e->Throw("Multiple requests of return type not allowed: "
 			 + TypeKW[i] );
@@ -432,20 +459,27 @@ namespace lib {
 
     // Load shared object, call function
 
-#ifdef _MSC_VER
-    HMODULE handle =  LoadLibrary((LPCSTR)image.c_str());
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	LPWSTR wchr = new WCHAR[image.size() + 1];
+    wchr[image.size()] = 0;
+    std::copy(image.begin(), image.end(), wchr);
+    HMODULE handle = LoadLibraryW(wchr);
+    delete(wchr);
 #else
+    // note following code line is correct for gcc and linux.
+    // in case of trouble on some architecture, find the correct option and make an #ifdef.
+    // Do *not* modify the following line. GD.
     void* handle =  dlopen(image.c_str(),  RTLD_NOW | RTLD_GLOBAL);
 #endif
     if (handle == NULL) {
-#ifndef _MSC_VER
+#if !defined(_WIN32) || defined(__CYGWIN__)
 	cout << dlerror() << endl;
 #endif
 	e->Throw("Error opening shared object: " + image);
     }
 
-#ifdef _MSC_VER
-    void* func = GetProcAddress(handle,entry.c_str());
+#if defined(_WIN32) && !defined(__CYGWIN__)
+    void* func = (void *)GetProcAddress(handle,entry.c_str());
 #else
     void* func = dlsym(handle,entry.c_str());
 #endif
@@ -492,7 +526,7 @@ namespace lib {
     }
 
     if (flagUnload) {
-#ifdef _MSC_VER
+#if defined(_WIN32) && !defined(__CYGWIN__)
 	while (! FreeLibrary(handle) ) {}
 #else
 	while (! dlclose(handle) ) {}
