@@ -61,7 +61,7 @@ template <>
 inline float string2real_helper<float>(const char* cStart, char** cEnd)
 {
 #ifdef _MSC_VER
-  return strtod(cStart, cEnd);
+  return StrToD(cStart, cEnd);
 #else
   return strtof(cStart, cEnd);
 #endif
@@ -70,7 +70,7 @@ inline float string2real_helper<float>(const char* cStart, char** cEnd)
 template <>
 inline double string2real_helper<double>(const char* cStart, char** cEnd)
 {
-  return strtod(cStart, cEnd);
+  return StrToD(cStart, cEnd);
 }
 
 template <typename real_t>
@@ -1247,7 +1247,7 @@ TRACE_CONVERT2
       {
       	Data_<SpDByte>* dest=new Data_<SpDByte>( dim, BaseGDL::NOZERO);
 #ifdef SINGLE_ELEMENT_OPTIMIZATION
-	if( nEl == 1) \
+	if( nEl == 1) 
 	{
 		const SizeT i = 0;
 		(*dest)[i]=Real2DByte<double>((*this)[i]);
@@ -1270,7 +1270,7 @@ TRACEOMP( __FILE__, __LINE__)
       {
       	Data_<SpDInt>* dest=new Data_<SpDInt>( dim, BaseGDL::NOZERO);
 #ifdef SINGLE_ELEMENT_OPTIMIZATION
-	if( nEl == 1) \
+	if( nEl == 1) 
 	{
 		const SizeT i = 0;
 		(*dest)[i]=Real2Int<DInt,double>((*this)[i]);
@@ -1292,7 +1292,7 @@ TRACEOMP( __FILE__, __LINE__)
       {
       	Data_<SpDUInt>* dest=new Data_<SpDUInt>( dim, BaseGDL::NOZERO);
 #ifdef SINGLE_ELEMENT_OPTIMIZATION
-	if( nEl == 1) \
+	if( nEl == 1) 
 	{
 		const SizeT i = 0;
 		(*dest)[i]=static_cast< DUInt>((*this)[i]);//Real2Int<DUInt,double>((*this)[i]);
@@ -1313,7 +1313,7 @@ TRACEOMP( __FILE__, __LINE__)
       {
       	Data_<SpDLong>* dest=new Data_<SpDLong>( dim, BaseGDL::NOZERO);
 #ifdef SINGLE_ELEMENT_OPTIMIZATION
-	if( nEl == 1) \
+	if( nEl == 1) 
 	{
 		const SizeT i = 0;
 		(*dest)[i]=Real2Int<DLong,double>((*this)[i]);
@@ -1335,7 +1335,7 @@ TRACEOMP( __FILE__, __LINE__)
       {
       	Data_<SpDULong>* dest=new Data_<SpDULong>( dim, BaseGDL::NOZERO);
 #ifdef SINGLE_ELEMENT_OPTIMIZATION
-	if( nEl == 1) \
+	if( nEl == 1) 
 	{
 		const SizeT i = 0;
 		(*dest)[i]=static_cast< DULong>((*this)[i]);//Real2Int<DULong,double>((*this)[i]);
@@ -1356,7 +1356,7 @@ TRACEOMP( __FILE__, __LINE__)
       {
       	Data_<SpDLong64>* dest=new Data_<SpDLong64>( dim, BaseGDL::NOZERO);
 #ifdef SINGLE_ELEMENT_OPTIMIZATION
-	if( nEl == 1) \
+	if( nEl == 1) 
 	{
 		const SizeT i = 0;
 		(*dest)[i]=Real2Int<DLong64,double>((*this)[i]);
@@ -1378,7 +1378,7 @@ TRACEOMP( __FILE__, __LINE__)
       {
       	Data_<SpDULong64>* dest=new Data_<SpDULong64>( dim, BaseGDL::NOZERO);
 #ifdef SINGLE_ELEMENT_OPTIMIZATION
-	if( nEl == 1) \
+	if( nEl == 1) 
 	{
 		const SizeT i = 0;
 		(*dest)[i]=static_cast< DULong64>((*this)[i]);//Real2Int<DULong64,double>((*this)[i]);
@@ -1517,10 +1517,10 @@ TRACEOMP( __FILE__, __LINE__)
     case GDL_INT:
       {
       	Data_<SpDInt>* dest=new Data_<SpDInt>( dim, BaseGDL::NOZERO);
-TRACEOMP( __FILE__, __LINE__)
-#pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl)) shared( errorFlag, mode)
-{
-#pragma omp for
+// TRACEOMP( __FILE__, __LINE__)
+// #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl)) shared( errorFlag, mode)
+// {
+// #pragma omp for
       	for( OMPInt i=0; i < nEl; ++i)
       	  {
       	    const char* cStart=(*this)[i].c_str();
@@ -1528,12 +1528,12 @@ TRACEOMP( __FILE__, __LINE__)
       	    (*dest)[i]=strtol(cStart,&cEnd,10);
       	    if( cEnd == cStart && (*this)[i] != "")
       	      {
-StringConversionError( errorFlag, mode, "Type conversion error: "
+		StringConversionError( errorFlag, mode, "Type conversion error: "
 				       "Unable to convert given STRING: '"+
 				       (*this)[i]+"' to INT.");
       	      }
 	  }
-}
+// }
 	if( errorFlag)
 	{
 		delete dest;
