@@ -116,6 +116,7 @@ void Data_<SpDPtr>::InsAt( Data_* srcIn, ArrayIndexListT* ixList, SizeT offset)
   nDim--;
 
   dimension srcDim=srcIn->Dim();
+  srcDim.Purge(); // newly inserted: variant of ticket #675
   SizeT len=srcDim[0]; // length of one segment to copy (one line of srcIn)
 
   //  SizeT nDim   =RankIx(ixDim.Rank());
@@ -231,6 +232,7 @@ void Data_<SpDObj>::InsAt( Data_* srcIn, ArrayIndexListT* ixList, SizeT offset)
   nDim--;
 
   dimension srcDim=srcIn->Dim();
+  srcDim.Purge(); // newly inserted: variant of ticket #675
   SizeT len=srcDim[0]; // length of one segment to copy (one line of srcIn)
 
   //  SizeT nDim   =RankIx(ixDim.Rank());
@@ -1342,6 +1344,7 @@ BaseGDL* Data_<SpDObj>::NewIx( SizeT ix)
     return result->Dup();
   }
   
+  GDLInterpreter::IncRefObj( s);
   return new Data_( s);
 }
 
