@@ -21,7 +21,7 @@
 #ifdef _MSC_VER
 #include <io.h> // isatty, windows
 #else
-#include <unistd.h> // isatty
+#include <unistd.h> // isatty, usleep
 #endif
 
 //#include <wordexp.h>
@@ -50,9 +50,17 @@
 #include "print_tree.hpp"
 #endif
 
+#if (__cplusplus >= 201103L || _MSC_VER >= 1800) && !defined(__MINGW32__)
+#   include <thread> // C++11
+#   define HAVE_CXX11THREAD
+#else
+#   include <pthread.h>
+#endif
+
 using namespace std;
 using namespace antlr;
 
+string inputstr;
 bool historyIntialized = false;
 
 // instantiation of static data
@@ -348,7 +356,7 @@ bool GDLInterpreter::SearchCompilePro(const string& pro, bool searchForPro)
   if( !found) return false;
   
   // file already opened?
-  for( StrArr::iterator i=openFiles.begin(); i != openFiles.end(); i++)
+  for( StrArr::iterator i=openFiles.begin(); i != openFiles.end(); ++i)
     {
       if( proFile == *i) return false;
     }
@@ -719,7 +727,7 @@ DInterpreter::CommandCode DInterpreter::CmdCompile( const string& command)
 DInterpreter::CommandCode DInterpreter::CmdRun( const string& command)
 {
   string cmdstr = command;
-  int sppos = cmdstr.find(" ",0);
+  size_t sppos = cmdstr.find(" ",0);
   if (sppos == string::npos) 
     {
       cout << "Interactive RUN not implemented yet." << endl;
@@ -733,7 +741,12 @@ DInterpreter::CommandCode DInterpreter::CmdRun( const string& command)
   while (pos < command.length()) 
     {
       sppos = command.find(" ",pos);
-      if (sppos == string::npos) sppos = command.length();
+      size_t spposComma = command.find(",",pos);
+      if (sppos == string::npos && spposComma == string::npos) 
+	sppos = command.length();
+      else if (sppos == string::npos)
+	sppos = spposComma;
+	
 
       // Found a file
       if ((sppos - pos) > 0) 
@@ -778,7 +791,7 @@ DInterpreter::CommandCode DInterpreter::CmdRun( const string& command)
 
   // actual run is perfomed in InterpreterLoop()
   RetAll( RetAllException::RUN); // throws (always)
-  //  return CC_OK;
+  return CC_OK; //avoid warnings
 }
 
 // execute GDL command (.run, .step, ...)
@@ -842,32 +855,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteCommand(const string& command)
       cout << "RNEW not implemented yet." << endl;
       return CC_OK;
     }
-  if( cmd( "SIZE"))
-    {
-      cout << "SIZE not implemented yet." << endl;
-      return CC_OK;
-    }
-  if( cmd( "SKIP"))
-    {
-      DLong sCount;
-      if( args == "")
-      {
-	  sCount = 1;
-      }
-      else
-      {
-	const char* cStart=args.c_str();
-	char* cEnd;
-	sCount = strtol(cStart,&cEnd,10);
-	if( cEnd == cStart)
-	{
-	  cout << "Type conversion error: Unable to convert given STRING: '"+args+"' to LONG." << endl;
-	  return CC_OK;
-	}
-      }
-      stepCount = sCount;
-      return CC_SKIP;
-    }
+  // GD:Here to have ".s" giving ".step"
   if( cmd( "STEP"))
     {
       DLong sCount;
@@ -890,11 +878,38 @@ DInterpreter::CommandCode DInterpreter::ExecuteCommand(const string& command)
 	debugMode = DEBUG_STEP;
 	return CC_STEP;
     }
+
+  if( cmd( "SKIP"))
+    {
+      DLong sCount;
+      if( args == "")
+      {
+	  sCount = 1;
+      }
+      else
+      {
+	const char* cStart=args.c_str();
+	char* cEnd;
+	sCount = strtol(cStart,&cEnd,10);
+	if( cEnd == cStart)
+	{
+	  cout << "Type conversion error: Unable to convert given STRING: '"+args+"' to LONG." << endl;
+	  return CC_OK;
+	}
+      }
+      stepCount = sCount;
+      return CC_SKIP;
+    }
   if( cmd( "STEPOVER"))
     {
       cout << "STEPOVER not implemented yet." << endl;
       return CC_OK;
     }
+  if( cmd( "SIZE"))
+    {
+      cout << "SIZE not implemented yet." << endl;
+      return CC_OK;
+    }
   if( cmd( "TRACE"))
     {
       cout << "TRACE not implemented yet." << endl;
@@ -929,7 +944,9 @@ void DInterpreter::ExecuteShellCommand(const string& command)
 string GetLine( istream* in)
 {
   string line = "";
-  while( line == "" && in->good()) 
+  while( in->good() && 
+    (  line == "" 
+    || line[0] == ';' )) // skip also comment lines (bug #663) 
     {
       getline( *in, line);
       StrTrim(line);
@@ -949,7 +966,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
 {
   string line = (in != NULL) ? ::GetLine(in) : GetLine();
 
-  //  cout << "ExecuteLine: " << line << endl;
+  // cout << "ExecuteLine: " << line << endl;
 
   string firstChar = line.substr(0,1);
 
@@ -1247,30 +1264,68 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
   return CC_OK;
 }
 
+#ifdef HAVE_CXX11THREAD
+void inputThread() {
+#else
+void *inputThread(void*) {
+#endif
+    while (1) {
+      // patch by Ole, 2017-01-06
+      //char ch = getchar(); if (ch==EOF) return NULL;
+      char ch = getchar();
+      if (ch==EOF) {
+#ifdef HAVE_CXX11THREAD
+	return;
+#else
+	return NULL;
+#endif
+      }        
+      inputstr += ch;
+      if (ch == '\n')
+	break;
+    }
+#ifndef HAVE_CXX11THREAD
+    return NULL;
+#endif
+}
+
 // if readline is not available or !EDIT_INPUT set to zero
 char* DInterpreter::NoReadline( const string& prompt)
 {
   if (isatty(0)) cout << prompt << flush;
-
-  ostringstream ostr;
-  char ch;
   if( feof(stdin)) return NULL;
-  for(;;)
+
+#ifdef HAVE_CXX11THREAD
+  thread th(inputThread);
+#else
+  pthread_t th;
+  pthread_create(&th, NULL, inputThread, NULL);
+#endif
+
+  for (;;)
     {
-      GDLEventHandler(); 
-
-      ch = getchar();
-
-      if( ch == '\n') break;
-      if( feof(stdin)) return NULL;
-      ostr << ch;
+        GDLEventHandler();
+        if (inputstr.size() && inputstr[inputstr.size() - 1] == '\n') break;
+        if (feof(stdin)) return NULL;
+#ifdef WIN32
+        Sleep(10);
+#else
+        usleep(10);
+#endif
     }
-  ostr << ends;
-  string str = ostr.str();
+  inputstr = inputstr.substr(0, inputstr.size() - 1); // removes '\n'
+  //if (inputstr[inputstr.size() - 1] == '\r')
+  //    inputstr = inputstr.substr(0, inputstr.size() - 1); // removes '\r' too, if exists
+  char *result = (char*)malloc((inputstr.length() + 1) * sizeof(char));
+  strcpy(result, inputstr.c_str()); // copies including terminating '\0'
+  inputstr.clear();
 
-  char *result = (char*) malloc((str.length()+1) * sizeof(char));
+#ifdef HAVE_CXX11THREAD
+  th.join();
+#else
+  pthread_join(th, NULL);
+#endif
 
-  strcpy(result,str.c_str()); // copies including terminating '\0'
   return result;
 }
 
@@ -1289,7 +1344,7 @@ string DInterpreter::GetLine()
 {
   clog << flush; cout << flush;
 
-#ifdef HAVE_LIBREADLINE
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
   int edit_input = SysVar::Edit_Input() && isatty(0);
 #endif
 
@@ -1302,7 +1357,7 @@ string DInterpreter::GetLine()
 
     lineEdit = true;
 
-#ifdef HAVE_LIBREADLINE
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
     
     if( edit_input != 0)
       cline = readline(const_cast<char*>(actualPrompt.c_str()));
@@ -1335,9 +1390,10 @@ string DInterpreter::GetLine()
 #endif
   
     StrTrim(line);
-  } while( line == "");
+  } while( line == "" 
+	|| line[0] == ';'); // skip also comment lines (bug #663)
   
-#ifdef HAVE_LIBREADLINE
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
   // SA: commented out to comply with IDL behaviour- allowing to 
   //     set the history-file length only in the startup file
   //if( edit_input > 20)
@@ -1534,112 +1590,91 @@ void DInterpreter::RunDelTree()
 
 // reads user input and executes it
 // the main loop
-RetCode DInterpreter::InterpreterLoop( const string& startup,
-  vector<string>& batch_files, const std::string& statement)
-{
+
+RetCode DInterpreter::InterpreterLoop(const string& startup,
+  vector<string>& batch_files, const std::string& statement) {
   // process startup file
-  if( startup != "")
-    {
-      ifstream in(startup.c_str());
+  if (startup != "") {
+    ifstream in(startup.c_str());
 
-      if( in.fail())
-	Warning( "Error opening startup file: "+startup);
+    if (in.fail())
+      Warning("Error opening startup file: " + startup);
 
-      ValueGuard<bool> guard( interruptEnable);
-      interruptEnable = false;
+    ValueGuard<bool> guard(interruptEnable);
+    interruptEnable = false;
 
-      bool runCmd = false;
-      try
-	{
-	  while( in.good())
-	    {
+    bool runCmd = false;
+    try {
+      while (in.good()) {
 #if defined (_MSC_VER) && _MSC_VER < 1800
-		  _clearfp();
+        _clearfp();
 #else
-	      feclearexcept(FE_ALL_EXCEPT);
+        feclearexcept(FE_ALL_EXCEPT);
 #endif
 
-	      try
-		{
-		  if( runCmd)
-		    {
-		      runCmd = false;
-		      RunDelTree();
-		    }
-		  else
-		    {		  
-		      DInterpreter::CommandCode ret=ExecuteLine( &in);
-	      
-		      if( debugMode != DEBUG_CLEAR)
-			{
-			  debugMode = DEBUG_CLEAR;
-			  Warning( "Prematurely closing batch file: "+startup);
-			  break;
-			}
-		    }
-		}
-	      catch( RetAllException& retAllEx)
-		{
-		  runCmd = (retAllEx.Code() == RetAllException::RUN);
-		  if( !runCmd) throw;
-		}
-	      catch( exception& e)
-		{
-		  cerr << startup << ": Exception: " << e.what() << endl;
-		}
-	      catch (...)
-		{	
-		  cerr << startup << ": Unhandled Error." << endl;
-		}
-	    } // while
-	}
-      catch( RetAllException& retAllEx)
-	{
-	}
-    } // if( startup...
+        try {
+          if (runCmd) {
+            runCmd = false;
+            RunDelTree();
+          } else {
+            DInterpreter::CommandCode ret = ExecuteLine(&in);
+
+            if (debugMode != DEBUG_CLEAR) {
+              debugMode = DEBUG_CLEAR;
+              Warning("Prematurely closing batch file: " + startup);
+              break;
+            }
+          }
+        } catch (RetAllException& retAllEx) {
+          runCmd = (retAllEx.Code() == RetAllException::RUN);
+          if (!runCmd) throw;
+        } catch (exception& e) {
+          cerr << startup << ": Exception: " << e.what() << endl;
+        } catch (...) {
+          cerr << startup << ": Unhandled Error." << endl;
+        }
+      } // while
+    } catch (RetAllException& retAllEx) {
+    }
+  } // if( startup...
 
 #ifdef USE_MPI
   int myrank = 0;
   int tag = 0;
   char mpi_procedure[256];
   MPI_Status status;
-  MPI_Comm_rank( MPI_COMM_WORLD, &myrank);
-  int size; 
+  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
+  int size;
   MPI_Comm_size(MPI_COMM_WORLD, &size);
   if (size > 1) {
     MPI_Recv(mpi_procedure, 256, MPI_CHAR, 0, tag, MPI_COMM_WORLD, &status);
 
     istringstream istr(StrUpCase(mpi_procedure) + "\n");
-    DInterpreter::CommandCode ret=ExecuteLine( &istr);
+    DInterpreter::CommandCode ret = ExecuteLine(&istr);
 
     MPI_Finalize();
-    exit( EXIT_SUCCESS);
+    exit(EXIT_SUCCESS);
   }
 #endif
 
-  if (statement.length() > 0) 
-  {
+  if (statement.length() > 0) {
     // execute single statement and exit (a new-line is added to statement in gdl.cpp)
     // (e.g. $ gdl -e "print, 'hello world'")
     istringstream iss(statement, ios_base::out);
     try {
-		ExecuteLine(&iss);
+      ExecuteLine(&iss);
+    } catch (RetAllException& retAllEx) {
     }
-      catch( RetAllException& retAllEx)
-	{
-	}
     return RC_OK;
-  }
-  else
-  {
+  } else {
     // execute batch files (e.g. $ gdl script.pro)
     // before entering the interactive mode
-    for (vector<string>::iterator it = batch_files.begin(); it < batch_files.end(); it++) 
+    for (vector<string>::iterator it = batch_files.begin(); it < batch_files.end(); ++it)
       ExecuteFile(*it);
     batch_files.clear(); // not needed anymore...
   }
 
-#ifdef HAVE_LIBREADLINE
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
 
   // initialize readline (own version - not pythons one)
   // in includefirst.hpp readline is disabled for python_module
@@ -1654,36 +1689,34 @@ RetCode DInterpreter::InterpreterLoop( const string& startup,
     int edit_input = SysVar::Edit_Input();
     stifle_history(edit_input == 1 || edit_input < 0 ? 200 : edit_input);
   }
-  
+
   // Eventually read back the ".gdl" path in user $HOME
   // we do not make one commun function with the save side
   // because on the save side we may need to create the .gdl/ PATH ...
-  int result, debug=0;
+  int result, debug = 0;
 #ifdef _WIN32
-  char *homeDir = getenv( "HOMEPATH");
+  char *homeDir = getenv("HOMEPATH");
 #else
-  char *homeDir = getenv( "HOME");
+  char *homeDir = getenv("HOME");
 #endif
-  if (homeDir != NULL)
-  {
+  if (homeDir != NULL) {
     string pathToGDL_history;
-    pathToGDL_history=homeDir;
+    pathToGDL_history = homeDir;
     AppendIfNeeded(pathToGDL_history, "/");
-    pathToGDL_history=pathToGDL_history+".gdl";
+    pathToGDL_history = pathToGDL_history + ".gdl";
     string history_filename;
     AppendIfNeeded(pathToGDL_history, "/");
-    history_filename=pathToGDL_history+"history";
-    if (debug) cout << "History file name: " <<history_filename << endl;
+    history_filename = pathToGDL_history + "history";
+    if (debug) cout << "History file name: " << history_filename << endl;
 
-    result=read_history(history_filename.c_str());
-    if (debug) 
-    { 
-      if (result == 0) cout<<"Successfull reading of ~/.gdl/history"<<endl;
-      else cout<<"Fail to read back ~/.gdl/history"<<endl;
+    result = read_history(history_filename.c_str());
+    if (debug) {
+      if (result == 0) cout << "Successfull reading of ~/.gdl/history" << endl;
+      else cout << "Fail to read back ~/.gdl/history" << endl;
     }
   }
 
-historyIntialized = true;
+  historyIntialized = true;
 
 #endif
 
@@ -1699,88 +1732,115 @@ historyIntialized = true;
     feclearexcept(FE_ALL_EXCEPT);
 #endif
 
-    try
-      {
-	if( runCmd)
-	  {
-	    runCmd = false;
-	    continueCmd = false;
-	    RunDelTree();
-	  }
-	else
-	  {
-	    DInterpreter::CommandCode ret=ExecuteLine();
+    try {
+      if (runCmd) {
+        runCmd = false;
+        continueCmd = false;
+        RunDelTree();
+      } else {
+        DInterpreter::CommandCode ret = ExecuteLine();
 
-	    // stop steppig when at main level
-	    stepCount = 0;
-	    debugMode = DEBUG_CLEAR;
+        // stop steppig when at main level
+        stepCount = 0;
+        debugMode = DEBUG_CLEAR;
 
-	    if( ret == CC_SKIP)
-	    {
-	      Message( "Can't continue from this point.");
-	    }		    
-	    else if( ret == CC_CONTINUE)
-	      {
-		if( static_cast<DSubUD*>
-		    (callStack.back()->GetPro())->GetTree() != NULL)
-		  {
-		    if( continueCmd) 
-		      runCmd = true;
-		    else
-		      {
-			cout << SysVar::MsgPrefix() << 
-			  "Starting at: $MAIN$" << endl;
-			continueCmd = true;
-		      }
-		  }
-		else
-		  cout << SysVar::MsgPrefix() << 
-		    "Cannot continue from this point." << endl;
-	      }
-	  }
+        if (ret == CC_SKIP) {
+          Message("Can't continue from this point.");
+        }
+        else if (ret == CC_CONTINUE) {
+          if (static_cast<DSubUD*>
+            (callStack.back()->GetPro())->GetTree() != NULL) {
+            if (continueCmd)
+              runCmd = true;
+            else {
+              cout << SysVar::MsgPrefix() <<
+                "Starting at: $MAIN$" << endl;
+              continueCmd = true;
+            }
+          } else
+            cout << SysVar::MsgPrefix() <<
+            "Cannot continue from this point." << endl;
+        }
       }
-    catch( RetAllException& retAllEx)
-      {
-	runCmd = (retAllEx.Code() == RetAllException::RUN);
-	bool resetCmd = (retAllEx.Code() == RetAllException::RESET);
-	bool fullResetCmd = (retAllEx.Code() == RetAllException::FULL_RESET);
-	if( resetCmd || fullResetCmd)
-	{
-	  // remove $MAIN$
-	  delete callStack.back();
-	  callStack.pop_back();
-	  assert( callStack.empty());
-	  
-	  ResetObjects();
-	  ResetHeap();
-	  if( fullResetCmd)
-	  {
-	    PurgeContainer(libFunList);
-	    PurgeContainer(libProList);
-	  }
-	  // initially done in InitGDL()
-	  // initializations
-	  InitObjects();
-	  // init library functions
-	  if( fullResetCmd)
-	  {
-	      LibInit(); 
-	  }
-	  
-	  // initially done in constructor: setup main level environment
-	  DPro* mainPro=new DPro();        // $MAIN$  NOT inserted into proList
-	  EnvUDT* mainEnv=new EnvUDT(NULL, mainPro);
-	  callStack.push_back(mainEnv);   // push main environment (necessary)
-	}
-      }
-    catch( exception& e)
-      {
-	cerr << "InterpreterLoop: Exception: " << e.what() << endl;
-      }
-    catch (...)
-      {	
-	cerr << "InterpreterLoop: Unhandled Error." << endl;
+    }    catch (RetAllException& retAllEx) {
+      runCmd = (retAllEx.Code() == RetAllException::RUN);
+      bool resetCmd = (retAllEx.Code() == RetAllException::RESET);
+      bool fullResetCmd = (retAllEx.Code() == RetAllException::FULL_RESET);
+      if (resetCmd || fullResetCmd) {
+        // remove $MAIN$
+        delete callStack.back();
+        callStack.pop_back();
+        assert(callStack.empty());
+
+        ResetObjects();
+        ResetHeap();
+        if (fullResetCmd) {
+          PurgeContainer(libFunList);
+          PurgeContainer(libProList);
+        }
+        // initially done in InitGDL()
+        // initializations
+        InitObjects();
+        // init library functions
+        if (fullResetCmd) {
+          LibInit();
+        }
+
+        // initially done in constructor: setup main level environment
+        DPro* mainPro = new DPro(); // $MAIN$  NOT inserted into proList
+        EnvUDT* mainEnv = new EnvUDT(NULL, mainPro);
+        callStack.push_back(mainEnv); // push main environment (necessary)
+
+        // re-process startup file
+        if (startup != "") {
+          ifstream in(startup.c_str());
+
+          if (in.fail())
+            Warning("Error opening startup file: " + startup);
+
+          ValueGuard<bool> guard(interruptEnable);
+          interruptEnable = false;
+
+          bool runCmd = false;
+          try {
+            while (in.good()) {
+#if defined (_MSC_VER) && _MSC_VER < 1800
+              _clearfp();
+#else
+              feclearexcept(FE_ALL_EXCEPT);
+#endif
+
+              try {
+                if (runCmd) {
+                  runCmd = false;
+                  RunDelTree();
+                } else {
+                  DInterpreter::CommandCode ret = ExecuteLine(&in);
+
+                  if (debugMode != DEBUG_CLEAR) {
+                    debugMode = DEBUG_CLEAR;
+                    Warning("Prematurely closing batch file: " + startup);
+                    break;
+                  }
+                }
+              } catch (RetAllException& retAllEx) {
+                runCmd = (retAllEx.Code() == RetAllException::RUN);
+                if (!runCmd) throw;
+              } catch (exception& e) {
+                cerr << startup << ": Exception: " << e.what() << endl;
+              } catch (...) {
+                cerr << startup << ": Unhandled Error." << endl;
+              }
+            } // while
+          } catch (RetAllException& retAllEx) {
+          }
+        } // if( startup...
       }
+    }    catch (exception& e) {
+      cerr << "InterpreterLoop: Exception: " << e.what() << endl;
+    }    catch (...) {
+      cerr << "InterpreterLoop: Unhandled Error." << endl;
+    }
   }
 }
 
