@@ -4,7 +4,7 @@
     begin                : July 22 2002
     copyright            : (C) 2002 by Marc Schellens
     email                : m_schellens@users.sf.net
- ***************************************************************************/
+***************************************************************************/
 
 /***************************************************************************
  *                                                                         *
@@ -45,8 +45,8 @@ DSub::~DSub() {}
 DLib::DLib( const string& n, const string& o, const int nPar_, 
 	    const string keyNames[],
 	    const string warnKeyNames[], const int nParMin_)
- : DSub(n,o)
- , hideHelp( false)
+  : DSub(n,o)
+  , hideHelp( false)
 {
   nPar=nPar_;
   nParMin = nParMin_;
@@ -55,7 +55,7 @@ DLib::DLib( const string& n, const string& o, const int nPar_,
   SizeT nKey_=0;
   if( keyNames != NULL)
     {
-    while( keyNames[nKey_] != "") ++nKey_;
+      while( keyNames[nKey_] != "") ++nKey_;
     }
 
   key.resize(nKey_);
@@ -77,7 +77,7 @@ DLib::DLib( const string& n, const string& o, const int nPar_,
   SizeT nWarnKey_=0;
   if( warnKeyNames != NULL)
     {
-    while( warnKeyNames[nWarnKey_] != "") ++nWarnKey_;
+      while( warnKeyNames[nWarnKey_] != "") ++nWarnKey_;
     }
 
   warnKey.resize(nWarnKey_);
@@ -100,6 +100,22 @@ const string DLibPro::ToString()
   return s;
 }
 
+const string DPro::ToString()
+{
+  string s;
+  if( object != "") s += object+"::";
+  s += name;
+  if( nPar == -1)
+    s += ",[inf. Args]";
+  else if( nPar == 1)
+    s += ",["+i2s( nPar)+" Arg]";
+  else if( nPar > 1)
+    s += ",["+i2s( nPar)+" Args]";
+  for( SizeT i=0; i<key.size(); ++i)
+    s += ","+key[ i];//+"=val";
+  return s;
+}
+
 const string DLibFun::ToString()
 {
   string s = "res=";
@@ -117,13 +133,38 @@ const string DLibFun::ToString()
       for( SizeT i=0; i<key.size(); ++i)
 	{
 	  s += key[ i];//+"=val";
-	    if( i+1 != key.size()) s += ",";
+	  if( i+1 != key.size()) s += ",";
 	}
     }
   s += ")";
   return s;
 }
 
+const string DFun::ToString()
+{
+  string s = "res=";
+  if( object != "") s += object+"::";
+  s += name+"(";
+  if( nPar == -1)
+    s += "[inf. Args]";
+  else if( nPar == 1)
+    s += "["+i2s( nPar)+" Arg]";
+  else if( nPar > 1)
+    s += "["+i2s( nPar)+" Args]";
+  if( key.size() > 0)
+    {
+      if( nPar != 0) s += ",";
+      for( SizeT i=0; i<key.size(); ++i)
+	{
+	  s += key[ i];//+"=val";
+	  if( i+1 != key.size()) s += ",";
+	}
+    }
+  s += ")";
+  return s;
+}
+
+
 // const string DLibFunDirect::ToString()
 // {
 //   string s = "res=";
@@ -134,40 +175,44 @@ const string DLibFun::ToString()
 // }
 
 DLibPro::DLibPro( LibPro p, const string& n, const string& o, const int nPar_, 
-	 const string keyNames[], const string warnKeyNames[], const int nParMin_)
+		  const string keyNames[], const string warnKeyNames[], const int nParMin_)
   : DLib(n,o,nPar_,keyNames, warnKeyNames, nParMin_), pro(p)
 {
   libProList.push_back(this);
+//  sort(libProList.begin(), libProList.end(),CompLibFunName());
 }
 DLibPro::DLibPro( LibPro p, const string& n, const int nPar_, 
-	 const string keyNames[], const string warnKeyNames[], const int nParMin_)
+		  const string keyNames[], const string warnKeyNames[], const int nParMin_)
   : DLib(n,"",nPar_,keyNames, warnKeyNames, nParMin_), pro(p)
 {
   libProList.push_back(this);
+//  sort(libProList.begin(), libProList.end(),CompLibFunName());
 }
 
 DLibFun::DLibFun( LibFun f, const string& n, const string& o, const int nPar_, 
-	 const string keyNames[], const string warnKeyNames[], const int nParMin_)
+		  const string keyNames[], const string warnKeyNames[], const int nParMin_)
   : DLib(n,o,nPar_,keyNames, warnKeyNames, nParMin_), fun(f)
 {
   libFunList.push_back(this);
+//  sort(libFunList.begin(), libFunList.end(),CompLibFunName());
 }
 
 DLibFun::DLibFun( LibFun f, const string& n, const int nPar_, 
-	 const string keyNames[], const string warnKeyNames[], const int nParMin_)
+		  const string keyNames[], const string warnKeyNames[], const int nParMin_)
   : DLib(n,"",nPar_,keyNames, warnKeyNames, nParMin_), fun(f)
 {
   libFunList.push_back(this);
+//  sort(libFunList.begin(), libFunList.end(),CompLibFunName());
 }
 DLibFunRetNew::DLibFunRetNew( LibFun f, const string& n, 
-			const string& o, const int nPar_, 
-			const string keyNames[], const string warnKeyNames[], const int nParMin_)
+			      const string& o, const int nPar_, 
+			      const string keyNames[], const string warnKeyNames[], const int nParMin_)
   : DLibFun(f,n,o,nPar_,keyNames, warnKeyNames, nParMin_)
 {}
 
 DLibFunRetNew::DLibFunRetNew( LibFun f, const string& n, const int nPar_, 
-			const string keyNames[], const string warnKeyNames[], bool rConstant,
-			const int nParMin_)
+			      const string keyNames[], const string warnKeyNames[], bool rConstant,
+			      const int nParMin_)
   : DLibFun(f,n,nPar_,keyNames, warnKeyNames, nParMin_), retConstant( rConstant)
 {}
 // DLibFunRetNew::DLibFunRetNew( LibFun f, const string& n, const int nPar_, 
@@ -200,8 +245,8 @@ DSubUD::~DSubUD()
 DSubUD::DSubUD(const string& n,const string& o,const string& f) : 
   DSub(n,o), file(f),
   tree( NULL),
-  labelList(),
   compileOpt(GDLParser::NONE),
+  labelList(),
   nForLoops( 0)
 {
   if( o != "")
@@ -271,8 +316,8 @@ DSubUD* DSubUD::AddKey(const string& k, const string& v)
       if(extraIx != -1) extraIx++; // update extra ix index
     }
   // as we only push_front during compilation, we better use a vector here
-//   key.push_front(k);
-//   var.push_back(v);
+  //   key.push_front(k);
+  //   var.push_back(v);
   // we want push_front
   key.resize( key.size() + 1);
   for( int i= key.size()-1; i>0; --i)
@@ -295,17 +340,17 @@ void DSubUD::ResolveLabel( ProgNodeP p)
 {
   if( p == NULL) return;
 
-// if( p->getNextSibling() != NULL)
-// 	std::cout << "Resolve("<< p->getLine()<<"): " << p << " keepRight: " << p->KeepRight()<< ": "<< p->getText() <<"  r: "<< p->GetNextSibling()->getText() << std::endl;
-// else
-// 	std::cout << "Resolve("<< p->getLine()<<"): " << p << " keepRight: " << p->KeepRight()<< ": "<< p->getText() <<"  r: NULL"<< std::endl;
+  // if( p->getNextSibling() != NULL)
+  // 	std::cout << "Resolve("<< p->getLine()<<"): " << p << " keepRight: " << p->KeepRight()<< ": "<< p->getText() <<"  r: "<< p->GetNextSibling()->getText() << std::endl;
+  // else
+  // 	std::cout << "Resolve("<< p->getLine()<<"): " << p << " keepRight: " << p->KeepRight()<< ": "<< p->getText() <<"  r: NULL"<< std::endl;
   
   if( p->getType() == GDLTreeParser::ON_IOERROR || 
       p->getType() == GDLTreeParser::GOTO)
     {
       int ix = labelList.Find( p->getText());
       if( ix == -1)
-		throw GDLException( p, ObjectName()+": Undefined label "+p->getText()+
+	throw GDLException( p, ObjectName()+": Undefined label "+p->getText()+
 			    " referenced in GOTO statement.",false,false);
       
       p->SetGotoIx( ix);
@@ -316,16 +361,16 @@ void DSubUD::ResolveLabel( ProgNodeP p)
     }
   
   if( !p->KeepDown())
-	ResolveLabel( p->getFirstChild());
+    ResolveLabel( p->getFirstChild());
   if( !p->KeepRight())
-	ResolveLabel( p->getNextSibling());
-//   else
-// 	ResolveLabel( p->getNextSibling());
+    ResolveLabel( p->getNextSibling());
+  //   else
+  // 	ResolveLabel( p->getNextSibling());
 }
 
-  // **************************************
-  // the final "compilation" takes part here 
-  // **************************************
+// **************************************
+// the final "compilation" takes part here 
+// **************************************
 void DSubUD::SetTree( RefDNode n)
 {
   //
@@ -335,20 +380,64 @@ void DSubUD::SetTree( RefDNode n)
   // ***************************************************
   tree = ProgNode::NewProgNode( n);
 
-//   if( tree != NULL)
-    nForLoops = ProgNode::NumberForLoops( tree);
-//   else
-// 	nForLoops = 0;
+  //   if( tree != NULL)
+  nForLoops = ProgNode::NumberForLoops( tree);
+  //   else
+  // 	nForLoops = 0;
   
 #ifdef GDL_DEBUG
-      cout << "DSubUD::SetTree:" << endl;
-      antlr::print_tree pt;
-      pt.pr_tree(tree);
-      cout << "DSubUD::SetTree end." << endl;
+  cout << "DSubUD::SetTree:" << endl;
+  antlr::print_tree pt;
+  pt.pr_tree(tree);
+  cout << "DSubUD::SetTree end." << endl;
 #endif
 
 }
 
+bool DSubUD::GetCommonVarName(const BaseGDL* p, std::string& varName)
+{
+  CommonBaseListT::iterator it;
+  for( it=common.begin(); it != common.end(); it++)
+    {
+      int vIx = (*it)->Find( p);
+      if( vIx >= 0) 
+	{
+	  varName=(*it)->VarName( vIx);
+	  return true;
+	}
+    }
+  return false;
+}
+
+BaseGDL* DSubUD::GetCommonVarNameList()
+{
+  DStringGDL* res = new DStringGDL(dimension(this->CommonsSize()), BaseGDL::NOZERO);
+  CommonBaseListT::iterator it;
+  SizeT iname=0;
+  for( it=common.begin(); it != common.end(); it++)
+    {
+    SizeT nVar=(*it)->NVar();
+    for (SizeT i=0; i< nVar ; ++i) (*res)[iname++]=(*it)->VarName(i);
+    }
+  return res;
+}
+
+bool DSubUD::GetCommonVarName4Help(const BaseGDL* p, std::string& varName)
+{
+  CommonBaseListT::iterator it;
+  for( it=common.begin(); it != common.end(); it++)
+    {
+      int vIx = (*it)->Find( p);
+      if( vIx >= 0) 
+	{
+	  varName=(*it)->VarName( vIx) +" ("+(*it)->Name()+')';
+	  return true;
+	}
+    }
+  return false;
+}
+
+
 bool DSubUD::isObsolete()
 {
   return compileOpt & GDLParser::OBSOLETE;
