@@ -18,7 +18,7 @@
 #ifndef DPRO_HPP_
 #define DPRO_HPP_
 
-#include <deque>
+// #include <deque>
 #include <string>
 #include <algorithm>
 #include <vector>
@@ -30,7 +30,7 @@
 #include "prognode.hpp"
 #include "str.hpp"
 
-#include "antlr/Token.hpp"
+#include <antlr/Token.hpp>
 
 template<typename T>  class Is_eq: public std::unary_function<T,bool>
 {
@@ -114,15 +114,15 @@ protected:
   // par_1,..,par_nPar, var1,..,varK
   // N=size(key)
   // K=size(var)-nPar-N
-  IDList              key;    // keyword names (IDList: typedefs.hpp)
-                              // (KEYWORD_NAME=keyword_value)
+  KeyVarListT 	      key;    // keyword names (IDList: typedefs.hpp)
+			      // (KEYWORD_NAME=keyword_value)
   int                 nPar;   // number of parameters (-1 = infinite)
   int                 nParMin;  // minimum number of parameters (-1 = infinite)
 
   ExtraType           extra;
   int                 extraIx; // index of extra keyword
 
-  IDList  warnKey;    // keyword names to accept but warn
+  IDList              warnKey;    // keyword names to accept but warn
   // (IDList: typedefs.hpp)
 
 public:
@@ -160,8 +160,8 @@ public:
   {
     String_abbref_eq searchKey(s);
     int ix=0;
-    for(IDList::iterator i=key.begin();
-	i != key.end(); i++, ix++) if( searchKey(*i)) {
+    for(KeyVarListT::iterator i=key.begin();
+	i != key.end(); ++i, ++ix) if( searchKey(*i)) {
 	  return ix;
 	}
     return -1;
@@ -180,11 +180,12 @@ public:
 };
 
 // Lib pro/fun ********************************************************
-class EnvT;
-
-typedef void     (*LibPro)(EnvT*);
-typedef BaseGDL* (*LibFun)(EnvT*);
-typedef BaseGDL* (*LibFunDirect)(BaseGDL* param,bool canGrab);
+// moved to prognode.hpp
+// class EnvT;
+// 
+// typedef void     (*LibPro)(EnvT*);
+// typedef BaseGDL* (*LibFun)(EnvT*);
+// typedef BaseGDL* (*LibFunDirect)(BaseGDL* param,bool canGrab);
 
 // library procedure/function (in cases both are handled the same way)
 class DLib: public DSub
@@ -200,6 +201,14 @@ public:
   
   bool GetHideHelp() const { return hideHelp;}
   void SetHideHelp( bool v) { hideHelp = v;}
+
+  // for sorting lists by name. Not used (lists too short to make a time gain. Long lists would, if existing,
+  // benefit from sorting by hash number in a std::map instead of a std::list.
+  struct CompLibFunName: public std::binary_function< DLib*, DLib*, bool>
+  {
+    bool operator() ( DLib* f1, DLib* f2) const
+    { return f1->ObjectName() < f2->ObjectName();}
+  };
 };
 
 // library procedure
@@ -292,7 +301,7 @@ class DSubUD: public DSub
 {
   std::string         file;        // filename were procedure is defined in
 
-  IDList              var;         // keyword values, parameters, local variables
+  KeyVarListT         var;         // keyword values, parameters, local variables
 
   CommonBaseListT     common;      // common blocks or references 
   ProgNodeP           tree;        // the 'code'
@@ -314,6 +323,11 @@ public:
   void SetTree( ProgNodeP t) { tree = t;}
 
   void AddCommon(DCommonBase* c) { common.push_back(c);}
+  void DeleteLastAddedCommon()
+  {
+    delete common.back();
+    common.pop_back();
+  }
   
   void ResolveAllLabels();
   LabelListT& LabelList() { return labelList;}
@@ -334,18 +348,24 @@ public:
   // add variables
   DSubUD*  AddPar(const std::string&); // add paramter
   unsigned AddVar(const std::string&); // add local variable
-  void     AddKey(const std::string&, const std::string&); // add keyword=value
+  DSubUD*  AddKey(const std::string&, const std::string&); // add keyword=value
 
   void     DelVar(const int ix) {var.erase(var.begin() + ix);}
 
-  SizeT Size() { return var.size();}
-
+  SizeT Size() {return var.size();}
+  SizeT CommonsSize() {
+   SizeT commonsize=0;
+   CommonBaseListT::iterator c = common.begin();
+   for(; c != common.end(); ++c) commonsize+=(*c)->NVar();
+   return commonsize;
+   }
+  
   int NForLoops() const { return nForLoops;}
   
   // search for variable returns true if its found in var or common blocks
   bool Find(const std::string& n)
   {
-    IDList::iterator f=std::find(var.begin(),var.end(),n);
+    KeyVarListT::iterator f=std::find(var.begin(),var.end(),n);
     if( f != var.end()) return true;
 
     CommonBaseListT::iterator c=
@@ -383,24 +403,14 @@ public:
     return key[ix];
   }
 
-  bool GetCommonVarName(const BaseGDL* p, std::string& varName)
-  {
-    for( CommonBaseListT::iterator c=common.begin();
-	 c != common.end(); c++)
-      {
-	int vIx = (*c)->Find( p);
-	if( vIx >= 0) 
-	  {
-	    varName=(*c)->VarName( vIx);
-	    return true;
-	  }
-      }
-    return false;
-  }
+  BaseGDL* GetCommonVarNameList();
+  bool GetCommonVarName(const BaseGDL* p, std::string& varName);
+  bool GetCommonVarName4Help(const BaseGDL* p, std::string& varName);
+
   BaseGDL** GetCommonVarPtr(const BaseGDL* p)
   {
     for( CommonBaseListT::iterator c=common.begin();
-	 c != common.end(); c++)
+	 c != common.end(); ++c)
       {
 	int vIx = (*c)->Find( p);
 	if( vIx >= 0) 
@@ -411,18 +421,43 @@ public:
       }
     return NULL;
   }
-
+  
+  BaseGDL** GetCommonVarPtr(std::string& s)
+  {
+    for(CommonBaseListT::iterator c=common.begin();
+   	c != common.end(); ++c)
+      {
+       	DVar* v=(*c)->Find(s);
+       	if (v) return &(v->Data());
+      }
+    return NULL;
+  }
+  
+  bool ReplaceExistingCommonVar(std::string& s, BaseGDL* val)
+  {
+    for(CommonBaseListT::iterator c=common.begin();
+   	c != common.end(); ++c)
+      {
+       	DVar* v=(*c)->Find(s);
+       	if (v) { 
+         delete (v)->Data();
+         (v)->SetData(val);
+         return true;
+        }
+      }
+    return false;
+  }  
   // returns the variable index (-1 if not found)
   int FindVar(const std::string& s)
   {
-    return FindInIDList(var,s);
+    return FindInKeyVarListT(var,s);
   }
 
   // returns ptr to common variable (NULL if not found)
   DVar* FindCommonVar(const std::string& s) 
   { 
     for(CommonBaseListT::iterator c=common.begin();
-   	c != common.end(); c++)
+   	c != common.end(); ++c)
       {
        	DVar* v=(*c)->Find(s);
        	if( v) return v;
@@ -465,6 +500,9 @@ public:
     DSubUD(n,o,f)
   {}
   
+  const std::string ToString();
+
+
   ~DPro() {}; 
 };
 
@@ -474,15 +512,17 @@ public:
   DFun(const std::string& n,const std::string& o="",const std::string& f=""): 
     DSubUD(n,o,f)
   {} 
-  
+
+  const std::string ToString();
+
   ~DFun() {};
 };
 
 
-typedef std::deque<DFun*> FunListT;
-typedef std::deque<DPro*> ProListT;
+typedef std::vector<DFun*> FunListT;
+typedef std::vector<DPro*> ProListT;
 
-typedef std::deque<DLibFun*> LibFunListT;
-typedef std::deque<DLibPro*> LibProListT;
+typedef std::vector<DLibFun*> LibFunListT;
+typedef std::vector<DLibPro*> LibProListT;
 
 #endif
