@@ -915,7 +915,7 @@ void EnvT::Help(const std::string s_help[], int size_of_s)
     throw GDLException( CallingNode(), pro->ObjectName()+": call to inline help");
   }
 }
-
+//TODO: variant enabling static ints in lieu of const string& (speedup!)
 void EnvBaseT::SetKeyword( const string& k, BaseGDL* const val) // value
 {
   int varIx=GetKeywordIx( k);
@@ -943,6 +943,7 @@ void EnvBaseT::SetKeyword( const string& k, BaseGDL* const val) // value
 
   env.Set( varIx,val);
 }
+//TODO: variant enabling static ints in lieu of const string& (speedup!)
 
 void EnvBaseT::SetKeyword( const string& k, BaseGDL** const val) // reference
 {
@@ -1098,35 +1099,29 @@ void EnvBaseT::AssureGlobalKW( SizeT ix)
   }
 }
 
-DStructGDL* EnvT::GetObjectPar( SizeT pIx)
-{
-  BaseGDL* p1= GetParDefined( pIx);
-  
-  if( p1->Type() != GDL_OBJ)
-    {
-      Throw( "Parameter must be an object reference"
-	     " in this context: "+
-	     GetParString(pIx));
-    }
-  else
-    {
-      DObjGDL* oRef = static_cast<DObjGDL*>(p1);
-      DObj objIx;
-      if( !oRef->Scalar( objIx))
-	Throw( "Parameter must be a scalar in this context: "+
-	       GetParString(pIx));
-      if( objIx == 0)
-	Throw( "Unable to invoke method"
-	       " on NULL object reference: "+GetParString(pIx));
-      
-      try {
-	return GetObjHeap( objIx);
-      }
-      catch ( GDLInterpreter::HeapException)
-	{
-	  Throw( "Object not valid: "+GetParString(pIx));
-	}
+DStructGDL* EnvT::GetObjectPar( SizeT pIx) {
+  BaseGDL* p1 = GetParDefined(pIx);
+
+  if (p1->Type() != GDL_OBJ) {
+    Throw("Parameter must be an object reference in this context: " +
+      GetParString(pIx));
+  } else {
+    DObjGDL* oRef = static_cast<DObjGDL*> (p1);
+    DObj objIx;
+    if (!oRef->Scalar(objIx))
+      Throw("Parameter must be a scalar or 1 element array in this context: " +
+      GetParString(pIx));
+    if (objIx == 0)
+      Throw("Unable to invoke method"
+      " on NULL object reference: " + GetParString(pIx));
+
+    try {
+      return GetObjHeap(objIx);
+    } catch (GDLInterpreter::HeapException) {
+      Throw("Object not valid: " + GetParString(pIx));
     }
+  }
+  return NULL; //keep clang happy.
 }
 
 // for exclusive use by lib::catch_pro
@@ -1171,8 +1166,9 @@ int EnvT::KeywordIx( const std::string& k)
 {
   //  cout << pro->ObjectName() << "  Key: " << k << endl;
   assert( pro != NULL);
-  assert( pro->FindKey( k) != -1);
-  return pro->FindKey( k);
+  int val=pro->FindKey( k);
+  assert( val != -1);
+  return val;
 }
 
 bool EnvT::KeywordPresent( const std::string& kw)
@@ -1232,9 +1228,7 @@ BaseGDL*& EnvBaseT::GetParDefined(SizeT i)
   SizeT ix = i + pro->key.size();
 
   // cout << i << " -> " << ix << "  " << env.size() << "  env[ix] " << env[ix] << endl;
-  if( ix >= env.size())
-    Throw("Incorrect number of arguments.");
-  if( env[ ix] == NULL || env[ ix] == NullGDL::GetSingleInstance())
+  if( ix >= env.size() || env[ ix] == NULL || env[ ix] == NullGDL::GetSingleInstance())
     Throw("Variable is undefined: "+GetString( ix));
   return env[ ix];
 }
@@ -1472,7 +1466,7 @@ void EnvBaseT::AssureLongScalarPar( SizeT pIx, DLong64& scalar)
   DLong64GDL* lp = static_cast<DLong64GDL*>(p->Convert2( GDL_LONG64, BaseGDL::COPY));
   Guard<DLong64GDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
-    Throw("Parameter must be a scalar in this context: "+
+    Throw("Parameter must be a scalar or 1 element array in this context: "+
 	  GetParString(pIx));
 }
 void EnvBaseT::AssureLongScalarPar( SizeT pIx, DLong& scalar)
@@ -1481,7 +1475,7 @@ void EnvBaseT::AssureLongScalarPar( SizeT pIx, DLong& scalar)
   DLongGDL* lp = static_cast<DLongGDL*>(p->Convert2( GDL_LONG, BaseGDL::COPY));
   Guard<DLongGDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
-    Throw("Parameter must be a scalar in this context: "+
+    Throw("Parameter must be a scalar or 1 element array in this context: "+
 	  GetParString(pIx));
 }
 void EnvT::AssureLongScalarPar( SizeT pIx, DLong64& scalar)
@@ -1528,7 +1522,7 @@ void EnvT::AssureLongScalarKW( SizeT eIx, DLong64& scalar)
   Guard<DLong64GDL> guard_lp( lp);
 
   if( !lp->Scalar( scalar))
-    Throw("Expression must be a scalar in this context: "+
+    Throw("Expression must be a scalar or 1 element array in this context: "+
 	  GetString(eIx));
 }
 void EnvT::AssureLongScalarKW( SizeT eIx, DLong& scalar)
@@ -1543,7 +1537,7 @@ void EnvT::AssureLongScalarKW( SizeT eIx, DLong& scalar)
   Guard<DLongGDL> guard_lp( lp);
 
   if( !lp->Scalar( scalar))
-    Throw("Expression must be a scalar in this context: "+
+    Throw("Expression must be a scalar or 1 element array in this context: "+
 	  GetString(eIx));
 }
 
@@ -1553,7 +1547,7 @@ void EnvT::AssureDoubleScalarPar( SizeT pIx, DDouble& scalar)
   DDoubleGDL* lp = static_cast<DDoubleGDL*>(p->Convert2( GDL_DOUBLE, BaseGDL::COPY));
   Guard<DDoubleGDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
-    Throw("Parameter must be a scalar in this context: "+
+    Throw("Parameter must be a scalar or 1 element array in this context: "+
 	  GetParString(pIx));
 }
 void EnvT::AssureDoubleScalarKWIfPresent( const std::string& kw, DDouble& scalar)
@@ -1585,7 +1579,7 @@ void EnvT::AssureDoubleScalarKW( SizeT eIx, DDouble& scalar)
   Guard<DDoubleGDL> guard_lp( lp);
 
   if( !lp->Scalar( scalar))
-    Throw("Expression must be a scalar in this context: "+
+    Throw("Expression must be a scalar or 1 element array in this context: "+
 	  GetString(eIx));
 }
 
@@ -1596,7 +1590,7 @@ void EnvT::AssureFloatScalarPar( SizeT pIx, DFloat& scalar)
   DFloatGDL* lp = static_cast<DFloatGDL*>(p->Convert2( GDL_FLOAT, BaseGDL::COPY));
   Guard<DFloatGDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
-    Throw("Parameter must be a scalar in this context: "+
+    Throw("Parameter must be a scalar or 1 element array in this context: "+
 	  GetParString(pIx));
 }
 void EnvT::AssureFloatScalarKWIfPresent( const std::string& kw, DFloat& scalar)
@@ -1628,7 +1622,7 @@ void EnvT::AssureFloatScalarKW( SizeT eIx, DFloat& scalar)
   Guard<DFloatGDL> guard_lp( lp);
 
   if( !lp->Scalar( scalar))
-    Throw("Expression must be a scalar in this context: "+
+    Throw("Expression must be a scalar or 1 element array in this context: "+
 	  GetString(eIx));
 }
 
@@ -1639,7 +1633,7 @@ void EnvT::AssureStringScalarPar( SizeT pIx, DString& scalar)
   DStringGDL* lp = static_cast<DStringGDL*>(p->Convert2( GDL_STRING, BaseGDL::COPY));
   Guard<DStringGDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
-    Throw("Parameter must be a scalar in this context: "+
+    Throw("Parameter must be a scalar or 1 element array in this context: "+
 	  GetParString(pIx));
 }
 void EnvT::AssureStringScalarKWIfPresent( const std::string& kw, DString& scalar)
@@ -1669,7 +1663,7 @@ void EnvT::AssureStringScalarKW( SizeT eIx, DString& scalar)
   Guard<DStringGDL> guard_lp( lp);
 
   if( !lp->Scalar( scalar))
-    Throw("Expression must be a scalar in this context: "+
+    Throw("Expression must be a scalar or 1 element array in this context: "+
 	  GetString(eIx));
 }
 
