@@ -18,6 +18,7 @@
 #ifndef ENVT_HPP_
 #define ENVT_HPP_
 
+#include <limits>
 #include <vector>
 #include <cstdlib>
 
@@ -83,7 +84,7 @@ protected:
   bool                 obj;       // member subroutine?
   ExtraT*            extra;
 
-  EnvBaseT*      newEnv;
+  EnvBaseT*      newEnvOff;
 
   // finds the local variable pp points to
 //   int FindLocalKW( BaseGDL** pp) { return env.FindLocal( pp);}
@@ -106,6 +107,13 @@ public:
     return ptrToReturnValue;
     
   }
+  bool InLoc( BaseGDL** pp)
+  {
+    return env.InLoc(pp);
+  }
+  
+  
+  void SetKW( SizeT ix, BaseGDL* newVal);
 
   // used by the interperter returns the keyword index, used for UD functions
   // and used by WRAPPED subroutines
@@ -167,8 +175,8 @@ public:
 // 		std::cout << this->pro->ObjectName() << std::endl;
 // 	}
 
-	EnvBaseT* GetNewEnv() {return newEnv; }
-	void SetNewEnv( EnvBaseT* nE) { newEnv = nE;}
+  EnvBaseT* GetNewEnv() {return newEnvOff; }
+  void SetNewEnv( EnvBaseT* nE) { newEnvOff = nE;}
 
   virtual void ObjCleanup( DObj actID);
 
@@ -320,6 +328,8 @@ public:
 
   virtual const std::string GetFilename() const=0;
 
+  void AssureGlobalKW( SizeT ix);
+
   // converts parameter 'ix' if necessary and sets 'scalar' 
   void AssureLongScalarPar( SizeT ix, DLong& scalar);
   void AssureLongScalarPar( SizeT ix, DLong64& scalar);
@@ -500,6 +510,7 @@ public:
     return static_cast<DSubUD*>( pro)->GotoTarget( ix);
   }
   CallContext GetCallContext() const { return callContext;} // left-function
+  void SetCallContext(CallContext cC) { callContext = cC;} // left-function
 
   void SetIOError( int targetIx) 
   { // this isn't a jump
@@ -552,7 +563,7 @@ public:
 
   // used by obj_new (basic_fun.cpp)
   EnvT* NewEnv(  DSub* newPro, SizeT skipP, DObjGDL** newObj=NULL);
-  void PushNewEnvUD(  DSubUD* newPro, SizeT skipP, DObjGDL** newObj=NULL);
+  EnvUDT* PushNewEnvUD(  DSubUD* newPro, SizeT skipP, DObjGDL** newObj=NULL);
   // for exclusive use by lib::on_error
   void OnError();
   // for exclusive use by lib::catch_pro
@@ -605,8 +616,10 @@ public:
     if( NumericType( p0->Type()))
 	return p0;
 
-    if( p0->Type() == GDL_STRING)
-      Throw( "String expression not allowed in this context: "+GetParString(ix));
+    // AC 2014-08-14 : in fact, in most case, a tentative of String to Numeric
+    // convertion is done. E.g. invert(['1']) is OK !
+    //    if( p0->Type() == GDL_STRING)
+    //  Throw( "String expression not allowed in this context: "+GetParString(ix));
     if( p0->Type() == GDL_STRUCT)
       Throw( "Struct expression not allowed in this context: "+GetParString(ix));
     if( p0->Type() == GDL_PTR)
@@ -657,8 +670,8 @@ public:
     BaseGDL* p = GetKW( ix);
     if( p == NULL)
       Throw( "Keyword is undefined: "+GetString( ix));
-	if( p->Type() == T::t)
-		return static_cast<T*>( p);
+    if( p->Type() == T::t)
+      return static_cast<T*>( p);
 //     T* res = dynamic_cast<T*>( p);
 //     if( res != NULL) return res;
     T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
@@ -672,8 +685,8 @@ public:
   {
     BaseGDL* p = GetPar( pIx);
     if( p == NULL) return NULL;
-	if( p->Type() == T::t)
-		return static_cast<T*>( p);
+    if( p->Type() == T::t)
+	return static_cast<T*>( p);
 //     T* res = dynamic_cast<T*>( p);
 //     if( res != NULL) return res;
     T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
@@ -769,7 +782,7 @@ public:
   // create the data (newVal) only if its necessary
   // if the functions throw, they delete newVal before -> no
   // guarding of newVal is needed
-  void SetKW( SizeT ix, BaseGDL* newVal);
+//   void SetKW( SizeT ix, BaseGDL* newVal);
   void SetPar( SizeT ix, BaseGDL* newVal);
 
   // Assure functions:
@@ -810,15 +823,17 @@ public:
   }
 
   void AssureGlobalPar( SizeT pIx);
-  void AssureGlobalKW( SizeT ix);
+//   void AssureGlobalKW( SizeT ix);
 
   // if keyword 'kw' is not set, 'scalar' is left unchanged
   void AssureLongScalarKWIfPresent( const std::string& kw, DLong& scalar);
   void AssureLongScalarKWIfPresent( SizeT ix, DLong& scalar);
   // converts keyword 'kw' if necessary and sets 'scalar' 
   void AssureLongScalarKW( const std::string& kw, DLong& scalar);
+  void AssureLongScalarKW( const std::string& kw, DLong64& scalar);
   // converts ix'th keyword if necessary and sets 'scalar' 
   void AssureLongScalarKW( SizeT ix, DLong& scalar);
+  void AssureLongScalarKW( SizeT ix, DLong64& scalar);
   // converts parameter 'ix' if necessary and sets 'scalar' 
   void AssureLongScalarPar( SizeT ix, DLong& scalar);
   void AssureLongScalarPar( SizeT ix, DLong64& scalar);
@@ -863,7 +878,7 @@ public:
   typedef SizeT size_type;
   typedef EnvUDT* pointer_type;
   
-  EnvStackT(): sz(defaultStackDepth), top(0) 
+  EnvStackT(): top(0), sz(defaultStackDepth) 
   {
     envStackFrame = new EnvUDT* [ sz+1];
     envStack = envStackFrame + 1;
