@@ -35,11 +35,11 @@ namespace lib {
 
   using namespace std;
 
-  static int szdbl=sizeof(double);
-  static int szflt=sizeof(float);
+//   static int szdbl=sizeof(double);
+//   static int szflt=sizeof(float);
 
   template < typename T>
-  T* fftw_template(BaseGDL* p0,
+  T* fftw_template(EnvT* e, BaseGDL* p0,
 		   SizeT nEl, SizeT dbl, SizeT overwrite, double direct)
   {
     int dim[MAXRANK];
@@ -49,8 +49,12 @@ namespace lib {
     if (overwrite == 0)
       res = new T( p0->Dim(), BaseGDL::ZERO);
     else
+    {
       res = (T*) p0;
-
+      if( e->GlobalPar(0))
+	e->SetPtrToReturnValue(&e->GetPar(0));
+    }
+    
     for( SizeT i=0; i<p0->Rank(); ++i) {
       dim[i] = (int) p0->Dim(p0->Rank()-i-1);
     }
@@ -77,7 +81,7 @@ TRACEOMP( __FILE__, __LINE__)
 #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
 {
 #pragma omp for
-	for( int i=0; i<nEl; ++i) {
+	for( OMPInt i=0; i<nEl; ++i) {
 	  out[i][0] /= nEl;
 	  out[i][1] /= nEl;
 	}
@@ -108,7 +112,7 @@ TRACEOMP( __FILE__, __LINE__)
 #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
 {
 #pragma omp for
-	for( int i=0; i<nEl; ++i) {
+	for( OMPInt i=0; i<nEl; ++i) {
 	  out_f[i][0] /= nEl;
 	  out_f[i][1] /= nEl;
 	}
@@ -176,11 +180,11 @@ TRACEOMP( __FILE__, __LINE__)
 
       DComplexDblGDL *p0C;
 
-      auto_ptr<BaseGDL> guard_p0C;
+      Guard<BaseGDL> guard_p0C;
 
       if( p0->Type() != GDL_COMPLEXDBL) {
 	p0C = static_cast<DComplexDblGDL*>(p0->Convert2( GDL_COMPLEXDBL, BaseGDL::COPY));
-        guard_p0C.reset(p0C); 
+        guard_p0C.Init(p0C); 
       } else
       {
 	  if( overwrite)
@@ -191,7 +195,7 @@ TRACEOMP( __FILE__, __LINE__)
        p0C = (DComplexDblGDL *) p0;
 	 }
 
-      return fftw_template< DComplexDblGDL> (p0C, nEl, dbl, overwrite, direct);
+      return fftw_template< DComplexDblGDL> (e,p0C, nEl, dbl, overwrite, direct);
 
     }
     else if( p0->Type() == GDL_COMPLEX) {
@@ -201,7 +205,7 @@ TRACEOMP( __FILE__, __LINE__)
 	  	e->StealLocalPar(0);
 // 		e->StealLocalParUndefGlobal(0);
 
-      return fftw_template< DComplexGDL> (p0, nEl, dbl, overwrite, direct);
+      return fftw_template< DComplexGDL> (e,p0, nEl, dbl, overwrite, direct);
 
     }
     else {
@@ -209,8 +213,8 @@ TRACEOMP( __FILE__, __LINE__)
 
       DComplexGDL* p0C = static_cast<DComplexGDL*>
 	(p0->Convert2( GDL_COMPLEX, BaseGDL::COPY));
-      auto_ptr<BaseGDL> guard_p0C( p0C); 
-      return fftw_template< DComplexGDL> (p0C, nEl, dbl, overwrite, direct);
+      Guard<BaseGDL> guard_p0C( p0C); 
+      return fftw_template< DComplexGDL> (e,p0C, nEl, dbl, overwrite, direct);
 
     }
   }
