@@ -17,9 +17,6 @@
 
 #include "includefirst.hpp"
 
-// should (Makefile.am) not be used anyway for the python module
-#ifndef PYTHON_MODULE
-
 // #ifndef VERSION
 // #define VERSION "0.9"
 // #endif
@@ -34,6 +31,13 @@
 #include <unistd.h> // isatty
 #endif
 #include <climits> // PATH_MAX
+//patch #90
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+#ifndef _WIN32
+#include <sys/resource.h> //rlimits to augment stack size (needed fot DICOM objects)
+#endif
 
 //#include <fenv.h>
 
@@ -102,7 +106,8 @@ void InitOpenMP() {
 
 void AtExit()
 {
-//   cerr << "AtExit()" << endl;
+  //this function probably cleans otherwise cleaned objets and should be called only for debugging purposes.
+  cerr << "Using AtExit() for debugging" << endl;
   cerr << flush; cout << flush; clog << flush;
   // clean up everything
   // (for debugging memory leaks)
@@ -111,9 +116,25 @@ void AtExit()
   PurgeContainer(libProList);
 }
 
+#ifndef _WIN32
+void GDLSetLimits()
+{
+#define GDL_PREFERED_STACKSIZE 20480000 //20000*1024 OK for the time being
+struct rlimit* gdlstack=new struct rlimit;
+  int r=getrlimit(RLIMIT_STACK,gdlstack); 
+//  cerr <<"Current rlimit = "<<gdlstack->rlim_cur<<endl;
+//  cerr<<"Max rlimit = "<<  gdlstack->rlim_max<<endl;     
+  if (gdlstack->rlim_max > GDL_PREFERED_STACKSIZE ) gdlstack->rlim_cur=GDL_PREFERED_STACKSIZE;
+  r=setrlimit(RLIMIT_STACK,gdlstack);
+}
+#endif
+
 void InitGDL()
 {
-#ifdef HAVE_LIBREADLINE
+#ifndef _WIN32
+  GDLSetLimits();
+#endif
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
   // initialize readline (own version - not pythons one)
   // in includefirst.hpp readline is disabled for python_module
   rl_initialize();
@@ -154,20 +175,25 @@ void InitGDL()
 
 // SA: for use in COMMAND_LINE_ARGS()
 namespace lib {
-  std::vector<char*> command_line_args;
+  extern std::vector<char*> command_line_args;
 }
 
 int main(int argc, char *argv[])
 {
+#if GDL_DEBUG
   if( atexit( AtExit) != 0) cerr << "atexit registration failed." << endl;
-
+#endif
   // indicates if the user wants to see the welcome message
   bool quiet = false;
+  bool gdlde = false;
 
   // keeps a list of files to be executed after the startup file
   // and before entering the interactive mode
   vector<string> batch_files;
   string statement;
+  string pretendRelease;
+  bool strict_syntax=false;
+  bool syntaxOptionSet=false;
 
   for( SizeT a=1; a< argc; ++a)
     {
@@ -179,6 +205,13 @@ int main(int argc, char *argv[])
 	  cout << "GDL options:" << endl;
 	  cout << "  --help (-h)        display this message" << endl;
 	  cout << "  --version (-V, -v) show version information" << endl;
+	  cout << "  --fakerelease X.y  pretend that !VERSION.RELEASE is X.y" << endl;
+	  cout << "  --fussy            implies that procedures adhere with modern IDL, where \"()\" are for functions and \"[]\" are for arrays." <<endl;
+      cout << "                     This speeds up (sometimes terribly) compilation but choke on every use of \"()\" with arrays." << endl;
+      cout << "                     Conversion of procedures to modern IDL can be done with D. Landsman's idlv4_to_v5 procedure." << endl;
+      cout << "                     Use enviromnment variable \"GDL_IS_FUSSY\" to set up permanently this feature." << endl;
+	  cout << "  --sloppy           Sets the traditional (default) compiling option where \"()\"  can be used both with functions and arrays." << endl;
+      cout << "                     Needed to counteract temporarily the effect of the enviromnment variable \"GDL_IS_FUSSY\"." << endl;
           cout << endl;
 	  cout << "IDL-compatible options:" << endl;
 	  cout << "  -arg value tells COMMAND_LINE_ARGS() to report" << endl;
@@ -254,6 +287,29 @@ int main(int argc, char *argv[])
         string(argv[a]) == "-vm" 
       )
         cerr << argv[0] << ": " << argv[a] << " option ignored." << endl;
+      else if (string(argv[a]) == "-gdlde")
+      {
+          gdlde = true;
+      }
+      else if (string(argv[a]) == "--fussy")
+      {
+          strict_syntax = true;
+          syntaxOptionSet = true;
+      }
+      else if (string(argv[a]) == "--sloppy")
+      {
+          strict_syntax = false;
+          syntaxOptionSet = true;
+      }      
+      else if (string(argv[a]) == "--fakerelease")
+      {
+        if (a == argc - 1)
+          {
+            cerr << "gdl: --fakerelease must be followed by a string argument like \"6.4\"" << endl;
+            return 0;
+          }
+        pretendRelease = string(argv[++a]);
+      }
       else if (*argv[a] == '-')
       {
         cerr << argv[0] << ": " << argv[a] << " option not recognized." << endl;
@@ -276,7 +332,7 @@ int main(int argc, char *argv[])
   // must be after !cpu initialisation
   InitOpenMP();
 
-  if( isatty(0) && !quiet) StartupMessage();
+  if (gdlde || (isatty(0) && !quiet)) StartupMessage();
 
   // instantiate the interpreter
   DInterpreter interpreter;
@@ -286,21 +342,32 @@ int main(int argc, char *argv[])
   if( gdlPath == "")
     {
       gdlPath = "+" GDLDATADIR "/lib";
-      if (isatty(0) && !quiet) cerr <<
+      if (gdlde || (isatty(0) && !quiet)) cerr <<
         "- Default library routine search path used (GDL_PATH/IDL_PATH env. var. not set): " << endl << 
         "  " << gdlPath << endl;
     }
+  std::string useWX=GetEnvString("GDL_USE_WX");
+  if (useWX == "YES" || useWX == "yes") cerr << "- Using WxWidgets as graphics library (windows and widgets)." <<endl;
   SysVar::SetGDLPath( gdlPath);
-
+  
+  if (!pretendRelease.empty()) SysVar::SetFakeRelease(pretendRelease);
+  //fussyness setup and change if switch at start
+  if (syntaxOptionSet) { //take it no matters any env. var.
+    if (strict_syntax == true) SetStrict(true);
+  } else {
+    if (GetEnvString("GDL_IS_FUSSY").size()> 0) SetStrict(true);
+  }
+  
+  
   string startup=GetEnvString("GDL_STARTUP");
   if( startup == "") startup=GetEnvString("IDL_STARTUP");
   if( startup == "")
     {
-      if (isatty(0) && !quiet) cerr << 
+      if (gdlde || (isatty(0) && !quiet)) cerr << 
         "- No startup file read (GDL_STARTUP/IDL_STARTUP env. var. not set). " << endl;
     }
 
-  if (isatty(0) && !quiet) 
+  if (gdlde || (isatty(0) && !quiet))
   {
     cerr << "- Please report bugs, feature or help requests and patches at:" << endl <<
       "  http://sourceforge.net/projects/gnudatalanguage/" << endl << endl;
@@ -348,5 +415,3 @@ int main(int argc, char *argv[])
 
   return 0;
 }
-
-#endif
