@@ -49,6 +49,8 @@ using namespace std;
 
 void GDLGStream::Thick(DFloat thick)
 {
+  //note that 'cmake' may not able to find correct value of HAVE_PLPLOT_WIDTH. Please report.
+  // in the meantime, you may edit "config.h" by hand.
 #ifdef HAVE_PLPLOT_WIDTH
     plstream::width(static_cast<PLFLT>(thick*thickFactor));
 #else
@@ -83,7 +85,7 @@ void GDLGStream::SetColorMap1SingleColor( ULong color)
     red[0] =red[1] = color & 0xFF;
     green[0] = green[1] =(color >> 8)  & 0xFF;
     blue[0]= blue[1]=(color >> 16) & 0xFF;
-    plstream::scmap1(red, green, blue, 2); 
+    SetColorMap1(red, green, blue, 2); 
 }
 
 void GDLGStream::SetColorMap1DefaultColors(PLINT ncolors, DLong decomposed)
@@ -91,15 +93,15 @@ void GDLGStream::SetColorMap1DefaultColors(PLINT ncolors, DLong decomposed)
   if (decomposed == 0) { //just copy Table0 to Table1 so that scale from 0 to 1 in table 1 goes through the whole table
     PLINT r[ctSize], g[ctSize], b[ctSize];
     GraphicsDevice::GetCT()->Get( r, g, b); 
-    plstream::scmap1(r, g, b, ctSize); 
+    SetColorMap1(r, g, b, ctSize); 
   } else {
     PLFLT r[2], g[2], b[2], pos[2];
     r[0] = pos[0] = 0.0;
     r[1] = pos[1] = 1.0;
     g[0] = g[1] = 0.0;
     b[0] = b[1] = 0.0;
-    plstream::scmap1n(ncolors);
-    plstream::scmap1l(TRUE,2,pos,r, g, b, NULL); 
+    SetColorMap1n(ncolors);
+    SetColorMap1l(TRUE,2,pos,r, g, b, NULL); 
   }
 }
 
@@ -132,7 +134,7 @@ void GDLGStream::SetColorMap1Table( PLINT tableSize, BaseGDL *passed_colors,  DL
       b[i] = (col >> 16) & 0xFF;   
      }
   }
-  plstream::scmap1(r, g, b, tableSize); 
+  SetColorMap1(r, g, b, tableSize); 
 }
 
 void GDLGStream::SetColorMap1Ramp(DLong decomposed, PLFLT minlight)
@@ -141,14 +143,14 @@ void GDLGStream::SetColorMap1Ramp(DLong decomposed, PLFLT minlight)
     h[0] = h[1] = s[0] = s[1] = pos[0] = 0.0;
     l[0] = minlight;
     l[1] = pos[1] = 1.0;
-    plstream::scmap1n(256);
-    plstream::scmap1l(FALSE,2,pos,h, l, s, NULL); 
+    SetColorMap1n(256);
+    SetColorMap1l(FALSE,2,pos,h, l, s, NULL); 
 }
 #define WHITEB 255
 void GDLGStream::Background( ULong color, DLong decomposed)
 {
   if ((*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("FLAGS"), 0)))[0] & 512 ) {  ;//printer like PostScript
-   plstream::scolbg( WHITEB, WHITEB, WHITEB );
+      GraphicsDevice::GetDevice()->SetDeviceBckColor(WHITEB, WHITEB, WHITEB );
    return;
   }
   DByte r,g,b;
@@ -161,7 +163,27 @@ void GDLGStream::Background( ULong color, DLong decomposed)
     green = (color >> 8)  & 0xFF;
     blue = (color >> 16) & 0xFF;
   }
-  plstream::scolbg( red, green, blue); //set background (col0 in plplot)
+  GraphicsDevice::GetDevice()->SetDeviceBckColor( red, green, blue);
+}
+void GDLGStream::DefaultBackground()
+{
+  if ((*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("FLAGS"), 0)))[0] & 512 ) {  ;//printer like PostScript
+    GraphicsDevice::GetDevice()->SetDeviceBckColor(WHITEB, WHITEB, WHITEB );
+    return;
+  }
+  DStructGDL* pStruct=SysVar::P();   //MUST NOT BE STATIC, due to .reset 
+  DLong background=(*static_cast<DLongGDL*>(pStruct->GetTag(pStruct->Desc()->TagIndex("BACKGROUND"), 0)))[0];
+  DByte r,g,b;
+  PLINT red,green,blue;
+  if (GraphicsDevice::GetDevice()->GetDecomposed() == 0) { //just an index
+    GraphicsDevice::GetCT()->Get( background & 0xFF, r, g, b);
+    red=r; green=g; blue=b;
+  } else {
+    red = background & 0xFF;
+    green = (background >> 8)  & 0xFF;
+    blue = (background >> 16) & 0xFF;
+  }
+  GraphicsDevice::GetDevice()->SetDeviceBckColor( red, green, blue);
 }
 #undef WHITEB
 void GDLGStream::DefaultCharSize()
@@ -169,18 +191,20 @@ void GDLGStream::DefaultCharSize()
   DString name = (*static_cast<DStringGDL*>(
     SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("NAME"), 0)
   ))[0];
-
-  if (name == "PS" || name=="SVG") schr( 3.5, 1.0);
+//values must be those that plplot think are good. Most of the time they are not.
+  if (name == "PS" || name=="SVG") schr( 2.5, 1.0);
   else 
 #if defined(_WIN32)
     schr(2.1, 1.4);  // from 1.5, 1.0 2014/09/18 //This is a feature of windows --- or of windows plplot -- to be confirmed.
 #else
-    schr(1.5, 1.0);
+    schr(1.5, 1.0); //is 6 pixels because plplot supposes 4ppm, which is not always true and never exact.
+    //Note that IDL 1) write strings of characters a bit wider than plplot and 2) also wider than one would suppose
+    //based on the value of !D.X_CH_SIZE and !D.Y_CH_SIZE
 #endif
   (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("X_CH_SIZE"), 0)))[0]=
-  theCurrentChar.dsx;
+  ceil(theCurrentChar.dsx);
   (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("Y_CH_SIZE"), 0)))[0]=
-  theCurrentChar.dsy;
+  ceil(theCurrentChar.dsx)*10.0/6.0;
 }
 
 void GDLGStream::NextPlot( bool erase )
@@ -208,7 +232,21 @@ void GDLGStream::NextPlot( bool erase )
     if( erase )
     {
       eop();           // overridden (for Z-buffer)
-      plstream::bop(); // changes charsize
+      //get background value (*not pen 0*, we try to avoid plplot's silly behaviour).
+      //use it for bop(), then reset the pen 0 to correct value.
+
+      PLINT red,green,blue;
+      DByte r,g,b;
+      PLINT red0,green0,blue0;
+      
+      GraphicsDevice::GetCT()->Get(0,r,g,b);red=r;green=g;blue=b;
+      
+      red0=GraphicsDevice::GetDevice()->BackgroundR();
+      green0=GraphicsDevice::GetDevice()->BackgroundG();
+      blue0=GraphicsDevice::GetDevice()->BackgroundB();
+      plstream::scolbg(red0,green0,blue0); //overwrites col[0]
+      plstream::bop(); // note: changes charsize
+      plstream::scolbg(red,green,blue); //resets col[0]
     }
 
 //    plstream::adv(1); //advance to first subpage
@@ -249,7 +287,7 @@ void GDLGStream::NoSub()
 
 
 // default is a wrapper for gpage(). Is overriden by, e.g., X driver.
-void GDLGStream::GetGeometry( long& xSize, long& ySize, long& xoff, long& yoff)
+void GDLGStream::GetGeometry( long& xSize, long& ySize)
 {
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"GDLGStream::GetGeometry()\n");
   PLFLT xp; PLFLT yp; 
@@ -266,13 +304,9 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize, long& xoff, long& yoff)
   if (name == "PS") { 
     xSize = (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("X_SIZE"), 0)))[0];
     ySize = (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("Y_SIZE"), 0)))[0];
-    xoff = 0;
-    yoff = 0;
   } else {
     xSize = xleng;
     ySize = yleng;
-    xoff = plxoff;
-    yoff = plyoff;
   }
   if (xSize<1.0||ySize<1) //plplot gives back crazy values! z-buffer for example!
   {
@@ -280,10 +314,8 @@ void GDLGStream::GetGeometry( long& xSize, long& ySize, long& xoff, long& yoff)
     plstream::gspa(xmin,xmax,ymin,ymax); //subpage in mm
     xSize=min(1.0,xmax-xmin);
     ySize=min(1.0,ymax-ymin);
-    xoff=0.0;
-    yoff=0.0;
   }
-  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"    found (%ld %ld %ld %ld)\n", xSize, ySize, xoff, yoff);
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"    found (%ld %ld)\n", xSize, ySize);
 
 }
 
@@ -441,7 +473,7 @@ bool GDLGStream::TranslateFormatCodes(const char *in, std::string & out)
           curr_lev--;
           out += "#d";
           // continues!
-        case 'D' : case 'd' : // subscript
+        case 'B' : case 'b' : case 'D' : case 'd' : // subscript
         case 'I' : case 'i' : // index
           curr_lev--;
           out += "#d";
@@ -449,10 +481,10 @@ bool GDLGStream::TranslateFormatCodes(const char *in, std::string & out)
         case 'R' : case 'r' : // restore position
           for (; save_pos < curr_pos; curr_pos--) out += "#b";
           // continues!
-        case 'A' : case 'a' : // shift above the division line
-          //TODO: plplot seems not to support it 
-        case 'B' : case 'b' : // shift below the division line
-          //TODO: plplot seems not to support it
+        //case 'A' : case 'a' : // shift above the division line
+          //TODO: plplot seems not to support it ----> treated as upperscript for the moment
+       // case 'B' : case 'b' : // shift below the division line
+          //TODO: plplot seems not to support it ----> treated as subscript for the moment
         case 'N' : case 'n' : // back to normal size
           while (curr_lev != 0) 
           {
@@ -462,7 +494,7 @@ bool GDLGStream::TranslateFormatCodes(const char *in, std::string & out)
           // assumed from examples in documentation
           if (in[i + 1] == 'N' || in[i + 1] == 'n') out += fonts[curr_fnt = next_fnt = default_fnt];
           break;
-        case 'U' : case 'u' : // superscript
+        case 'A' : case 'a' : case 'U' : case 'u' : // superscript
         case 'E' : case 'e' : // exponent
           curr_lev++;
           out += "#u";
@@ -855,6 +887,14 @@ retrn:
   return true; 
 }
 
+void GDLGStream::setSymbolSize( PLFLT scale )
+{
+  if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"setSymbolScale(%f)\n",scale);
+  plstream::ssym(0.0, scale);
+  theCurrentSymSize=scale;
+}
+PLFLT GDLGStream::getSymbolSize(){return theCurrentSymSize;}
+
 void GDLGStream::mtex( const char *side, PLFLT disp, PLFLT pos, PLFLT just,
                        const char *text)
 {
@@ -887,6 +927,8 @@ void GDLGStream::sizeChar( PLFLT scale )
 void GDLGStream::vpor(PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
 {
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"vpor(): requesting x[%f:%f],y[%f:%f] (normalized, subpage)\n",xmin,xmax,ymin,ymax);
+  //note that plplot apparently does not write the y=0 line of pixels (in device coords). IDL page is on the contrary limited to
+  // [0..1[ in both axes (normalized coordinates)
   plstream::vpor(xmin, xmax, ymin, ymax);
   theBox.nx1=xmin;
   theBox.nx2=xmax;
@@ -939,3 +981,43 @@ void GDLGStream::adv(PLINT page)
   if (thePage.curPage > thePage.nbPages) thePage.curPage=1;
   if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"adv() now at page %d\n",thePage.curPage);
 }
+//get region (3BPP data)
+bool GDLGStream::GetRegion(DLong& x_gdl, DLong& y_gdl, DLong& nx_gdl, DLong& ny_gdl){
+    DByteGDL *bitmap = static_cast<DByteGDL*>(this->GetBitmapData());
+    if (bitmap==NULL)  return false; //need to GDLDelete bitmap on exit after this line.
+
+    bool error=false;
+    DLong nx=bitmap->Dim(0);
+    DLong ny=bitmap->Dim(1);
+    
+    DLong xref,xval,xinc,yref,yval,yinc,xmax11,ymin11;
+    long x_11=0;
+    long y_11=0;
+    xref=0;xval=0;xinc=1;
+    yref=0;yval=0;yinc=1;
+    
+    x_11=xval+(x_gdl-xref)*xinc;
+    y_11=yval+(y_gdl-yref)*yinc;
+    xmax11=xval+(x_gdl+nx_gdl-1-xref)*xinc;    
+    ymin11=yval+(y_gdl+ny_gdl-1-yref)*yinc;
+    if (y_11 < 0 || y_11 > ny-1) error=true;
+    if (x_11 < 0 || x_11 > nx-1) error=true;
+    if (xmax11 < 0 || xmax11 > nx-1) error=true;
+    if (ymin11 < 0 || ymin11 > ny-1) error=true;
+    if (error) {  GDLDelete(bitmap); return false; }
+    GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
+    unsigned char* data=actDevice->SetCopyBuffer(nx_gdl*ny_gdl*3);  
+    for ( SizeT i =0; i < nx_gdl ; ++i ) {
+      for ( SizeT j = 0; j < ny_gdl ; ++j ) {
+       for ( SizeT k = 0 ; k < 3 ; ++k) data[3 * (j * nx_gdl + i) + k] = (*bitmap)[3 * ((j+y_11) * nx + (i+x_11)) + k]; 
+      }
+    }
+    GDLDelete(bitmap);
+    return true;
+}
+
+bool GDLGStream::SetRegion(DLong& xs, DLong& ys, DLong& nx, DLong& ny){
+  DLong pos[4]={xs,nx,ys,ny};
+  GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
+  return this->PaintImage(actDevice->GetCopyBuffer(), nx, ny, pos, 1, 0);  
+}
