@@ -25,14 +25,12 @@
 #define GDL_DEBUG_PLSTREAM 0
 #endif
 
-#include <plplot/plstream.h>
-#include <plplot/plstrm.h>
-
-
 #ifdef HAVE_CONFIG_H
 // we should not add all the plplot's internal defines (which are 
 // added if HAVE_CONFIG_H is defined) to ours. Makes problem with distros.
 #undef HAVE_CONFIG_H 
+#include <plplot/plstream.h>
+#include <plplot/plstrm.h>
 #include <plplot/plplot.h>
 #define HAVE_CONFIG_H 1
 #endif
@@ -45,6 +43,8 @@
 #include "typedefs.hpp"
 #include "gdlexception.hpp"
 
+#include "initsysvar.hpp"
+
 #ifdef _MSC_VER
 #include <algorithm>
 #endif
@@ -54,30 +54,30 @@ const double MMToINCH = 0.039370078 ; // 1./2.54;
 using namespace std;
 
 // Graphic Structures:
-  typedef struct _P_GRAPHICS {
-    DLong background;
-    DFloat charSize;
-    DFloat charThick;
-    DLong clip[6];
-    DLong color;
-    DLong font;
-    DLong lineStyle;
-    DLong multi[5];
-    DLong noClip;
-    DLong noErase;
-    DLong nsum;
-    DFloat position[4];
-    DLong psym;
-    DFloat region[4];
-    DString subTitle;
-    DFloat symSize;
-    DDouble t[4][4];
-    DLong t3d;
-    DFloat thick;
-    DString title;
-    DFloat ticklen;
-    DLong channel;
-  } pstruct ;
+//  typedef struct _P_GRAPHICS {
+//    DLong background;
+//    DFloat charSize;
+//    DFloat charThick;
+//    DLong clip[6];
+//    DLong color;
+//    DLong font;
+//    DLong lineStyle;
+//    DLong multi[5];
+//    DLong noClip;
+//    DLong noErase;
+//    DLong nsum;
+//    DFloat position[4];
+//    DLong psym;
+//    DFloat region[4];
+//    DString subTitle;
+//    DFloat symSize;
+//    DDouble t[4][4];
+//    DLong t3d;
+//    DFloat thick;
+//    DString title;
+//    DFloat ticklen;
+//    DLong channel;
+//  } pstruct ;
 
   typedef struct GDL_BOX {
     bool initialized;
@@ -138,6 +138,8 @@ using namespace std;
     DDouble mmsy; //
     PLFLT wsx;  //in current world coordinates
     PLFLT wsy;
+    PLFLT convx; //symbol size conversion factor, (for PSYMs, not characters: PSYMs are handled by GDL, CHARS by plplot)
+    PLFLT convy; //set only while in 2D mode, used in 2D or 3D mode.
   } gdlCharInfo;
 
 class GDLGStream: public plstream
@@ -157,18 +159,26 @@ protected:
   gdlpage thePage;
   PLStream* pls;
   DFloat thickFactor;
-  
+  PLFLT theCurrentSymSize;
+  bool usedAsPixmap; //for WINDOW,/PIXMAP retains the fact that this is a pixmap (invisible) window.
 public:
 
    GDLGStream( int nx, int ny, const char *driver, const char *file=NULL)
-    : plstream( nx, ny, driver, file), valid( true), thickFactor(1.0)
+    : plstream( nx, ny, driver, file), valid( true), thickFactor(1.0), usedAsPixmap(false)
   {
     if (!checkPlplotDriver(driver))
       ThrowGDLException(std::string("PLplot installation lacks the requested driver: ") + driver);
     gdlDefaultCharInitialized=0;
     thePage.nbPages=0;
+    thePage.length=0;
+    thePage.height=0;
+    thePage.plxoff=0;
+    thePage.plyoff=0;
     theBox.initialized=false;
     plgpls( &pls);
+    //you can debug plplot things with
+    //    pls->verbose=1;
+
 	if (GDL_DEBUG_PLSTREAM) printf(" new GDLGstream( %d , %d , %s ):pls=%p \n", nx, ny, driver, (void *)pls);
 
   }
@@ -198,11 +208,17 @@ public:
       for (int maxnumdevs = numdevs_plus_one;; numdevs_plus_one = maxnumdevs += 16)
       {
 #ifdef HAVE_OLDPLPLOT
-        devlongnames = static_cast<char**>(realloc(devlongnames, maxnumdevs * sizeof(char*)));
-        devnames = static_cast<char**>(realloc(devnames, maxnumdevs * sizeof(char*)));
+        //handles gracefully the improbable failure of realloc
+        void* tmp = realloc(devlongnames, maxnumdevs * sizeof(char*));
+        if (tmp) devlongnames = static_cast<char**>(tmp); else return false;
+        tmp = realloc(devnames, maxnumdevs * sizeof(char*));
+        if (tmp) devnames = static_cast<char**>(tmp); else return false;
 #else
-        devlongnames = static_cast<const char**>(realloc(devlongnames, maxnumdevs * sizeof(char*)));
-        devnames = static_cast<const char**>(realloc(devnames, maxnumdevs * sizeof(char*)));
+        //handles gracefully the improbable failure of realloc
+        void* tmp = realloc(devlongnames, maxnumdevs * sizeof(char*));
+        if (tmp) devlongnames = static_cast<const char**>(tmp); else return false;
+        tmp = realloc(devnames, maxnumdevs * sizeof(char*));
+        if (tmp) devnames = static_cast<const char**>(tmp); else return false;
 #endif
         plgDevs(&devlongnames, &devnames, &numdevs_plus_one);
         numdevs_plus_one++;
@@ -217,9 +233,6 @@ public:
       free(devnames);
     }
 
-    // for debug
-    std::vector<std::string> devnamesDbg = devNames;
-
     return std::find( devNames.begin(), devNames.end(), std::string( driver)) != devNames.end();
   }
 
@@ -228,14 +241,17 @@ public:
   virtual void Init()=0;
 
   // called after draw operation
-  virtual void Update() {}
+  //virtual void Update() {}
+  virtual void Update(){plstream::cmd(PLESC_EXPOSE, NULL);}
   
   virtual void EventHandler() {}
 
-  virtual void GetGeometry( long& xSize, long& ySize, long& xoff, long& yoff);
+  virtual void GetGeometry( long& xSize, long& ySize);
   virtual unsigned long GetWindowDepth () {return 0;}
   virtual DLong GetVisualDepth() {return -1;}
   virtual DString GetVisualName() {return "";}
+  virtual BaseGDL* GetFontnames(DString pattern) {return NULL;}
+  virtual DLong GetFontnum(DString pattern) {return -1;}
   virtual bool UnsetFocus(){return false;}
   virtual bool SetBackingStore(int value){return false;}
   virtual bool SetGraphicsFunction(long value ){return false;}
@@ -245,7 +261,7 @@ public:
   virtual void eop()          { plstream::eop();}
   virtual void SetDoubleBuffering() {}
   virtual void UnSetDoubleBuffering() {}
-  virtual bool HasDoubleBuffering() {return false;}
+  virtual bool HasDoubleBuffering() {return pls->db;}
   virtual bool HasSafeDoubleBuffering() {return false;}
   virtual void Raise()         {}
   virtual void Lower()        {}
@@ -257,8 +273,13 @@ public:
   virtual void Clear()         {}
   virtual void Clear( DLong chan)          {}
   virtual bool PaintImage(unsigned char *idata, PLINT nx, PLINT ny, DLong *pos, DLong tru, DLong chan){return false;}
-  virtual bool HasImage(){return false;}
   virtual bool HasCrossHair() {return false;}
+  virtual void UnMapWindow() {usedAsPixmap=true;} 
+  bool IsPixmapWindow() {return usedAsPixmap;}
+  virtual BaseGDL* GetBitmapData(){return NULL;}
+  virtual void SetCurrentFont(std::string fontname){}//do nothing
+  bool GetRegion(DLong& xs, DLong& ys, DLong& nx, DLong& ny);//{return false;}
+  bool SetRegion(DLong& xd, DLong& yd, DLong& nx, DLong& ny);//{return false;}
 
   virtual void CheckValid() {}
   void SetValid( bool v) { valid = v;}
@@ -273,8 +294,18 @@ public:
     if (((theBox.nx1==0) && (theBox.nx2==0)) 
 	|| ((theBox.ny1==0) && (theBox.ny2==0))) return false; else return true;
   }
+  inline PLFLT getPsymConvX(){return theCurrentChar.convx;}
+  inline PLFLT getPsymConvY(){return theCurrentChar.convy;}
+  inline void setSymbolSizeConversionFactors(){
+    PLFLT symsize=this->getSymbolSize();
+    theCurrentChar.convx=(0.5*symsize*(this->wCharLength()/this->charScale())); //be dependent only on symsize!
+    theCurrentChar.convy=(0.5*symsize*(this->wCharHeight()/this->charScale()));
+    PLFLT wun, wdeux, wtrois, wquatre; //take care of axes world orientation!
+    this->pageWorldCoordinates(wun, wdeux, wtrois, wquatre);
+    if ((wdeux-wun)<0) theCurrentChar.convx*=-1.0;
+    if ((wquatre-wtrois)<0) theCurrentChar.convy*=-1.0;} 
   inline PLFLT charScale(){return theCurrentChar.scale;}
-  inline PLFLT nCharLength(){return theCurrentChar.ndsx;}
+  inline PLFLT nCharWidth(){return theCurrentChar.ndsx;}
   inline PLFLT nCharHeight(){return theCurrentChar.ndsy;}
   inline PLFLT dCharLength(){return theCurrentChar.dsx;}
   inline PLFLT dCharHeight(){return theCurrentChar.dsy;}
@@ -579,12 +610,15 @@ public:
   {
     if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"updatePageInfo():\n");
     if (thePage.nbPages==0) {if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"            FAILED\n");return false;}
-    long xsize,ysize,xoff,yoff;
-    GetGeometry(xsize,ysize,xoff,yoff);
+    long xsize,ysize;
+    GetGeometry(xsize,ysize);
+    if (thePage.length==xsize && thePage.height==ysize ) return true;
     thePage.length=xsize;
     thePage.height=ysize;
-    thePage.plxoff=xoff;
-    thePage.plyoff=yoff;
+    (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("X_SIZE"), 0)))[0] = xsize;
+    (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("Y_SIZE"), 0)))[0] = ysize;
+    (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("X_VSIZE"), 0)))[0] = xsize;
+    (*static_cast<DLongGDL*>(SysVar::D()->GetTag(SysVar::D()->Desc()->TagIndex("Y_VSIZE"), 0)))[0] = ysize;
     if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"             %fx%f device units.\n",thePage.length, thePage.height);
     return true;
   }
@@ -689,8 +723,9 @@ public:
   }
   
   void Thick( DFloat thick);
-  void Color( ULong c, DLong decomposed=0);
+  virtual void Color( ULong c, DLong decomposed=0);
   void Background( ULong c, DLong decomposed=0);
+  void DefaultBackground();
   void SetColorMap1SingleColor( ULong color);
   //if decomposed, create a red ramp. If not, copy the colormap0 in colormap1
   void SetColorMap1DefaultColors(PLINT ncolors, DLong decomposed=0);
@@ -738,63 +773,40 @@ public:
                                     theCurrentChar.wsx,theCurrentChar.wsy);
   }
   
-  void RenewPlplotDefaultCharsize(PLFLT newMmSize)
+  inline void RenewPlplotDefaultCharsize(PLFLT newMmSize)
   {
-    PLFLT oldsize=theDefaultChar.mmsy;
     plstream::schr(newMmSize, 1.0);
-    PLFLT fact=newMmSize/oldsize; 
-    PLFLT aspect=theDefaultChar.ndsx/theDefaultChar.ndsy; // width/height
-    theDefaultChar.mmsy=newMmSize;
-    theDefaultChar.mmsx=aspect*newMmSize; //since aspect do not change
-    theDefaultChar.ndsx*=(fact*aspect);
-    theDefaultChar.ndsy*=fact;
-    theDefaultChar.dsx*=(fact*aspect);
-    theDefaultChar.dsy*=fact;
-    gdlDefaultCharInitialized=1;
-    CurrentCharSize(1.0);
+    gdlDefaultCharInitialized=0;
+    GetPlplotDefaultCharSize();
   }
   
   void GetPlplotDefaultCharSize()
   {
+        
     if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"GetPlPlotDefaultCharsize()\n");
     if (thePage.nbPages==0)   {return;}
     //dimensions in normalized, device and millimetres
     if (gdlDefaultCharInitialized==1) {if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"     Already initialized\n"); return;}
-
-    PLFLT nxmin, nxmax, nymin, nymax, wxmin, wxmax, wymin, wymax;
-    plstream::gvpd(nxmin, nxmax, nymin, nymax); //save norm of current box
-    if((nxmin==0.0&&nxmax==0.0)||(nymin==0.0&&nymax==0.0)) //if not initialized, set normalized mode
-    {
-    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"          Warning: initializing viewport\n");
-      plstream::vpor(0, 1, 0, 1);
-      plstream::gvpd(nxmin, nxmax, nymin, nymax);
-      plstream::wind(0.0,1.0,0.0,1.0);
-    }
-    plstream::gvpw(wxmin, wxmax, wymin, wymax); //save world of current box
-    PLFLT vpXmin, vpXmax, vpYmin, vpYmax;
-    PLFLT vpXmin2, vpXmax2, vpYmin2, vpYmax2;
-    plstream::vpor(0, 1, 0, 1);
-    plstream::wind(0.0,1.0,0.0,1.0);
-    plstream::gvpd(vpXmin, vpXmax, vpYmin, vpYmax);
-    plstream::vsta();
-    plstream::gvpd(vpXmin2, vpXmax2, vpYmin2, vpYmax2);
-    theDefaultChar.ndsx=0.5*((vpXmin2-vpXmin)/8.0+(vpXmax-vpXmax2)/5.0);
-    theDefaultChar.ndsy=0.5*((vpYmin2-vpYmin)/5.0+(vpYmax-vpYmax2)/5.0);
-    theDefaultChar.dsx=0.5*((vpXmin2-vpXmin)/8.0+(vpXmax-vpXmax2)/5.0)*thePage.length;
-    theDefaultChar.dsy=0.5*((vpYmin2-vpYmin)/5.0+(vpYmax-vpYmax2)/5.0)*thePage.height;
-    plstream::vpor(nxmin, nxmax, nymin, nymax); //restore norm of current box
-    plstream::wind(wxmin, wxmax, wymin, wymax); //restore world of current box
-    PLFLT defhmm, scalhmm;
-    plgchr(&defhmm, &scalhmm); // height of a letter in millimetres
-    theDefaultChar.mmsy=scalhmm;
-    theDefaultChar.mmsx=theDefaultChar.ndsx/theDefaultChar.ndsy*scalhmm;
+    theDefaultChar.scale=1.0;
+    theDefaultChar.mmsx=pls->chrht; //millimeter
+    theDefaultChar.mmsy=pls->chrht;
+    theDefaultChar.ndsx=mm2ndx(theDefaultChar.mmsx); //normalized device
+    theDefaultChar.ndsy=mm2ndy(theDefaultChar.mmsy);
+    theDefaultChar.dsy=theDefaultChar.ndsy*thePage.height;
+    theDefaultChar.dsx=theDefaultChar.ndsx*thePage.length;
+    theDefaultChar.wsx=mm2wx(theDefaultChar.mmsx); //world
+    theDefaultChar.wsy=mm2wy(theDefaultChar.mmsy);
     if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"             %fx%f(mm)\n",theDefaultChar.mmsx,theDefaultChar.mmsy);
     if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"             %fx%f(norm)\n",theDefaultChar.ndsx,theDefaultChar.ndsy);
+    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"             %fx%f(dev)\n",theDefaultChar.dsx,theDefaultChar.dsy);
+    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"             %fx%f(world)\n",theDefaultChar.wsx,theDefaultChar.wsy);
     gdlDefaultCharInitialized=1;
   }
   // SA: overloading plplot methods in order to handle IDL-plplot extended
   // text formating syntax conversion
   bool TranslateFormatCodes(const char *text, std::string &out);
+  void setSymbolSize( PLFLT scale );
+  PLFLT getSymbolSize();
   void mtex( const char *side, PLFLT disp, PLFLT pos, PLFLT just,
                          const char *text);
   void ptex( PLFLT x, PLFLT y, PLFLT dx, PLFLT dy, PLFLT just,
@@ -806,20 +818,6 @@ public:
   void wind( PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax );
   void ssub( PLINT nx, PLINT ny);
   void adv(PLINT page);
-  void gpage(PLFLT& xp, PLFLT& yp, PLINT& xleng, PLINT& yleng,
-                PLINT& xoff, PLINT& yoff)
-  {
-    if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"gpage() [%f,%f]\n",xp,yp);
-    if(updatePageInfo()==true)
-    {
-        xp=thePage.xdpmm/MMToINCH;
-        yp=thePage.ydpmm/MMToINCH;
-        xleng=(PLINT)thePage.length;
-        yleng=(PLINT)thePage.height;
-        xoff=(PLINT)thePage.plxoff;
-        yoff=(PLINT)thePage.plyoff;
-    }
-  }
 
   inline void syncPageInfo()
   {
@@ -843,7 +841,7 @@ public:
         thePage.xsizemm=thePage.length/thePage.xdpmm;
         thePage.ysizemm=thePage.height/thePage.ydpmm;
         if (GDL_DEBUG_PLSTREAM) fprintf(stderr,"         device resolution [%f,%f]mm^-1, device size [%f,%f], [%f,%f] mm\n",
-                thePage.xdpmm,thePage.ydpmm,thePage.length,thePage.height,thePage.xsizemm,thePage.ysizemm);
+         thePage.xdpmm,thePage.ydpmm,thePage.length,thePage.height,thePage.xsizemm,thePage.ysizemm);
         thePage.subpage.dxoff=offx_mm*thePage.xdpmm;
         thePage.subpage.dyoff=offy_mm*thePage.ydpmm;
         thePage.subpage.dxsize=bxsize_mm*thePage.xdpmm;
@@ -871,6 +869,26 @@ public:
       thickFactor=tf;
   }
 
+  //GD: enables overloading scmap0,1... to accelerate plots for X11 and possibly others
+  // Set color map 0 colors by 8 bit RGB values
+  virtual void SetColorMap0( const PLINT *r, const PLINT *g, const PLINT *b, PLINT ncol0 ) {
+   plstream::scmap0( r, g, b, ncol0);
+  }
+  // Set color map 1 colors by 8 bit RGB values
+  virtual void SetColorMap1( const PLINT *r, const PLINT *g, const PLINT *b, PLINT ncol1 ) {
+   plstream::scmap1( r, g, b, ncol1);
+  }
+  // Set color map 1 colors using a piece-wise linear relationship between
+  // intensity [0,1] (cmap 1 index) and position in HLS or RGB color space.
+  virtual void SetColorMap1l( bool itype, PLINT npts, const PLFLT *intensity, const PLFLT *coord1, const PLFLT *coord2, const PLFLT *coord3, const bool *rev = NULL ) {
+   plstream::scmap1l(itype,npts,intensity,coord1,coord2,coord3,rev);
+  }
+  // Set number of colors in cmap 1
+  virtual void SetColorMap1n( PLINT ncol1 ){
+   plstream::scmap1n(ncol1);
+  }
+
+
 };
 
 #endif
