@@ -34,10 +34,15 @@ void GDLXStream::Init() {
   {
     Display* display = XOpenDisplay(NULL);
     if (display == NULL) {
+      display = XOpenDisplay(":0");//IDL also opens :0 when DISPLAY is not set.
+      if (display == NULL) {
       valid = false;
       ThrowGDLException("Cannot connect to X server");
+      }
     }
     XGetInputFocus(display, &term_window, &revert_to);
+    //avoid doing impossible things if term_window is *not* a bona fide window.
+    if (term_window == PointerRoot || term_window == None) term_window=0;
     XCloseDisplay(display);
   }
 
@@ -58,7 +63,7 @@ void GDLXStream::Init() {
   }
   if ( okToRevertToTerminal ) XSetInputFocus(xwd->display,term_window,RevertToParent,CurrentTime);
   else UnsetFocus(); //desperate method, since it prevents iconifying etc...
-  XFlush(xwd->display);
+//  XFlush(xwd->display);
   GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
   //current cursor:
   CursorStandard(actDevice->getCursorId());
@@ -93,6 +98,7 @@ void GDLXStream::EventHandler() {
     if (event.xclient.message_type == wm_protocols &&
             event.xclient.data.l[0] == wm_delete_window) {
       valid = false;
+      GraphicsDevice::GetDevice()->TidyWindowsList(); //necessary since we removed TidyWindowList() from GraphicsMultiDevice::EventHandler()
       return; // no more event handling
     } else
       XPutBackEvent(xwd->display, &event);
@@ -124,39 +130,11 @@ bool GDLXStream::GetWindowPosition(long& xpos, long& ypos ) {
   } else return false;
 }
 
-void GDLXStream::GetGeometry(long& xSize, long& ySize, long& xOffset, long& yOffset) {
-
+void GDLXStream::GetGeometry(long& xSize, long& ySize) {
+//for some reason, with the x11 driver, this is better than plstream::gpage to get actual sizes.
   XwDev *dev = (XwDev *) pls->dev;
-  XwDisplay *xwd = (XwDisplay *) dev->xwd;
-  XWindowAttributes wa;
-  /* query the window's attributes. */
-  if (XGetWindowAttributes(xwd->display, dev->window, &wa)){
-    int addx,addy;
-    Window child;
-    XTranslateCoordinates( xwd->display, dev->window, wa.root, 0, 0, &addx, &addy, &child );
-    xSize=wa.width;
-    ySize=wa.height;
-    xOffset = addx-wa.x;
-    yOffset = dev->height-addy+wa.y+1;
-  } else GDLGStream::GetGeometry(xSize, ySize, xOffset, yOffset);
-  
-  int debug=0;
-  if (debug) {
-    int screen_num, screen_width, screen_height;
-    screen_num = DefaultScreen(xwd->display);
-    screen_width = DisplayWidth(xwd->display, screen_num);
-    screen_height = DisplayHeight(xwd->display, screen_num);
-    cout << "---- Begin Inside GetX11Geometry ----" << endl;
-    cout << "display size : " << screen_width << " " << screen_height << endl;
-    cout << "win_attributes W/H: " << wa.width << " " << wa.height << endl;
-    cout << "win_attributes border width: " << wa.border_width << endl;
-    cout << "win_attributes X/Y: " << wa.x << " " << wa.y << endl;
-    cout << "results: " << endl;
-    cout << "xSize/ySize: " << xSize << " " << ySize << endl;
-    cout << "xOffset/yOffset: " << xOffset << " " << yOffset << endl;
-    cout << "---- End Inside GetX11Geometry ----" << endl;
-  }
-
+  xSize=dev->width;
+  ySize=dev->height;
 }
 
   bool GDLXStream::CursorStandard(int cursorNumber)
@@ -167,36 +145,65 @@ void GDLXStream::GetGeometry(long& xSize, long& ySize, long& xOffset, long& yOff
     XDefineCursor(xwd->display,dev->window,XCreateFontCursor(xwd->display,num));
     return true;
   }
-  
+
 // plplot 5.3 does not provide the clear function for c++
 
 void GDLXStream::Clear() {
-  // dummy call to get private function set_stream() called
-  // gd: tested, fail to see why use this dummy call ?
-//  char dummy;
-//  gesc(&dummy);
-  // this mimics better the *DL behaviour but plbob create a new page, etc..
-  ::c_plbop();
-  //plclear clears only the current subpage. But it clears it. One has
-  //just to set the number of subpages to 1
-  ::c_plclear();
+  Clear(-1);
+//  // this mimics better the *DL behaviour but plbop create a new page, etc..
+//  //plclear clears only the current subpage. But it clears it. One has
+//  //just to set the number of subpages to 1
+//  PLINT red, green, blue;
+//  DByte r, g, b;
+//  PLINT red0, green0, blue0;
+//
+//  GraphicsDevice::GetCT( )->Get( 0, r, g, b );
+//  red = r;
+//  green = g;
+//  blue = b;
+////we get around the index 0=background color "feature" of plplot. GDL uses a separate backgroud color.
+//  red0 = GraphicsDevice::GetDevice( )->BackgroundR( );
+//  green0 = GraphicsDevice::GetDevice( )->BackgroundG( );
+//  blue0 = GraphicsDevice::GetDevice( )->BackgroundB( );
+//  plstream::scolbg( red0, green0, blue0 ); //overwrites col[0]
+//  ::c_plbop( );
+////  ::c_plclear( );
+//  plstream::scolbg( red, green, blue ); //resets col[0]
 }
 
 void GDLXStream::Clear(DLong chan) {
-  static const int planemask[3]={0x0000FF,0x00FF00,0xFF0000};
-//fill screen with background (since plotting erase has called background) on channel chan
   XwDev *dev = (XwDev *) pls->dev;
   XwDisplay *xwd = (XwDisplay *) dev->xwd;
-  XSetForeground(xwd->display,dev->gc,xwd->cmap0[0].pixel);
-  XSetPlaneMask(xwd->display,dev->gc,planemask[chan]);
-  if (dev->write_to_pixmap)
+
+  DByte r = (GraphicsDevice::GetDevice()->BackgroundR());
+  DByte g = (GraphicsDevice::GetDevice()->BackgroundG());
+  DByte b = (GraphicsDevice::GetDevice()->BackgroundB());
+  unsigned long curcolor= 
+    ((unsigned long)(r) << (ffs(xwd->visual->red_mask)-1) ) +
+    ((unsigned long)(g) << (ffs(xwd->visual->green_mask)-1) ) +
+    ((unsigned long)(b) << (ffs(xwd->visual->blue_mask)-1) );
+
+  XSetForeground( xwd->display, dev->gc, curcolor); // myColor.pixel );
+
+  switch(chan){
+      case 0:
+        XSetPlaneMask(xwd->display,dev->gc,xwd->visual->red_mask);
+        break;
+      case 1:
+        XSetPlaneMask(xwd->display,dev->gc,xwd->visual->green_mask);
+        break;
+      case 2:
+        XSetPlaneMask(xwd->display,dev->gc,xwd->visual->blue_mask);
+        break;
+    }
+  if (dev->write_to_pixmap==1)
     XFillRectangle(xwd->display, dev->pixmap, dev->gc, 0, 0, dev->width, dev->height);
-  if (1) // not (dev->write_to_window): always!
+  if (dev->write_to_window)
     XFillRectangle(xwd->display, dev->window, dev->gc, 0, 0, dev->width, dev->height);
   XSetForeground(xwd->display,dev->gc,dev->curcolor.pixel);
   XSetPlaneMask(xwd->display,dev->gc,AllPlanes);
 }
-  
+
 unsigned long GDLXStream::GetWindowDepth() {
   XwDev *dev = (XwDev *) pls->dev;
   XwDisplay *xwd = (XwDisplay *) dev->xwd;
@@ -209,6 +216,22 @@ DLong GDLXStream::GetVisualDepth() {
   if(XGetWindowAttributes( xwd->display, dev->window, &wa )) return (long)wa.depth;
   else return -1;
 }
+BaseGDL* GDLXStream::GetFontnames(DString pattern) {
+  if (pattern.length()<=0) return NULL;
+  XwDev *dev = (XwDev *) pls->dev;
+  XwDisplay *xwd = (XwDisplay *) dev->xwd;
+  int actual_count_return;
+  char **list= XListFonts(xwd->display, pattern.c_str(), 30000, &actual_count_return);
+  if (!actual_count_return) return NULL;
+  DStringGDL* myList=new DStringGDL(dimension(actual_count_return));
+  for (int i=0; i< actual_count_return; ++i) (*myList)[i].assign(list[i]);
+  XFreeFontNames(list);
+  return myList;
+}
+DLong GDLXStream::GetFontnum(DString pattern){
+  if (pattern.length()<=0) return -1;
+  return this->GetFontnames(pattern)->N_Elements();
+}
 DString GDLXStream::GetVisualName() {
   static const char* visual_classes_names[] = {
    "StaticGray" , 
@@ -247,14 +270,25 @@ DString GDLXStream::GetVisualName() {
     } 
     else 
     { 
-      XWMHints gestw;
-      gestw.input = FALSE;
-      gestw.flags = InputHint;
       XSetInputFocus(xwd->display, DefaultRootWindow(xwd->display),RevertToParent,CurrentTime);
     }
     return true;
   }  
   
+// This helps cursor window leave focus.
+//
+  bool GDLXStream::setFocus(bool value=true)
+  {
+    XwDev *dev = (XwDev *) pls->dev;
+    if( dev == NULL) return false;
+    XwDisplay *xwd = (XwDisplay *) dev->xwd;
+    XWMHints gestw;
+    gestw.input = value;
+    gestw.flags = InputHint;
+    XSetWMHints(xwd->display, dev->window, &gestw);
+    return true;
+  }
+  
   bool GDLXStream::SetBackingStore(int value)
   {
     XwDev *dev = (XwDev *) pls->dev;
@@ -298,12 +332,23 @@ void GDLXStream::DeIconic() {
   XMapWindow(dev->xwd->display, dev->window);
 }
 
-void GDLXStream::Flush() {
+void GDLXStream::UnMapWindow() {
+  //Used for /PIXMAP windows: 1) insure write_to_pixmap and not write_to_window, and 2) hide the window.
   XwDev *dev = (XwDev *) pls->dev;
+  dev->write_to_pixmap=1;
+  dev->write_to_window=0;
   XwDisplay *xwd = (XwDisplay *) dev->xwd;
-  XFlush(xwd->display);
+  XWithdrawWindow(xwd->display, dev->window, xwd->screen);
+  usedAsPixmap=true;
+}
+void GDLXStream::Flush() {
+//  XwDev *dev = (XwDev *) pls->dev;
+//  XwDisplay *xwd = (XwDisplay *) dev->xwd;
+//  XFlush(xwd->display);
+}
+void GDLXStream::Update() {
+  XFlush(static_cast<XwDisplay *>(static_cast<XwDev *>(pls->dev)->xwd)->display);
 }
-
 void GDLXStream::WarpPointer(DLong x, DLong y) {
   XwDev *dev = (XwDev *) pls->dev;
   XwDisplay *xwd = (XwDisplay *) dev->xwd;
@@ -322,9 +367,6 @@ void GDLXStream::UnSetDoubleBuffering() {
   pls->db = 0;
 }
 
-bool GDLXStream::HasDoubleBuffering() {
-  return true;
-}
 //modified version. Will not tell double buffering is available if current graphic function is not pure "copy".
 bool GDLXStream::HasSafeDoubleBuffering() {
     XwDev *dev = (XwDev *) pls->dev;
@@ -368,6 +410,7 @@ bool GDLXStream::GetGin(PLGraphicsIn *gin, int mode) {
   if (ostate & Button4Mask) gin->button = 4;
   if (ostate & Button5Mask) gin->button = 5; //IDL does not support buttons 4-5 but we may?
   //return if NOWAIT
+  setFocus(false);  // first try to get out of focus.
   if (mode == NOWAIT) return true;
   
   unsigned long event_mask = (PointerMotionMask|ButtonMotionMask);
@@ -625,172 +668,219 @@ unsigned long event_mask = (EnterWindowMask| LeaveWindowMask | KeyPressMask  | K
   gin->dY = (PLFLT) gin->pY / (dev->height - 1);
   if(grabbed)     XUngrabPointer(xwd->display,CurrentTime);
   if(kbgrabbed)   XUngrabKeyboard(xwd->display,CurrentTime);
-  XFlush(xwd->display);
+//  XFlush(xwd->display);
   return status;
 }
-#define ToXColor(a) (((0xFF & (a)) << 8) | (a))
 
 bool GDLXStream::PaintImage(unsigned char *idata, PLINT nx, PLINT ny, DLong *pos,
-        DLong trueColorOrder, DLong chan) {
+  DLong trueColorOrder, DLong chan) {
 
-  PLINT ix, iy;
   XwDev *dev = (XwDev *) pls->dev;
   XwDisplay *xwd = (XwDisplay *) dev->xwd;
-  XImage *ximg = NULL, *ximg_pixmap = NULL;
-
-  int x, y;
-
-  int (*oldErrorHandler)(Display*, XErrorEvent*);
-
-  oldErrorHandler = XSetErrorHandler(DeviceX::GetImageErrorHandler);
-
-  XFlush(xwd->display);
   
-  if (dev->write_to_pixmap) {
-    ximg = XGetImage(xwd->display, dev->pixmap, 0, 0,
-            dev->width, dev->height,
-            AllPlanes, ZPixmap);
-    ximg_pixmap = ximg;
-  }
-
-  if (dev->write_to_window)
-    ximg = XGetImage(xwd->display, dev->window, 0, 0,
-          dev->width, dev->height,
-          AllPlanes, ZPixmap);
-
-  XSetErrorHandler(oldErrorHandler);
-
-  if (ximg == NULL) {
-    XSync(xwd->display, 0);
-    x = 0;
-    y = 0;
-    if (dev->write_to_pixmap) {
-      XCopyArea(xwd->display, dev->pixmap, dev->window, dev->gc,
-              x, y, dev->width, dev->height, x, y);
-      XSync(xwd->display, 0);
-      ximg = ximg_pixmap;
-    }
-  }
-
-
-  int ncolors;
-  PLINT iclr1, ired, igrn, iblu;
-  if (trueColorOrder == 0 && chan == 0) {
-
-    ncolors = 256;
-
-    if (xwd->ncol1 != ncolors) {
-      //was free_mem from plplotP.h which is forbidden - thanks to GJ for pointing this.
-      if ( xwd->cmap1 != NULL ) { free( (void *)  xwd->cmap1); xwd->cmap1 = NULL; } 
-      xwd->cmap1 = (XColor *) calloc(ncolors, (size_t) sizeof (XColor));
-    }
-    //#endif
-
-    for (SizeT i = 0; i < ncolors; i++) {
-
-      xwd->cmap1[i].red = ToXColor(pls->cmap0[i].r);
-      xwd->cmap1[i].green = ToXColor(pls->cmap0[i].g);
-      xwd->cmap1[i].blue = ToXColor(pls->cmap0[i].b);
-      xwd->cmap1[i].flags = DoRed | DoGreen | DoBlue;
-
-      if (XAllocColor(xwd->display, xwd->map, &xwd->cmap1[i]) == 0)
-        break;
-    }
-    xwd->ncol1 = ncolors;
-  }
+  //how much do we need?
   PLINT xoff = (PLINT) pos[0]; //(pls->wpxoff / 32767 * dev->width + 1);
   PLINT yoff = (PLINT) pos[2]; //(pls->wpyoff / 24575 * dev->height + 1);
-  PLINT kx, ky;
+  PLINT xmax = dev->width - xoff;
+  PLINT ymax = dev->height - yoff;
+  if (nx < xmax) xmax = nx;
+  if (ny < ymax) ymax = ny;
 
+  PLINT rint[ctSize], gint[ctSize], bint[ctSize];
+  //load original table
+  GDLCT* actCT = GraphicsDevice::GetCT();
+  actCT->Get(rint,gint,bint,ctSize);
+  DLong decomposed=GraphicsDevice::GetDevice()->GetDecomposed();
+//define & populate if necessary XImage img.  
+  XImage *ximg = NULL;
+  if (chan>0) { //we need to get the destination part of screen image back to write on it
+    int x, y;
+//   XFlush(xwd->display); //could be overkill...
+
+    int (*oldErrorHandler)(Display*, XErrorEvent*);
+    oldErrorHandler = XSetErrorHandler(GetImageErrorHandler);
+    if (dev->write_to_pixmap == 1) {
+      ximg = XGetImage(xwd->display, dev->pixmap, xoff, dev->height-yoff-ymax, xmax, ymax, AllPlanes, ZPixmap);
+    } else {
+      ximg = XGetImage(xwd->display, dev->window, xoff, dev->height-yoff-ymax, xmax, ymax, AllPlanes, ZPixmap);
+    }
+    if (ximg == NULL) { //last chance!!!
+//      XSync(xwd->display, 0); //could be overkill...
+      x = 0;
+      y = 0;
+      if (dev->write_to_pixmap == 1) {
+        XCopyArea(xwd->display, dev->pixmap, dev->window, dev->gc, xoff, dev->height-yoff-ymax, xmax, ymax, x, y);
+//        XSync(xwd->display, 0); //could be overkill...
+      }
+    }
+    XSetErrorHandler(oldErrorHandler);
+    if (ximg == NULL) {
+      cerr << "Unhandled unsuccessful XCopyArea, returning." << endl;
+      return false;
+    }
+  } else {
+    void *imgData=malloc(xmax*ymax*4); //too much size allocated, but keeps X11 happy (no need of scanline length computation)
+    ximg = XCreateImage(xwd->display, xwd->visual, xwd->depth, ZPixmap, 0, (char*)imgData , xmax, ymax, 8, 0); //8 seems reasonable, see XCreateImage doc.
+  }
+
+  PLINT ix, iy;
   XColor curcolor;
+  curcolor = xwd->fgcolor; //default
+  PLINT iclr1, ired, igrn, iblu;
+  // parallelize does not work when using XGet[Put]Pixel in the loop below, otherwise would be OK!
+  // please allow parallelization only after removing this problem ;^)
+  //#ifdef _OPENMP
+  //  SizeT nOp = kxLimit * kyLimit;
+  //#endif
+  //  #pragma omp parallel if (nOp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nOp)) private(ired,igrn,iblu,kx,ky,iclr1,curcolor)
+  //  {
+  //  #pragma omp for
+  for (ix = 0; ix < xmax; ++ix) {
+    for (iy = 0; iy < ymax; ++iy) {
 
-  PLINT kxLimit = dev->width - xoff;
-  PLINT kyLimit = dev->height - yoff;
-
-  if (nx < kxLimit) kxLimit = nx;
-  if (ny < kyLimit) kyLimit = ny;
-
-  /*#ifdef _OPENMP
-  SizeT nOp = kxLimit * kyLimit;
-#endif
-  #pragma omp parallel if (nOp >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nOp)) private(ired,igrn,iblu,kx,ky,iclr1,curcolor)
-  {
-  #pragma omp for*/
-  for (ix = 0; ix < kxLimit; ++ix) {
-    for (iy = 0; iy < kyLimit; ++iy) {
-
-      kx = xoff + ix;
-      ky = yoff + iy;
-
-      if (trueColorOrder == 0 && chan == 0) {
-        iclr1 = idata[iy * nx + ix];
-
-        if (xwd->color)
-          curcolor = xwd->cmap1[iclr1];
-        else
-          curcolor = xwd->fgcolor;
-
-        //	  printf("ix: %d  iy: %d  pixel: %d\n", ix,iy,curcolor.pixel);
-
-      } else {
-        if (chan == 0) {
-          if (trueColorOrder == 1) {
-            ired = idata[3 * (iy * nx + ix) + 0];
-            igrn = idata[3 * (iy * nx + ix) + 1];
-            iblu = idata[3 * (iy * nx + ix) + 2];
-          } else if (trueColorOrder == 2) {
-            ired = idata[nx * (iy * 3 + 0) + ix];
-            igrn = idata[nx * (iy * 3 + 1) + ix];
-            iblu = idata[nx * (iy * 3 + 2) + ix];
-          } else if (trueColorOrder == 3) {
-            ired = idata[nx * (0 * ny + iy) + ix];
-            igrn = idata[nx * (1 * ny + iy) + ix];
-            iblu = idata[nx * (2 * ny + iy) + ix];
+      if (xwd->color) {
+        if (trueColorOrder == 0 && chan == 0) {
+           
+          iclr1 = idata[iy * nx + ix];
+          if (decomposed == 1){
+            curcolor.pixel = iclr1 * 256 * 256 + iclr1 * 256 + iclr1;
+          } else {
+            if (xwd->rw_cmap==0) {
+              ired = rint[iclr1];
+              igrn = gint[iclr1];
+              iblu = bint[iclr1];
+            } else {
+              ired = pls->cmap0[iclr1].r;
+              igrn = pls->cmap0[iclr1].g;
+              iblu = pls->cmap0[iclr1].b;
+            }
+            curcolor.pixel = ired * 256 * 256 + igrn * 256 + iblu;
           }
-          curcolor.pixel = ired * 256 * 256 + igrn * 256 + iblu;
-        }else{
-          unsigned long pixel = XGetPixel(ximg, kx, dev->height - 1 - ky);
-          if (chan == 1) {
-            pixel &= 0x00ffff;
-            ired = idata[1 * (iy * nx + ix) + 0];
-            curcolor.pixel = ired * 256 * 256 + pixel;
-          } else if (chan == 2) {
-            pixel &= 0xff00ff;
-            igrn = idata[1 * (iy * nx + ix) + 1];
-            curcolor.pixel = igrn * 256 + pixel;
-          } else if (chan == 3) {
-            pixel &= 0xffff00;
-            iblu = idata[1 * (iy * nx + ix) + 2];
-            curcolor.pixel = iblu + pixel;
+        } else {
+          if (chan == 0) {
+            if (trueColorOrder == 1) {
+              ired = idata[3 * (iy * nx + ix) + 0];
+              igrn = idata[3 * (iy * nx + ix) + 1];
+              iblu = idata[3 * (iy * nx + ix) + 2];
+            } else if (trueColorOrder == 2) {
+              ired = idata[nx * (iy * 3 + 0) + ix];
+              igrn = idata[nx * (iy * 3 + 1) + ix];
+              iblu = idata[nx * (iy * 3 + 2) + ix];
+            } else if (trueColorOrder == 3) {
+              ired = idata[nx * (0 * ny + iy) + ix];
+              igrn = idata[nx * (1 * ny + iy) + ix];
+              iblu = idata[nx * (2 * ny + iy) + ix];
+            }
+            curcolor.pixel = ired * 256 * 256 + igrn * 256 + iblu;
+          } else { //channel mode -> ximg was got from display -> is reversed
+            unsigned long pixel = XGetPixel(ximg, ix, ymax-1-iy);
+            if (chan == 1) { //1 byte bitmap passed
+              pixel &= 0x00ffff;
+              ired = idata[1 * (iy * nx + ix) + 0];
+              curcolor.pixel = ired * 256 * 256 + pixel;
+            } else if (chan == 2) {
+              pixel &= 0xff00ff;
+              igrn = idata[1 * (iy * nx + ix) + 0];
+              curcolor.pixel = igrn * 256 + pixel;
+            } else if (chan == 3) {
+              pixel &= 0xffff00;
+              iblu = idata[1 * (iy * nx + ix) + 0];
+              curcolor.pixel = iblu + pixel;
+            }
           }
         }
       }
-
-      //std::cout << "XPutPixel: "<<kx<<"  "<< dev->height-ky-1 << std::endl;
-      // TODO check if XPutPixel() and XGetPixel() are thread save
-      if (ky < dev->height && kx < dev->width)
-        XPutPixel(ximg, kx, dev->height - 1 - ky, curcolor.pixel);
+      XPutPixel(ximg, ix, ymax-1-iy, curcolor.pixel); //ximg IS reversed allways.
     }
   }
-  //}
-
-  if (dev->write_to_pixmap)
+  //  } //end parallelize 
+  if (dev->write_to_pixmap == 1)
     XPutImage(xwd->display, dev->pixmap, dev->gc, ximg, 0, 0,
-          0, 0, dev->width, dev->height);
+    xoff, dev->height-yoff-ymax, xmax, ymax);
 
-  if (1) //(dev->write_to_window) //always write
+  if (dev->write_to_window==1) //always write
     XPutImage(xwd->display, dev->window, dev->gc, ximg, 0, 0,
-          0, 0, dev->width, dev->height);
+    xoff, dev->height-yoff-ymax, xmax, ymax);
 
-  if (ximg != ximg_pixmap) {
-    XDestroyImage(ximg);
-    XDestroyImage(ximg_pixmap);
-  } else {
-    XDestroyImage(ximg);
-  }
+  XDestroyImage(ximg);
   return true;
 }
-#undef ToXColor
 
+void GDLXStream::Color( ULong color, DLong decomposed ) {
+  XwDev *dev = (XwDev *) pls->dev;
+  XwDisplay *xwd = (XwDisplay *) dev->xwd;
+  if (xwd->rw_cmap)  { //not treated here, revert to safety with plplot's overhead.
+    if ( decomposed == 0 ) { //get current colormap
+      plstream::col0( color & 0xFF ); //just set color index [0..255]. simple and fast.
+    } else {
+      GDLGStream::SetColorMap1SingleColor(color);
+      plstream::col1(1);
+    }
+    return;
+  }
+  unsigned int curcolor;
+  DByte r, g, b;
+  if ( decomposed == 0 ) { //get current colormap
+    //load original table
+    GDLCT* actCT = GraphicsDevice::GetCT();
+    actCT->Get(color & 0xFF,r,g,b);
+  } else { 
+  //truecolor=decomposed case
+    r = (color & 0xFF);
+    g = (color >> 8 & 0xFF);
+    b = (color >> 16 & 0xFF);
+  }
+  curcolor= ( (unsigned long) (r) << (ffs(xwd->visual->red_mask)-1)) +
+    ((unsigned long)(g) << (ffs(xwd->visual->green_mask)-1) ) +
+    ((unsigned long)(b) << (ffs(xwd->visual->blue_mask)-1) );
+  XSetForeground( xwd->display, dev->gc, curcolor);
+}
+
+//Read X11 bitmapdata -- normally on 4BPP=Allplanes, return 3BPP ignoring Alpha plane.
+DByteGDL* GDLXStream::GetBitmapData() {
+//  plstream::cmd( PLESC_FLUSH, NULL );
+  GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
+  
+  XwDev *dev = (XwDev *) pls->dev;
+  XwDisplay *xwd = (XwDisplay *) dev->xwd;
+  XImage *ximg = NULL;
+  XWindowAttributes win_attr;
+
+  /* query the window's attributes. */
+  Status rc = XGetWindowAttributes(xwd->display, dev->window, &win_attr);
+  unsigned int nx = win_attr.width;
+  unsigned int ny = win_attr.height;
+
+    int (*oldErrorHandler)(Display*, XErrorEvent*);
+    oldErrorHandler = XSetErrorHandler(GetImageErrorHandler);
+    if (dev->write_to_pixmap==1) {
+      ximg = XGetImage(xwd->display, dev->pixmap, 0, 0, nx, ny, AllPlanes, ZPixmap);
+    } else {
+      ximg = XGetImage( xwd->display, dev->window, 0, 0, nx, ny, AllPlanes, ZPixmap);
+    }
+    XSetErrorHandler(oldErrorHandler);
+    
+    if (ximg == NULL) return NULL;
+    if (ximg->bits_per_pixel != 32) return NULL;
+
+    SizeT datadims[3];
+    datadims[0] = nx;
+    datadims[1] = ny;
+    datadims[2] = 3;
+    dimension datadim(datadims, (SizeT) 3);
+    DByteGDL *bitmap = new DByteGDL( datadim, BaseGDL::NOZERO);
+    //PADDING is 4BPP -- we take 3BPP and revert Y to respect IDL default
+    SizeT kpad = 0;
+    for ( SizeT iy =0; iy < ny ; ++iy ) {
+      for ( SizeT ix = 0; ix < nx; ++ix ) {
+        (*bitmap)[3 * ((ny-1-iy) * nx + ix) + 2] = ximg->data[kpad++];
+        (*bitmap)[3 * ((ny-1-iy) * nx + ix) + 1] = ximg->data[kpad++];
+        (*bitmap)[3 * ((ny-1-iy) * nx + ix) + 0] = ximg->data[kpad++];
+        kpad++; //pad to 4
+      }
+    }
+    XDestroyImage(ximg);
+    return bitmap;
+}
 
 #endif
