@@ -24,36 +24,29 @@
 #endif
 // #undef cerr if you want to try it.
 
+//According the to Windows Dev Center WIN32_LEAN_AND_MEAN excludes APIs such as Cryptography, DDE, RPC, Shell, and Windows Sockets.
+//It speeds the build process  by excluding some of the less frequently used APIs. VC_EXTRALEAN is possibly another option still "leaner"
+#ifdef WIN32
+#  define WIN32_LEAN_AND_MEAN 1
+#endif
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-//#else
-//#error "config.h required. Compile with -DHAVE_CONFIG_H"
-//#endif
 #ifdef _MSC_VER
 #define NOMINMAX
 #define _WINSOCKAPI_
+#pragma warning( disable : 4716 )
 #endif
 
 // Python.h must be included before everything else
 #if defined(USE_PYTHON) || defined(PYTHON_MODULE)
 
-// save HAVE_LIBREADLINE status (Python.h defines HAVE_LIBREADLINE)
 #ifndef HAVE_LIBREADLINE
 #define GDL_NOT_HAVE_READLINE
 #endif
 
-//#undef _POSIX_C_SOURCE // get rid of warning
 #include <Python.h>
-//#ifndef _POSIX_C_SOURCE 
-//#warning "_POSIX_C_SOURCE not defined in Python.h (remove #undef)"
-//#endif
-
-// for the python module pyhton's readline is used
-// this is ok as long as there is no GDL prompt within python
-#ifdef PYTHON_MODULE
-#undef HAVE_LIBREADLINE
-#endif
 
 #ifdef GDL_NOT_HAVE_READLINE
 #undef HAVE_LIBREADLINE
@@ -61,7 +54,6 @@
 
 #undef GDL_NOT_HAVE_READLINE
 
-//#if defined(USE_PYTHON) || defined(PYTHON_MODULE)
 #endif
 
 #if defined(USE_EIGEN)
