@@ -52,18 +52,6 @@
 // for extensions
 #include "new.hpp"
 
-// for sorting lists by name
-struct CompLibFunName: public std::binary_function< DLibFun*, DLibFun*, bool>
-{
-  bool operator() ( DLibFun* f1, DLibFun* f2) const
-  { return f1->ObjectName() < f2->ObjectName();}
-};
-
-struct CompLibProName: public std::binary_function< DLibPro*, DLibPro*, bool>
-{
-  bool operator() ( DLibPro* f1, DLibPro* f2) const
-  { return f1->ObjectName() < f2->ObjectName();}
-};
 
 using namespace std;
 
@@ -74,6 +62,7 @@ void LibInit_mes(); // libinit_mes.cpp
 void LibInit_ac(); // libinit_ac.cpp
 void LibInit_gm(); // libinit_gm.cpp
 void LibInit_ng(); // libinit_ng.cpp
+void LibInit_jp(); // libinit_jp.cpp
 
 void LibInit()
 {
@@ -84,6 +73,7 @@ void LibInit()
   LibInit_ac();
   LibInit_gm();
   LibInit_ng(); 
+  LibInit_jp();
   const char KLISTEND[] = "";
 
   const string listKey[]={"EXTRACT", "LENGTH", "NO_COPY", KLISTEND};
@@ -92,8 +82,12 @@ void LibInit()
   const string hashKey[]={"NO_COPY", KLISTEND};
   new DLibFunRetNew(lib::hash_fun,string("HASH"),-1,hashKey);
 
+  new DLibFun(lib::scope_level,string("SCOPE_LEVEL"),0);
+
   const string scope_varfetchKey[]={"LEVEL", KLISTEND};
   new DLibFun(lib::scope_varfetch_value,string("SCOPE_VARFETCH"),-1,scope_varfetchKey);
+  const string scope_tracebackKey[]={"STRUCTURE","SYSTEM", KLISTEND};
+  new DLibFunRetNew(lib::scope_traceback,string("SCOPE_TRACEBACK"),0,scope_tracebackKey);
 
   const string cpuKey[]={ "RESET","RESTORE","TPOOL_MAX_ELTS", "TPOOL_MIN_ELTS",
 					"TPOOL_NTHREADS","VECTOR_ENABLE",KLISTEND};
@@ -111,11 +105,16 @@ void LibInit()
 				  "PARAMETERS","SOURCE", KLISTEND};
   new DLibFunRetNew(lib::routine_info,string("ROUTINE_INFO"),1,routine_infoKey);
 
-#ifndef _MSC_VER
+#ifdef _WIN32
+//Please note that NOWAIT and HIDE are WINDOWS-Reserved Keywords.
+  const string spawnKey[] = { "COUNT", "EXIT_STATUS", "PID",
+	  "SH", "NOSHELL", "UNIT", "HIDE", "NOWAIT", KLISTEND };
+#else
   const string spawnKey[]={ "COUNT","EXIT_STATUS","PID",
 			    "SH","NOSHELL","UNIT",KLISTEND};
-  new DLibPro(lib::spawn_pro,string("SPAWN"),3,spawnKey);
 #endif
+ 
+  new DLibPro(lib::spawn_pro,string("SPAWN"),3,spawnKey);
 
   const string bytsclKey[]={"MIN","MAX","TOP","NAN",KLISTEND};
   new DLibFunRetNew(lib::bytscl,string("BYTSCL"),3,bytsclKey);
@@ -134,25 +133,33 @@ void LibInit()
   const string obj_classKey[]={"COUNT","SUPERCLASS",KLISTEND};
   new DLibFunRetNew(lib::obj_class,string("OBJ_CLASS"),1,obj_classKey);
 
-  new DLibFunRetNew(lib::obj_isa,string("OBJ_ISA"),2);
+  new DLibFunRetNew(lib::obj_isa,string("OBJ_ISA"),2,NULL,NULL,false,2);
 
   const string rebinKey[]={"SAMPLE",KLISTEND};
   new DLibFunRetNew(lib::rebin_fun,string("REBIN"),9,rebinKey);
 
-  const string convolKey[]={"CENTER","EDGE_TRUNCATE","EDGE_WRAP",
-			    "BIAS","NORMALIZE",KLISTEND};
+  const string convolKey[]={"CENTER","EDGE_TRUNCATE","EDGE_WRAP","EDGE_ZERO",
+			    "BIAS","NORMALIZE","NAN", "INVALID", "MISSING",KLISTEND};
   new DLibFunRetNew(lib::convol_fun,string("CONVOL"),3,convolKey);
 
-#ifndef _MSC_VER
   const string file_searchKey[]={"COUNT","EXPAND_ENVIRONMENT","EXPAND_TILDE",
 				 "FOLD_CASE","ISSUE_ACCESS_ERROR",
 				 "MARK_DIRECTORY","NOSORT","QUOTE",
 				 "MATCH_INITIAL_DOT",
-				 "MATCH_ALL_INITIAL_DOT","FULLY_QUALIFY_PATH",KLISTEND};
+                                 "MATCH_ALL_INITIAL_DOT","FULLY_QUALIFY_PATH",
+                                "TEST_DIRECTORY", "TEST_EXECUTABLE", "TEST_READ",
+                                 "TEST_REGULAR", "TEST_WRITE", "TEST_ZERO_LENGTH",
+                                 "TEST_SYMLINK",
+                                 KLISTEND};
   new DLibFunRetNew(lib::file_search,string("FILE_SEARCH"),2,file_searchKey);
-#endif
 
-  const string expand_pathKey[]={"ARRAY","ALL_DIRS","COUNT",KLISTEND};
+  const string file_expand_pathKey[]={KLISTEND};
+  new DLibFunRetNew(lib::file_expand_path,string("FILE_EXPAND_PATH"),1,file_expand_pathKey);
+
+  const string file_readlinkKey[]={"ALLOW_NONEXISTENT","ALLOW_NONSYMLINK","NOEXPAND_PATH",KLISTEND};
+  new DLibFunRetNew(lib::file_readlink,string("FILE_READLINK"),1,file_readlinkKey);
+
+  const string expand_pathKey[]={"ARRAY","ALL_DIRS","COUNT","PATTERN",KLISTEND};
   new DLibFunRetNew(lib::expand_path,string("EXPAND_PATH"),1,expand_pathKey);
   
   const string strjoinKey[]={"SINGLE",KLISTEND};
@@ -165,12 +172,12 @@ void LibInit()
 
   new DLibFunRetNew(lib::arg_present,string("ARG_PRESENT"),1);
 
-  const string messageKey[]={"CONTINUE","INFORMATIONAL","IOERROR",
+  const string messageKey[]={"CONTINUE","INFORMATIONAL","IOERROR","LEVEL",
 			     "NONAME","NOPREFIX","NOPRINT",
 			     "RESET","REISSUE_LAST","TRACEBACK", KLISTEND}; 
   //TRACEBACK is in MESSAGE but obsolete since 5.0. it is used widely in CMSVlib !
-  const string messageWarnKey[]={"LEVEL", "NAME", "BLOCK",KLISTEND};
-  new DLibPro(lib::message,string("MESSAGE"),1,messageKey,messageWarnKey);
+  const string messageWarnKey[]={"NAME", "BLOCK",KLISTEND};
+  new DLibPro(lib::message_pro,string("MESSAGE"),-1,messageKey,messageWarnKey);
   
   const string cdKey[]={"CURRENT",KLISTEND};
   new DLibPro(lib::cd_pro,string("CD"),1,cdKey);
@@ -179,7 +186,7 @@ void LibInit()
 			       "REGULAR","WRITE","ZERO_LENGTH",
 			       "GET_MODE",
 			       "BLOCK_SPECIAL","CHARACTER_SPECIAL",
-			       "NAMED_PIPE","SOCKET","SYMLINK","NOEXPAND_PATH",KLISTEND};
+			       "NAMED_PIPE","SOCKET","SYMLINK","NOEXPAND_PATH","DANGLING_SYMLINK",KLISTEND};
   new DLibFunRetNew(lib::file_test,string("FILE_TEST"),1,file_testKey);
 
   const string file_basenameKey[]={"FOLD_CASE",KLISTEND};
@@ -199,6 +206,7 @@ void LibInit()
   new DLibPro(lib::file_mkdir,string("FILE_MKDIR"),-1,file_mkdirKey);
 
   new DLibFunRetNew(lib::shift_fun,string("SHIFT"),9,NULL,NULL,true);
+  new DLibFunRetNew(lib::ishft_fun,string("ISHFT"),2,NULL,NULL,true);
 
   const string sortKey[]={"L64",KLISTEND};
   new DLibFunRetNew(lib::sort_fun,string("SORT"),1,sortKey,NULL,true);
@@ -219,38 +227,48 @@ void LibInit()
   const string exitKey[]={"NO_CONFIRM","STATUS",KLISTEND};
   new DLibPro(lib::exitgdl,string("EXIT"),0,exitKey);
   
-  const string helpKey[]={"ALL_KEYS","BRIEF","CALLS","FUNCTIONS","HELP","INFO",
-			  "INTERNAL_LIB_GDL","LAST_MESSAGE","LIB","MEMORY",
+  const string helpKey[]={"ALL_KEYS","BRIEF","FULL","CALLS","DEVICE","FUNCTIONS","HELP","INFO",
+			  "INTERNAL_LIB_GDL","KEYS","LAST_MESSAGE","LIB","MEMORY","NAMES",
 			  "OUTPUT","PATH_CACHE","PREFERENCES","PROCEDURES",
-			  "RECALL_COMMANDS","ROUTINES","SOURCE_FILES","STRUCTURES", KLISTEND};
-  const string helpWarnKey[]={"FULL","TRACEBACK", KLISTEND};
-  new DLibPro(lib::help,string("HELP"),-1,helpKey,helpWarnKey);
-
+			  "RECALL_COMMANDS","ROUTINES","SOURCE_FILES","STRUCTURES",
+              "SYSTEM_VARIABLES","TRACEBACK", "COMMON","LEVEL", KLISTEND};
+  const string helpWarnKey[]={"BREAKPOINTS","DLM","FILES","HEAP_VARIABLES","MESSAGES",
+			      "OBJECTS","SHARED_MEMORY", KLISTEND};
+  new DLibPro(lib::help_pro,string("HELP"),-1,helpKey,helpWarnKey);
+  
+  //stub to avoid setting errors on pref_set. One may want to really write pref_set,
+  // but this function is just here to prevent setting !ERR=-1 when stumbling on a pref_set command,
+  // since !ERR=-1 is frequently checked by legacy procedures.
+  const string pref_setKey[] = {"FILENAME","COMMIT","DEFAULT", KLISTEND };
+  new DLibPro(lib::pref_set_pro, string("PREF_SET"), -1, pref_setKey);
+  
+  
   const string memoryKey[]={"CURRENT","HIGHWATER","NUM_ALLOC",
     "NUM_FREE","STRUCTURE","L64",KLISTEND};
   new DLibFunRetNew(lib::memory, string("MEMORY"), 1, memoryKey, NULL);
 
   // printKey, readKey and stringKey are closely associated
   // as the same functions are called "FORMAT" till "MONTH"
-  // must be the first four keywords
-  const string printKey[]={"FORMAT","AM_PM","DAYS_OF_WEEK","MONTH",
-			   "STDIO_NON_FINITE",KLISTEND};
+  // must be the first four keywords. The inner print_os function is BASED on this ORDER!
+  //NOTE THAT AM_PM, DAYS_OF_WEEK and MONTHS are silently ignored!!!
+  #define COMMONKEYWORDSFORSTRINGFORMATTING "FORMAT","AM_PM","DAYS_OF_WEEK","MONTHS"
+  const string printKey[]={COMMONKEYWORDSFORSTRINGFORMATTING, "STDIO_NON_FINITE",KLISTEND};
   new DLibPro(lib::print,string("PRINT"),-1,printKey);
   new DLibPro(lib::printf,string("PRINTF"),-1,printKey);
-
-  const string readKey[]={"FORMAT","AM_PM","DAYS_OF_WEEK","MONTH",
-			  "PROMPT",KLISTEND};
-  new DLibPro(lib::read,string("READ"),-1,readKey);
-  new DLibPro(lib::readf,string("READF"),-1,readKey);
-
-  const string readsKey[]={"FORMAT","AM_PM","DAYS_OF_WEEK","MONTH",
-			   KLISTEND}; // no PROMPT
-  new DLibPro(lib::reads,string("READS"),-1,readsKey);
-
   // allow printing (of expressions) with all keywords 
   // (easier to implement this way)
   new DLibPro(lib::stop,string("STOP"),-1,printKey); 
 
+  const string readKey[]={COMMONKEYWORDSFORSTRINGFORMATTING,"PROMPT",KLISTEND};
+  new DLibPro(lib::read,string("READ"),-1,readKey);
+  new DLibPro(lib::readf,string("READF"),-1,readKey);
+
+  const string readsKey[]={COMMONKEYWORDSFORSTRINGFORMATTING,	   KLISTEND}; // no PROMPT
+  new DLibPro(lib::reads,string("READS"),-1,readsKey);
+
+  const string stringKey[]={COMMONKEYWORDSFORSTRINGFORMATTING,"PRINT",KLISTEND};
+  new DLibFun(lib::string_fun,string("STRING"),-1,stringKey);
+
   const string defsysvKey[]={"EXISTS",KLISTEND};
   new DLibPro(lib::defsysv,string("DEFSYSV"),3,defsysvKey); 
 
@@ -314,7 +332,7 @@ void LibInit()
 
   new DLibFunRetNew(lib::n_elements,string("N_ELEMENTS"),1,NULL,NULL,true,1);
 
-  new DLibFun(lib::execute,string("EXECUTE"),2);
+  new DLibFun(lib::execute_fun,string("EXECUTE"),3);
 
   const string openKey[]={"APPEND","COMPRESS","BUFSIZE",
 			  "DELETE","ERROR","F77_UNFORMATTED",
@@ -340,30 +358,26 @@ void LibInit()
 
   new DLibPro(lib::flush_lun,string("FLUSH"),-1);
 
-  const string closeKey[]={"EXIT_STATUS","FORCE","FILE","ALL",KLISTEND};
-  new DLibPro(lib::close_lun,string("CLOSE"),-1,closeKey);
+  const string close_lunKey[]={"FORCE","FILE","ALL",KLISTEND};
+  const string close_lunWarnKey[]={"EXIT_STATUS",KLISTEND};
+  new DLibPro(lib::close_lun,string("CLOSE"),-1,close_lunKey,close_lunWarnKey);
   
-  const string free_lunKey[]={"EXIT_STATUS","FORCE",KLISTEND};
-  new DLibPro(lib::free_lun,string("FREE_LUN"),-1,free_lunKey);
+  const string free_lunKey[]={"FORCE",KLISTEND};
+  const string free_lunWarnKey[]={"EXIT_STATUS",KLISTEND};
+  new DLibPro(lib::free_lun,string("FREE_LUN"),-1,free_lunKey,free_lunWarnKey);
 
   const string writeuKey[]={"TRANSFER_COUNT",KLISTEND};
   new DLibPro(lib::writeu,string("WRITEU"),-1,writeuKey);
   new DLibPro(lib::readu,string("READU"),-1,writeuKey);
 
-  const string resolve_routineWarnKey[]={"EITHER","IS_FUNCTION","NO_RECOMPILE","COMPILE_FULL_FILE",KLISTEND};
-  const string resolve_routineKey[]={KLISTEND};
+  const string resolve_routineWarnKey[]={"COMPILE_FULL_FILE",KLISTEND};
+  const string resolve_routineKey[]={"NO_RECOMPILE","IS_FUNCTION","EITHER",KLISTEND};
   new DLibPro(lib::resolve_routine,string("RESOLVE_ROUTINE"),1,
-	      NULL, resolve_routineWarnKey);
+	      resolve_routineKey,resolve_routineWarnKey);
 
   const string assocKey[]={"PACKED",KLISTEND};
   new DLibFunRetNew(lib::assoc,string("ASSOC"),3,assocKey);
-  
-  const string stringKey[]={"FORMAT","AM_PM","DAYS_OF_WEEK","MONTH",
-			    "PRINT",KLISTEND};
-//  new DLibFunRetNew(lib::string_fun,string("STRING"),-1,stringKey,NULL,true);
-//  new DLibFunRetNew(lib::byte_fun,string("BYTE"),10,NULL,NULL,true);
-// that's apparently the desired bahaviour, see bug no. 3151760
-  new DLibFun(lib::string_fun,string("STRING"),-1,stringKey,NULL);
+
   new DLibFun(lib::byte_fun,string("BYTE"),10,NULL,NULL);
 
 /*
@@ -510,10 +524,15 @@ void LibInit()
 
   const string set_plotKey[]={"COPY","INTERPOLATE",KLISTEND};
   new DLibPro(lib::set_plot,string("SET_PLOT"),1,set_plotKey);
-  
-  const string get_screen_sizeKey[]={"RESOLUTION",KLISTEND};
-  new DLibFunRetNew(lib::get_screen_size,string("GET_SCREEN_SIZE"),1,get_screen_sizeKey);
 
+#ifdef HAVE_X  
+  const string get_screen_sizeKey[]={"RESOLUTION","DISPLAY_NAME",KLISTEND};
+  new DLibFunRetNew(lib::get_screen_size,string("GET_SCREEN_SIZE"),1,get_screen_sizeKey);
+#else
+  const string get_screen_sizeKey[]={"RESOLUTION",KLISTEND};
+  // DisplayName option or parameter only with X11.
+  new DLibFunRetNew(lib::get_screen_size,string("GET_SCREEN_SIZE"),0,get_screen_sizeKey);
+#endif
   const string tvlctKey[]={"GET","HLS","HSV",KLISTEND};
   new DLibPro(lib::tvlct,string("TVLCT"),4,tvlctKey);
 
@@ -525,26 +544,80 @@ void LibInit()
   
   const string deviceKey[]=
     {
-      "CLOSE_FILE","FILENAME","LANDSCAPE","PORTRAIT",
+      "CLOSE_FILE", "FILENAME", "LANDSCAPE", "PORTRAIT",  //there is a "CLOSE" for device Z defined... and a CLOSE_DOCUMENT for printer 
       "DECOMPOSED","GET_DECOMPOSED","Z_BUFFERING","SET_RESOLUTION",
-      "SET_CHARACTER_SIZE","GET_VISUAL_DEPTH","XSIZE","YSIZE",
-      "COLOR","GET_SCREEN_SIZE","INCHES","WINDOW_STATE","SCALE_FACTOR", 
+      "XSIZE","YSIZE",
+      "COLOR","GET_PAGE_SIZE","GET_SCREEN_SIZE","INCHES","WINDOW_STATE","SCALE_FACTOR", 
       "XOFFSET", "YOFFSET", "ENCAPSULATED", "GET_GRAPHICS_FUNCTION", 
       "SET_GRAPHICS_FUNCTION", "CURSOR_STANDARD", "CURSOR_ORIGINAL",
-      "CURSOR_CROSSHAIR","RETAIN", KLISTEND
+      "CURSOR_CROSSHAIR","RETAIN", "BITS_PER_PIXEL", 
+      "GET_WINDOW_POSITION","GET_PIXEL_DEPTH","GET_VISUAL_DEPTH","GET_VISUAL_NAME",
+      "GET_WRITE_MASK", "COPY","GET_FONTNAMES","SET_FONT","GET_CURRENT_FONT","GET_FONTNUM",
+      "SET_PIXEL_DEPTH", //Z
+      KLISTEND
     };
-  const string deviceWarnKey[] = {"SET_FONT", "HELVETICA", 
-    "AVANTGARDE", "BKMAN", "COURIER", "PALATINO", 
-    "SCHOOLBOOK", "TIMES", "ZAPFCHANCERY", "ZAPFDINGBATS", "BITS_PER_PIXEL", 
-    "ITALIC", "BOLD", "TRUE_COLOR", "CURSOR_IMAGE","CURSOR_MASK","CURSOR_XY", KLISTEND};
+  const string deviceWarnKey[] = {
+    "AVANTGARDE","BKMAN","COURIER","HELVETICA","ISOLATIN","PALATINO","SCHOOLBOOK","SYMBOL","TIMES","ZAPFCHANCERY","ZAPFDINGBATS", //PS
+  //"AVERAGE_LINES", (REGIS)
+  //"BINARY","NCAR","TEXT", (CGM)
+  "BOLD", //PS
+  "BOOK", //PS
+  "BYPASS_TRANSLATION", //(WIN,X)
+  "CMYK", //PS
+  "CURSOR_IMAGE", //WIN,X
+  "CURSOR_MASK",  //WIN,X
+  "CURSOR_XY",  //WIN,X
+  "DIRECT_COLOR", //X
+  //"EJECT", (HP)
+  //"ENCODING", (CGM)
+  "FLOYD", //(PCL,X)
+  "FONT_INDEX", //PS
+  "FONT_SIZE", //PS
+  //"GIN_CHARS", (TEX)
+  "GLYPH_CACHE", //PRINTER,PS,WIN,Z,METAFILE
+  //"INDEX_COLOR", (METAFILE, PRINTER)
+  "ITALIC", //PS
+  "LANGUAGE_LEVEL" //PS,
+  "DEMI","LIGHT","MEDIUM","NARROW","OBLIQUE", //PS
+  //"OPTIMIZE", (PCL)
+  "ORDERED", //(PCL,X)
+  "OUTPUT", //(HP,PS)
+  //"PIXELS", (PCL)
+  //"PLOT_TO", (REGIS,TEK)
+  //"PLOTTER_ON_OFF", "POLYFILL", (HP)
+  "PRE_DEPTH","PRE_XSIZE","PRE_YSIZE","PREVIEW", //PS
+  "PRINT_FILE",// WIN
+  "PSEUDO_COLOR",//X
+  //"RESET_STRING", (TEK)
+  //"RESOLUTION", (PCL)
+  //"SET_COLORMAP",(PCL)
+  "SET_CHARACTER_SIZE", //(all)
+  "SET_COLORS", //Z
+  //"SET_STRING", (TEK)
+  "SET_TRANSLATION", //X
+  "SET_WRITE_MASK", //(X,Z)
+  "STATIC_COLOR",//X
+  "STATIC_GRAY",//X
+  //"TEK4014","TEK4100", (TEK)
+  "THRESHOLD", //X
+  "TRANSLATION", //(WIN,X)
+  "TRUE_COLOR", //(METAFILE, PRINTER, X)
+  "TT_FONT", //(METAFILE, PRINTER, X, PS, WIN, Z)
+  //"TTY", (REGIS, TEK)
+  //"VT240","VT241","VT340","VT341",(REGIS)
+  //"XON_XOFF" (HP)
+   KLISTEND};
   new DLibPro(lib::device,string("DEVICE"),0, deviceKey, deviceWarnKey);
 
   const string usersymKey[]= 
   {
-    "COLOR","FILL","THICK",
-    KLISTEND
+    "FILL", KLISTEND
    };
-  new DLibPro(lib::usersym,string("USERSYM"),2, usersymKey);
+    const string usersymWarnKey[]= 
+  {
+    "COLOR","THICK", KLISTEND
+   };
+  new DLibPro(lib::usersym,string("USERSYM"),2, usersymKey, usersymWarnKey);
 
   const string plotKey[]=
     {
@@ -555,7 +628,7 @@ void LibInit()
      "SYMSIZE",   "THICK",    "TICKLEN", "TITLE",
      "MAX_VALUE", "MIN_VALUE",
      "XLOG",      "YLOG",
-     "YNOZERO",   "XTYPE",    "YTYPE",   "POLAR", "NSUM",
+     "YNOZERO",   "XTYPE",    "YTYPE",   "POLAR", "NSUM", //XTYPE and YTYPE are oldies, equivalent to XLOG when value is odd.
       "XCHARSIZE", "YCHARSIZE",
       "XGRIDSTYLE", "YGRIDSTYLE",
       "XMARGIN", "YMARGIN",
@@ -576,11 +649,14 @@ void LibInit()
       "ZCHARSIZE", "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
       "ZRANGE", "ZSTYLE", "ZTHICK", "ZTICK_GET", "ZTICKFORMAT", "ZTICKINTERVAL",
       "ZTICKLAYOUT", "ZTICKLEN", "ZTICKNAME", "ZTICKS", "ZTICKUNITS", "ZTICKV",
-      "ZTITLE", "T3D", "ZVALUE", KLISTEND
+      "ZTITLE", "T3D", "ZVALUE", "FONT", "CHANNEL", KLISTEND
     };
   //
-  const string plotWarnKey[]= { "FONT","CHANNEL", KLISTEND };
-  new DLibPro(lib::plot,string("PLOT"),2,plotKey,plotWarnKey);
+//  const string plotWarnKey[]= { "FONT","CHANNEL", KLISTEND };
+  new DLibPro(lib::plot,string("PLOT"),2,plotKey);//,plotWarnKey);
+  new DLibPro(lib::plot_io,string("PLOT_IO"),2,plotKey);//,plotWarnKey);
+  new DLibPro(lib::plot_oo,string("PLOT_OO"),2,plotKey);//,plotWarnKey);
+  new DLibPro(lib::plot_oi,string("PLOT_OI"),2,plotKey);//,plotWarnKey);
 
   const string axisKey[]=
   {
@@ -610,11 +686,10 @@ void LibInit()
     "ZAXIS", "ZLOG", "ZVALUE", "ZCHARSIZE", "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
     "ZRANGE", "ZSTYLE", "ZTHICK", "ZTICK_GET", "ZTICKFORMAT", "ZTICKINTERVAL",
     "ZTICKLAYOUT", "ZTICKLEN", "ZTICKNAME", "ZTICKS", "ZTICKUNITS", "ZTICKV",
-    "ZTITLE",
+    "ZTITLE", "CHANNEL",
     KLISTEND
   };
-  const string axisWarnKey[]={"CHANNEL",KLISTEND};
-  new DLibPro(lib::axis,string("AXIS"),3,axisKey,axisWarnKey);
+  new DLibPro(lib::axis,string("AXIS"),3,axisKey);
 
   const string oplotKey[]=
     {
@@ -625,10 +700,9 @@ void LibInit()
       "PSYM", "SYMSIZE",  "T3D",  "ZVALUE", "THICK",
       // 8
       "MAX_VALUE", "MIN_VALUE", "NSUM", "POLAR",
-      KLISTEND
+      "CHANNEL", KLISTEND
     };
-  const string oplotWarnKey[]={"CHANNEL",KLISTEND};
-  new DLibPro(lib::oplot, string("OPLOT"),2,oplotKey,oplotWarnKey);
+  new DLibPro(lib::oplot, string("OPLOT"),2,oplotKey);
 
   const string plotsKey[]=
     {
@@ -653,9 +727,12 @@ void LibInit()
 
   const string shade_surfKey[]=
     {
-      "AX", "AZ",  "MAX_VALUE", "MIN_VALUE", "SHADES", "XLOG", "YLOG","ZLOG", 
+      "AX", "AZ",  "MAX_VALUE", "MIN_VALUE", "SHADES", 
+      "HORIZONTAL", "LOWER_ONLY", "UPPER_ONLY", "BOTTOM", 
+      // ([xyz]type undocumented but still existing in SHADE_SURF ...)
+      "XLOG", "YLOG", "ZLOG", "XTYPE", "YTYPE", "ZTYPE", 
       //General Graphics KW
-      "BACKGROUND", "NOERASE",
+      "BACKGROUND", "NOERASE", "CLIP", "NOCLIP",
       "CHARSIZE", "CHARTHICK", "COLOR", "DATA", "DEVICE", "NORMAL", "FONT",
       "NODATA", "POSITION", "SUBTITLE", "THICK", "TICKLEN", "TITLE",
       //Axis KW
@@ -677,86 +754,60 @@ void LibInit()
       "ZTICK_GET", "YTICK_GET", "XTICK_GET",
       "ZTITLE", "YTITLE", "XTITLE",
       //3D KW
-      "ZVALUE","T3D", "SAVE", "SKIRT", "ZAXIS", KLISTEND
+      "ZVALUE","T3D", "SAVE", "SKIRT", "ZAXIS", "CHANNEL", KLISTEND
     };
   const string shade_surfWarnKey[]=
   {
-      "IMAGE", "PIXELS", "CHANNEL", KLISTEND
+      "IMAGE", "PIXELS", KLISTEND
   };
   new DLibPro(lib::shade_surf,string("SHADE_SURF"),3,shade_surfKey, shade_surfWarnKey);
 
   const string surfaceKey[]=
     {
      // GRAPHIC KEYWORDS
-     // 0
      "BACKGROUND","CHARSIZE","CHARTHICK","CLIP",
-     // 4
      "COLOR",     "DATA",    "DEVICE",   "FONT",
-     // 8
      "LINESTYLE", "NOCLIP",  "NODATA",   "NOERASE", 
-     // 12
      "NORMAL",    "POSITION",     "SUBTITLE",
-     // 15
-     "THICK",    "TICKLEN", 
-     // 18
-     "TITLE",
-     // 19
+     "THICK",    "TICKLEN", "TITLE",
      "XCHARSIZE",    "XGRIDSTYLE", "XMARGIN", "XMINOR",
-     // 23
      "XRANGE",       "XSTYLE",     "XTHICK",  "XTICKFORMAT",
-     // 27
      "XTICKINTERVAL","XTICKLAYOUT","XTICKLEN","XTICKNAME",
-     // 31
-     "XTICKS",       "XTICKUNITS", "XTICKV",  "XTICK_GET",
-     // 35
-     "XTITLE",
-     // 36
+     "XTICKS",       "XTICKUNITS", "XTICKV",  "XTICK_GET", "XTITLE",
      "YCHARSIZE",    "YGRIDSTYLE", "YMARGIN", "YMINOR",
-     // 40
      "YRANGE",       "YSTYLE",     "YTHICK",  "YTICKFORMAT",
-     // 44
      "YTICKINTERVAL","YTICKLAYOUT","YTICKLEN","YTICKNAME",
-     // 48
-     "YTICKS",       "YTICKUNITS", "YTICKV",  "YTICK_GET",
-     // 52
-     "YTITLE",
-     // 53
+     "YTICKS",       "YTICKUNITS", "YTICKV",  "YTICK_GET", "YTITLE",
      "ZCHARSIZE",    "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
-     // 57
      "ZRANGE",       "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
-     // 61
      "ZTICKINTERVAL","ZTICKLAYOUT","ZTICKLEN","ZTICKNAME",
-     // 65
-     "ZTICKS",       "ZTICKUNITS", "ZTICKV",  "ZTICK_GET",
-     // 69
-     "ZTITLE",
-     // 70
-
-
+     "ZTICKS",       "ZTICKUNITS", "ZTICKV",  "ZTICK_GET", "ZTITLE",
      // SURFACE keywords
-     // 73
-     "MAX_VALUE",  "MIN_VALUE", "AX", "AZ", "XLOG", "YLOG", "ZLOG",
+     "MAX_VALUE",  "MIN_VALUE", "AX", "AZ", 
+     // ([xyz]type undocumented but still existing in SURFACE ...)
+     "XLOG", "YLOG", "ZLOG", "XTYPE", "YTYPE", "ZTYPE", 
      "HORIZONTAL", "LOWER_ONLY", "UPPER_ONLY", "SHADES", "ZAXIS",  "BOTTOM", 
-     "SKIRT", "SAVE", "T3D",  "ZVALUE", KLISTEND
+     "SKIRT", "SAVE", "T3D",  "ZVALUE", "CHANNEL", KLISTEND
     };
   const string surfaceWarnKey[]=
   {
-      "LEGO", "CHANNEL",KLISTEND
+      "LEGO", KLISTEND
   };
   new DLibPro(lib::surface,string("SURFACE"),3,surfaceKey, surfaceWarnKey);
 
   const string contourKey[]=
     {
       "BACKGROUND","CHARSIZE","CHARTHICK","CLIP",
-      "COLOR",     "DATA",    "DEVICE",   "FONT",
+      "COLOR",     "DATA",    "DEVICE",
       "NOCLIP",  "NODATA",   "NOERASE",
       "NORMAL",    "POSITION", "SUBTITLE",
       "T3D",     "THICK",    "TICKLEN",
       "TITLE" ,    "LEVELS", "NLEVELS",
       "MAX_VALUE", "MIN_VALUE",
-      "XLOG", "YLOG", "FILL", "ISOTROPIC",
-      "FOLLOW",
-
+      // ([xy]type undocumented but still existing in CONTOUR ...)
+      "XLOG", "YLOG", "XTYPE", "YTYPE",
+      
+      "FILL", "ISOTROPIC", "FOLLOW",
       "XCHARSIZE", "YCHARSIZE", "ZCHARSIZE",
       "XGRIDSTYLE", "YGRIDSTYLE", "ZGRIDSTYLE",
       "XMARGIN", "YMARGIN", "ZMARGIN",
@@ -778,18 +829,17 @@ void LibInit()
       "C_CHARSIZE","OVERPLOT","C_COLORS","C_LINESTYLE",
       "C_LABELS", "C_CHARTHICK", "C_ORIENTATION", "C_SPACING", "C_THICK",
       "PATH_INFO","PATH_XY",
-      "ZLOG","IRREGULAR", //ZLOG is an addition for GDL only
+      "ZLOG","IRREGULAR", "CHANNEL", "FONT", //ZLOG is an addition for GDL only & FONT is not permitted apparently
       KLISTEND
     };
    // NO SUPPORT AT ALL for:,"CLOSED","DOWNHILL","IRREGULAR","PATH_DATA_COORDS","PATH_FILENAME",
    // "PATH_INFO","PATH_XY","TRIANGULATION","PATH_DOUBLE","ZAXIS"
-  // "CHANNEL" is supposed to be passed from CONTOUR, PLOT, OPLOT, SHADE_SURF etc to ERASER
+  // "CHANNEL" is supposed to be passed from CONTOUR, PLOT, OPLOT, SHADE_SURF etc to ERASE
    const string contourWarnKey[]=
     {
-      
       "CELL_FILL","C_ANNOTATIONS","CLOSED","DOWNHILL",
       "PATH_DATA_COORDS","PATH_FILENAME",
-      "PATH_INFO","PATH_XY","TRIANGULATION","PATH_DOUBLE","CHANNEL",KLISTEND
+      "PATH_INFO","PATH_XY","TRIANGULATION","PATH_DOUBLE",KLISTEND
     };
    new DLibPro(lib::contour,string("CONTOUR"),3,contourKey,contourWarnKey);
 
@@ -801,7 +851,6 @@ void LibInit()
       "ORIENTATION","ALIGNMENT","CHARSIZE","SIZE","CHARTHICK", //note SIZE is not in doc but in XYOUTS demo!
       "NOCLIP","T3D","Z","TEXT_AXES","WIDTH","FONT",KLISTEND
     };
-   const string xyoutsWarnKey[]={KLISTEND};  
    new DLibPro(lib::xyouts, string("XYOUTS"), 3, xyoutsKey);//, xyoutsWarnKey);
    
    const string polyfillKey[]=
@@ -833,6 +882,8 @@ void LibInit()
   const string eraseKey[]= {"CHANNEL","COLOR",KLISTEND};
   new DLibPro(lib::erase,string("ERASE"),1, eraseKey);
   
+  new DLibFunRetNew(lib::format_axis_values,string("FORMAT_AXIS_VALUES"),1);
+  
   const string laguerreKey[]={"DOUBLE","COEFFICIENTS",KLISTEND};
   new DLibFunRetNew(lib::laguerre,string("LAGUERRE"),3,laguerreKey);
 
@@ -848,9 +899,10 @@ void LibInit()
   const string newtonKey[] = {"DOUBLE", "ITMAX", "TOLF", "TOLX", "HYBRID", KLISTEND };
   const string newtonWarnKey[] = {"CHECK", "STEPMAX", "TOLMIN", KLISTEND};
   new DLibFunRetNew(lib::newton_broyden, string("NEWTON"), 2, newtonKey, newtonWarnKey);
-
-  const string broydenKey[] = {"DOUBLE", "ITMAX", "TOLF", "TOLX", KLISTEND };
-  const string broydenWarnKey[] = {"CHECK", "EPS", "STEPMAX", "TOLMIN", KLISTEND};
+  //WARNING: THESE FUNCTIONS MUST HAVE THE SAME OPTION LIST. NOTE THAT /HYBRID is not part of either functions,
+  //it is used by imsl_zerosys.pro
+  const string broydenKey[] = {"DOUBLE", "ITMAX", "TOLF", "TOLX", "HYBRID", KLISTEND };
+  const string broydenWarnKey[] = {"CHECK", "STEPMAX", "TOLMIN",  "EPS",  KLISTEND};
   new DLibFunRetNew(lib::newton_broyden, string("BROYDEN"), 2, broydenKey, broydenWarnKey);
 
   new DLibFunRetNew(lib::parse_url, string("PARSE_URL"), 1);
@@ -913,9 +965,5 @@ void LibInit()
   new DLibPro(lib::sem_delete, string("SEM_DELETE"), 1);
   new DLibFunRetNew(lib::sem_lock, string("SEM_LOCK"), 1);
   new DLibPro(lib::sem_release, string("SEM_RELEASE"), 1);
-
-  // sort lists
-  sort( libFunList.begin(), libFunList.end(), CompLibFunName());
-  sort( libProList.begin(), libProList.end(), CompLibProName());
 }
 
