@@ -57,11 +57,14 @@ void LibInit_ac()
   const string trisolKey[]={"DOUBLE",KLISTEND};
   new DLibFunRetNew(lib::trisol_fun,string("TRISOL"),4,trisolKey);
 
-  // QSIMP/QROMB/QROMO (we used same code for QSIMP and QROMB)
+  // QSIMP/QROMB (we share code for QSIMP and QROMB)
   const string qsimpKey[]={"DOUBLE","EPS","JMAX",KLISTEND};
-  new DLibFunRetNew(lib::qromb_fun,string("QSIMP"),3,qsimpKey);
-  const string qrombKey[]={"DOUBLE","EPS","JMAX","K",KLISTEND};
-  new DLibFunRetNew(lib::qromb_fun,string("QROMB"),3,qrombKey);
+  const string qsimpWarnKey[]={"K",KLISTEND}; //RSIMP has no 'K'
+  new DLibFunRetNew(lib::qromb_fun,string("QSIMP"),3,qsimpKey,qsimpWarnKey);
+  const string qrombKey[]={"DOUBLE","EPS","JMAX",KLISTEND};
+  const string qrombWarnKey[]={"K",KLISTEND}; //QROMB code does not make use of 'K' key
+  new DLibFunRetNew(lib::qromb_fun,string("QROMB"),3,qrombKey,qrombWarnKey);
+  //QROMO
   const string qromoKey[]={"DOUBLE","EPS","JMAX","K","MIDEXP",
 			   "MIDINF","MIDPNT","MIDSQL","MIDSQU",KLISTEND};
   new DLibFunRetNew(lib::qromo_fun,string("QROMO"),3,qromoKey);
@@ -88,10 +91,10 @@ void LibInit_ac()
   new DLibFunRetNew(lib::matrix_multiply,string("MATRIX_MULTIPLY"),2,matrix_multiplyKey);
 
   // Levan Loria and Alain Coulais, September 2014 : to be extensively tested
-
-  const string smoothKey[]={"NAN",KLISTEND};
-  new DLibFunRetNew(lib::smooth2_fun,string("SMOOTH2"),2,smoothKey);
-  new DLibFunRetNew(lib::smooth3_fun,string("SMOOTH3"),2,smoothKey); 
+// please replace smooth with one of these functions only when thoroughly tested.
+//  const string smoothKey[]={"NAN",KLISTEND};
+//  new DLibFunRetNew(lib::smooth2_fun,string("SMOOTH2"),2,smoothKey);
+//  new DLibFunRetNew(lib::smooth3_fun,string("SMOOTH3"),2,smoothKey); 
 
 }
 
