@@ -111,13 +111,9 @@
 
   DPtr GetLISTNode( EnvUDT* e, DStructGDL* self, DLong targetIx)
   {
-    static DString listName("LIST");
-    static DString cNodeName("GDL_CONTAINER_NODE");
     static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
     static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
-    static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
     static unsigned pNextTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PNEXT");
-    static unsigned pDataTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PDATA");
   
     DPtr actP;
     if( targetIx == -1)
@@ -157,10 +153,6 @@
 
   void LIST__ToStream( DStructGDL* oStructGDL, std::ostream& o, SizeT w, SizeT* actPosPtr)
   {	  
-    static DString listName("LIST");
-    static DString cNodeName("GDL_CONTAINER_NODE");
-// 	  static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
-// 	  static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
     static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
     static unsigned pNextTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PNEXT");
     static unsigned pDataTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PDATA");
@@ -176,8 +168,7 @@
       if( data == NULL) data = NullGDL::GetSingleInstance();
       
       data->ToStream( o, w, actPosPtr);
-      if( (i+1) < nList)
-	o << std::endl;
+      if( (i+1) < nList) o << std::endl;
 
       pActNode = (*static_cast<DPtrGDL*>(actNode->GetTag( pNextTag, 0)))[0];
     }
@@ -190,9 +181,6 @@
     DStructGDL* self = listStruct;
 
     // here static is fine
-    static DString listName("LIST");
-    static DString cNodeName("GDL_CONTAINER_NODE");
-    static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
     static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
     static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
     static unsigned pNextTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PNEXT");
@@ -229,13 +217,10 @@
   void LISTCleanup( EnvUDT* e, DStructGDL* self)
   {
     // here static is fine
-    static DString listName("LIST");
-    static DString cNodeName("GDL_CONTAINER_NODE");
     static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
     static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
     static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
     static unsigned pNextTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PNEXT");
-    static unsigned pDataTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PDATA");
 
     DLong nList = (*static_cast<DLongGDL*>( self->GetTag( nListTag, 0)))[0];	      
   
@@ -311,10 +296,6 @@ BaseGDL* LIST___OverloadEQOp( EnvUDT* e)
   if( nParam < 3) // consider implicit SELF
     ThrowFromInternalUDSub( e, "Two parameters are needed: LEFT, RIGHT.");
 
-  static DString listName("LIST");
-  static DString cNodeName("GDL_CONTAINER_NODE");
-  static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
-  static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
   static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
   static unsigned pNextTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PNEXT");
   static unsigned pDataTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PDATA");
@@ -505,9 +486,26 @@ BaseGDL* LIST___OverloadEQOp( EnvUDT* e)
 	}
 	else
 	{
-	  BaseGDL* rConvert = dataR->Convert2(dataL->Type(),BaseGDL::COPY);
-	  Guard<BaseGDL> rCovertGuard( rConvert);
-	  BaseGDL* eqRes = dataL->EqOp( rConvert);
+	  
+	  BaseGDL* eqRes = NULL;
+
+	  DType aTy=dataL->Type();
+	  DType bTy=dataR->Type();
+	  if( DTypeOrder[aTy] > DTypeOrder[bTy])
+	    {
+	      // convert b to a
+	      BaseGDL* rConvert = dataR->Convert2(dataL->Type(),BaseGDL::COPY);
+	      Guard<BaseGDL> rConvertGuard(rConvert);
+	      eqRes = dataL->EqOp( rConvert);
+	    }
+	  else
+	    {
+	      // convert a to b
+	      BaseGDL* lConvert = dataL->Convert2(dataR->Type(),BaseGDL::COPY);
+	      Guard<BaseGDL> lConvertGuard(lConvert);
+	      eqRes = dataR->EqOp( lConvert);
+	    }
+	  	  
 	  if( eqRes->Type() != GDL_BYTE)
 	  {
 	    Guard<BaseGDL> eqResGuardTmp( eqRes);
@@ -541,8 +539,6 @@ BaseGDL* LIST___OverloadEQOp( EnvUDT* e)
     if( nParam < 3) 
       ThrowFromInternalUDSub( e, "Two parameters are needed: LEFT, RIGHT.");
 
-    static DString listName("LIST");
-    static DString cNodeName("GDL_CONTAINER_NODE");
     static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
     static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
     static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
@@ -669,8 +665,6 @@ BaseGDL* LIST___OverloadBracketsRightSide( EnvUDT* e)
 //   // because of .RESET_SESSION, we cannot use static here
 //   DStructDesc* containerDesc=structDesc::GDL_CONTAINER_NODE;
 
-  static DString listName("LIST");
-  static DString cNodeName("GDL_CONTAINER_NODE");
   static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
   static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
   static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
@@ -781,22 +775,6 @@ BaseGDL* LIST___OverloadBracketsRightSide( EnvUDT* e)
       targetIx = -1;
     DPtr actP = GetLISTNode( e, self, targetIx);
 
-//     DPtr actP;// = (*static_cast<DPtrGDL*>(self->GetTag( pTailTag, 0)))[0];
-//     if( targetIx == (listSize-1)) // 0
-//     {
-//       actP = (*static_cast<DPtrGDL*>( self->GetTag( pHeadTag, 0)))[0];
-//     }
-//     else
-//     {
-//       actP = (*static_cast<DPtrGDL*>(self->GetTag( pTailTag, 0)))[0];
-//       for( SizeT elIx = 0; elIx < targetIx; ++elIx)
-//       {
-// 	DStructGDL* actPStruct = GetStruct( e, actP);
-// 
-// 	actP = (*static_cast<DPtrGDL*>( actPStruct->GetTag( pNextTag, 0)))[0];
-//       }
-//     }
-//    DStructGDL* actPStruct = GetStruct( e, actP);
     DStructGDL* actPStruct = GetLISTStruct( e, actP);   
 
     actP = (*static_cast<DPtrGDL*>(actPStruct->GetTag( pDataTag, 0)))[0];
@@ -848,7 +826,7 @@ BaseGDL* LIST___OverloadBracketsRightSide( EnvUDT* e)
   }
   
   (*static_cast<DPtrGDL*>( listStruct->GetTag( pHeadTag, 0)))[0] = cID;	      
-  (*static_cast<DLongGDL*>( listStruct->GetTag( nListTag, 0)))[0] = listSize + allIx->size();      
+  (*static_cast<DLongGDL*>( listStruct->GetTag( nListTag, 0)))[0] = allIx->size();      
 
   newObjGuard.Release();
   return newObj;
@@ -898,12 +876,7 @@ void LIST___OverloadBracketsLeftSide( EnvUDT* e)
   DObj selfID = (*selfObj)[0];
   DStructGDL* self = e->Interpreter()->GetObjHeap( selfID);
 
-  static DString listName("LIST");
-  static DString cNodeName("GDL_CONTAINER_NODE");
-  static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
-  static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
   static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
-  static unsigned pNextTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PNEXT");
   static unsigned pDataTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PDATA");
 
   // default behavior: Exact like scalar indexing
@@ -1131,10 +1104,8 @@ BaseGDL* list__toarray( EnvUDT* e)
     assert( containerDesc != NULL && containerDesc->NTags() > 0);
 
     // here static is fine
-    static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
     static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
     static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
-    static unsigned pNextTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PNEXT");
     static unsigned pDataTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PDATA");
 
     DLong nList = (*static_cast<DLongGDL*>( self->GetTag( nListTag, 0)))[0];	      
@@ -1252,13 +1223,109 @@ BaseGDL* list__toarray( EnvUDT* e)
       
     }
     catch( GDLException& ex)
-    {
-      ThrowFromInternalUDSub( e, ex.ANTLRException::getMessage());
-    }
+      {
+	ThrowFromInternalUDSub( e, ex.ANTLRException::getMessage());
+      }
     assert(false);
     return NULL;
   }
 
+BaseGDL* list__isempty( EnvUDT* e)
+{
+  static int kwSELFIx = 0;
+  DStructGDL* self = GetSELF( e->GetKW( kwSELFIx), e);
+  static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
+  DLong nList = (*static_cast<DLongGDL*>( self->GetTag( nListTag, 0)))[0];	      
+  if (nList > 0) return new DByteGDL(0); else return new DLongGDL(1);
+}
+
+BaseGDL* list__count( EnvUDT* e)
+{
+    static int kwSELFIx = 0;
+    static int kwVALUEIx = 1;
+    static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
+
+    SizeT nParam = e->NParam(1); // minimum SELF
+	
+    DStructGDL* self = GetSELF( e->GetKW( kwSELFIx), e);
+
+    if( nParam > 1)
+    {
+      BaseGDL* r = e->GetKW( kwVALUEIx);
+    
+      DObjGDL* selfObj = static_cast<DObjGDL*>(e->GetKW( kwSELFIx));
+      
+      DByteGDL* result = static_cast<DByteGDL*>(selfObj->EqOp( r));
+      Guard<DByteGDL> newObjGuard( result);
+      
+      DLong nList = 0;
+      for( SizeT i=0; i<result->N_Elements(); ++i)
+      {
+	if( (*result)[i] != 0)
+	  ++nList;
+      }
+      return new DLongGDL( nList);
+    }
+    
+    DLong nList = (*static_cast<DLongGDL*>( self->GetTag( nListTag, 0)))[0];	      
+    return new DLongGDL( nList);
+  }
+
+  BaseGDL* list__where( EnvUDT* e)
+  {
+    const int kwNCOMPLEMENTIx = 0; 
+    const int kwCOUNTIx = 1; 
+    const int kwCOMPLEMENTIx = 2; 
+    const int kwSELFIx = 3;
+    const int kwVALUEIx = 4;
+
+    SizeT nParam = e->NParam(2); // SELF, VALUE
+
+    BaseGDL* selfP = e->GetKW( kwSELFIx);
+    DStructGDL* self = GetSELF( selfP, e); // checks
+
+    BaseGDL* r = e->GetKW( kwVALUEIx);
+  
+    DObjGDL* selfObj = static_cast<DObjGDL*>(e->GetKW( kwSELFIx));
+
+    BaseGDL* result = selfObj->EqOp( r);
+    Guard<BaseGDL> resultGuard( result);
+
+    DLong nEl = result->N_Elements();
+    
+    SizeT count;
+    DLong* ixList = result->Where( e->KeywordPresent( kwCOMPLEMENTIx), count);
+    
+    if( e->KeywordPresent( kwNCOMPLEMENTIx)) // NCOMPLEMENT
+    {
+	e->SetKW( kwNCOMPLEMENTIx, new DLongGDL( nEl - count));
+    }
+    if( e->KeywordPresent( kwCOUNTIx)) // COUNT
+    {
+	e->SetKW( kwCOUNTIx, new DLongGDL( count));
+    }
+    if( e->KeywordPresent( kwCOMPLEMENTIx)) // COMPLEMENT
+    {
+        SizeT nCount = nEl - count;
+	if( nCount == 0)
+	{
+	  e->SetKW( kwCOMPLEMENTIx, NullGDL::GetSingleInstance());
+	}
+	else
+	{
+	  DLongGDL* cIxList = new DLongGDL( dimension( &nCount, 1), BaseGDL::NOZERO);	  
+	  SizeT cIx = nEl;
+	  for( SizeT i=0; i<nCount; ++i)
+	      (*cIxList)[ i] = ixList[ --cIx];
+	  e->SetKW( kwCOMPLEMENTIx, cIxList);
+	}
+    }
+    
+    if( count == 0)
+      return NullGDL::GetSingleInstance();
+    else
+      return new DLongGDL( ixList, count);
+  }
   
   
   BaseGDL* list__remove( EnvUDT* e, bool asFunction);
@@ -1290,8 +1357,6 @@ BaseGDL* list__toarray( EnvUDT* e)
 //   DStructDesc* containerDesc= structDesc::GDL_CONTAINER_NODE;
 
   // here static is fine
-  static DString listName("LIST");
-  static DString cNodeName("GDL_CONTAINER_NODE");
   static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
   static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
   static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
@@ -1662,13 +1727,10 @@ BaseGDL* list__toarray( EnvUDT* e)
 
     DStructGDL* self = GetSELF( e->GetKW( 0), e);
   
-    static DString listName("LIST");
-    static DString cNodeName("GDL_CONTAINER_NODE");
     static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
     static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
     static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
     static unsigned pNextTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PNEXT");
-    static unsigned pDataTag = structDesc::GDL_CONTAINER_NODE->TagIndex( "PDATA");
     
     DLong nList = (*static_cast<DLongGDL*>( self->GetTag( nListTag, 0)))[0];	      
 
@@ -1719,8 +1781,6 @@ BaseGDL* list__toarray( EnvUDT* e)
   
   DStructGDL* self = GetSELF( e->GetKW( kwSELFIx), e);
   
-  static DString listName("LIST");
-  static DString cNodeName("GDL_CONTAINER_NODE");
   static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
   static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
   static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
@@ -1814,7 +1874,7 @@ BaseGDL* list__toarray( EnvUDT* e)
 	DPtr pTail = (*static_cast<DPtrGDL*>( listStruct->GetTag( pTailTag, 0)))[0];	      
 	
 	(*static_cast<DPtrGDL*>( cStruct->GetTag( pNextTag, 0)))[0] = pTail;
-	(*static_cast<DPtrGDL*>( listStruct->GetTag( pTail, 0)))[0] = firstID;	      
+	(*static_cast<DPtrGDL*>( listStruct->GetTag( pTailTag, 0)))[0] = firstID;	      
       }
       else
       {
@@ -1864,7 +1924,7 @@ BaseGDL* list__toarray( EnvUDT* e)
 	DPtr pTail = (*static_cast<DPtrGDL*>( listStruct->GetTag( pTailTag, 0)))[0];	      
 	
 	(*static_cast<DPtrGDL*>( cStruct->GetTag( pNextTag, 0)))[0] = pTail;
-	(*static_cast<DPtrGDL*>( listStruct->GetTag( pTail, 0)))[0] = cID;	      
+	(*static_cast<DPtrGDL*>( listStruct->GetTag( pTailTag, 0)))[0] = cID;	      
       }
       else
       {
@@ -1903,22 +1963,8 @@ BaseGDL* list__toarray( EnvUDT* e)
 	listLength = 0;
     }
     
-    ProgNodeP cN = e->CallingNode();
     DInterpreter* ip = e->Interpreter();
     
-//     static DString listName("LIST");
-//     static DString cNodeName("GDL_CONTAINER_NODE");
-//     
-// 
-//     // here static is fine
-//     static unsigned pHeadTag = listDesc->TagIndex( "PHEAD");
-//     static unsigned pTailTag = listDesc->TagIndex( "PTAIL");
-//     static unsigned nListTag = listDesc->TagIndex( "NLIST");
-// 
-//     static unsigned pNextTag = containerDesc->TagIndex( "PNEXT");
-//     static unsigned pDataTag = containerDesc->TagIndex( "PDATA");
-    static DString listName("LIST");
-    static DString cNodeName("GDL_CONTAINER_NODE");
     static unsigned pHeadTag = structDesc::LIST->TagIndex( "PHEAD");
     static unsigned pTailTag = structDesc::LIST->TagIndex( "PTAIL");
     static unsigned nListTag = structDesc::LIST->TagIndex( "NLIST");
@@ -2045,9 +2091,6 @@ BaseGDL* list__toarray( EnvUDT* e)
       }
     }
 
-//     if( cStruct != NULL)
-//       (*static_cast<DPtrGDL*>( cStruct->GetTag( pNextTag, 0)))[0] = 0;
-	    
     (*static_cast<DPtrGDL*>( listStruct->GetTag( pHeadTag, 0)))[0] = cID;	      
     (*static_cast<DLongGDL*>( listStruct->GetTag( nListTag, 0)))[0] = added;	      
 
