@@ -119,9 +119,9 @@
     e->Throw("Complex not implemented (GSL limitation). ");
 
 
-// Use this macro to define Inf and NaN, number of elements and result in a function with one parameter.
+// Use this macro to define Inf and NaN, number of elements and result in a function with one parameter.  Note no STATIC for SysVar::, due to .reset 
 #define GM_DF1()							\
-  static DStructGDL *Values =  SysVar::Values();			\
+  DStructGDL *Values =  SysVar::Values();			\
   DDouble d_infinity=(*static_cast<DDoubleGDL*>(Values->GetTag(Values->Desc()->TagIndex("D_INFINITY"), 0)))[0]; \
   DDouble d_nan=(*static_cast<DDoubleGDL*>(Values->GetTag(Values->Desc()->TagIndex("D_NAN"), 0)))[0]; \
   									\
@@ -129,33 +129,26 @@
 
 // Use this macro to define Inf and NaN, number of elements and result in a function with two parameters.
 #define GM_DF2()							\
-  static DStructGDL *Values =  SysVar::Values();			\
+  DStructGDL *Values =  SysVar::Values();			\
   DDouble d_infinity=(*static_cast<DDoubleGDL*>(Values->GetTag(Values->Desc()->TagIndex("D_INFINITY"), 0)))[0]; \
   DDouble d_nan=(*static_cast<DDoubleGDL*>(Values->GetTag(Values->Desc()->TagIndex("D_NAN"), 0)))[0]; \
   									\
   DDoubleGDL* res;							\
-  if (nElp0 == 1 && nElp1 == 1)						\
-  {                                                                     \
-    if (p0->Rank() == 0 && p1->Rank() == 0)                             \
-      res = new DDoubleGDL(BaseGDL::NOZERO);				\
-    else                                                                \
-      res = new DDoubleGDL(1, BaseGDL::NOZERO);				\
-  }                                                                     \
-  else if (nElp0 > 1 && nElp1 == 1)					\
-    res = new DDoubleGDL(p0->Dim(), BaseGDL::NOZERO);			\
-  else if (nElp0 == 1 && nElp1 > 1)					\
+									\
+  if (p0->Rank() == 0)							\
     res = new DDoubleGDL(p1->Dim(), BaseGDL::NOZERO);			\
-  else if (nElp0 <= nElp1)						\
+  else if (p1->Rank() == 0)						\
     res = new DDoubleGDL(p0->Dim(), BaseGDL::NOZERO);			\
+  else if (p0->N_Elements() > p1->N_Elements())				\
+    res = new DDoubleGDL(p1->Dim(), BaseGDL::NOZERO);			\
   else									\
-    res = new DDoubleGDL(p1->Dim(), BaseGDL::NOZERO);			\
-  									\
+    res = new DDoubleGDL(p0->Dim(), BaseGDL::NOZERO);			\
   SizeT nElp = res->N_Elements();					\
+									\
   //cout << "nElp0 : " << nElp0 << std::endl;				\
   //cout << "nElp1 : " << nElp1 << std::endl;				\
   //cout << "nElp  : " << nElp  << std::endl;
 
-
   // Use this macro to convert result in a function with one parameter.
 #define GM_CV1() \
   static DInt doubleKWIx = e->KeywordIx("DOUBLE");			\
