@@ -55,8 +55,8 @@ extern GDLFileListT  fileUnits;
 const SizeT DefaultTPOOL_MIN_ELTS = 100000;
 const SizeT DefaultTPOOL_MAX_ELTS = 0;
 extern DLong CpuTPOOL_NTHREADS;
-extern DLong CpuTPOOL_MIN_ELTS;
-extern DLong CpuTPOOL_MAX_ELTS;
+extern DLong64 CpuTPOOL_MIN_ELTS;
+extern DLong64 CpuTPOOL_MAX_ELTS;
 
 //extern DeviceListT   deviceList;
 //extern Graphics*     actDevice;
@@ -95,6 +95,8 @@ int LibProIx(const std::string& n);
 int LibFunIx(const std::string& n);
 
 bool IsFun(antlr::RefToken); // used by Lexer and Parser
+bool IsRelaxed(); //tells if syntax is not strict (i.e. parenthesis for array indexes).
+void SetStrict(bool value);
 
 bool BigEndian();
 
