@@ -40,7 +40,7 @@ namespace lib {
     e->AssureDoubleScalarKWIfPresent ( zvIx, zValue );
     //T3D
     static int t3dIx = e->KeywordIx( "T3D");
-    doT3d=(e->KeywordSet(t3dIx) || T3Denabled(e));
+    doT3d=(e->KeywordSet(t3dIx) || T3Denabled());
 
     DFloat xMarginL, xMarginR, yMarginB, yMarginT; 
 
@@ -54,17 +54,17 @@ namespace lib {
     //IDL behaviour for XAXIS and YAXIS and ZAXIS options: only one option is considered, and ZAXIS above YAXIS above XAXIS
     if( (e->GetKW( xaxisIx) != NULL) ) {
       xAxis = true;
-      e->AssureLongScalarKWIfPresent( "XAXIS", xaxis_value);
+      e->AssureLongScalarKWIfPresent(xaxisIx, xaxis_value);
       if (xaxis_value == 0) {standardNumPos = true;} else {standardNumPos = false;}
     }      
     if( e->GetKW( yaxisIx) != NULL) {
       yAxis = true; xAxis = false; // like in IDL, yaxis overrides xaxis
-      e->AssureLongScalarKWIfPresent( "YAXIS", yaxis_value);
+      e->AssureLongScalarKWIfPresent( yaxisIx, yaxis_value);
       if (yaxis_value == 0) {standardNumPos = true;} else {standardNumPos = false;}
     }
     if( e->GetKW( zaxisIx) != NULL) {
       zAxis = true; xAxis = false; yAxis=false; // like in IDL, zaxis overrides all
-      e->AssureLongScalarKWIfPresent( "ZAXIS", zaxis_value);
+      e->AssureLongScalarKWIfPresent( zaxisIx, zaxis_value);
     }
     if( (e->GetKW( xaxisIx) == NULL) && (e->GetKW( yaxisIx) == NULL )  && ((e->GetKW( zaxisIx) == NULL )||!doT3d))  {
       xAxis = true; standardNumPos = true; 
@@ -84,75 +84,115 @@ namespace lib {
     xLog=xAxisWasLog;
     yLog=yAxisWasLog; //by default logness is similar until another option is set
     zLog=zAxisWasLog;
-
+    
     enum
     {
       DATA=0,
       NORMAL,
       DEVICE
     } coordinateSystem=DATA;
+    static int DATAIx=e->KeywordIx("DATA");
+    static int DEVICEIx=e->KeywordIx("DEVICE");
+    static int NORMALIx=e->KeywordIx("NORMAL");
+    static int SAVEIx=e->KeywordIx("SAVE");
     //check presence of DATA,DEVICE and NORMAL options
-    if ( e->KeywordSet("DATA") ) coordinateSystem=DATA;
-    if ( e->KeywordSet("DEVICE") ) coordinateSystem=DEVICE;
-    if ( e->KeywordSet("NORMAL") ) coordinateSystem=NORMAL;
+    if ( e->KeywordSet(DATAIx) ) coordinateSystem=DATA;
+    if ( e->KeywordSet(DEVICEIx) ) coordinateSystem=DEVICE;
+    if ( e->KeywordSet(NORMALIx) ) coordinateSystem=NORMAL;
 
-    // x and y range, old and new
-    DDouble oxStart, oxEnd;
-    DDouble oyStart, oyEnd;
-    DDouble xStart, xEnd;
-    DDouble yStart, yEnd;
     // get viewport coordinates in normalised units
     PLFLT ovpXL, ovpXR, ovpYB, ovpYT;
-    actStream->gvpd(ovpXL, ovpXR, ovpYB, ovpYT);
-    //undefined previous viewport, seems IDL returns without complain:
-    if ((ovpXL==0.0 && ovpXR==0.0) || (ovpYB==0.0 && ovpYT==0.0)) return;
+    gdlGetCurrentAxisWindow("X", ovpXL, ovpXR);
+    gdlGetCurrentAxisWindow("Y", ovpYB, ovpYT);
+    //undefined or null previous viewport, seems IDL returns without complain:
+    if ((ovpXL==ovpXR) || (ovpYB==ovpYT)) return;
+
+
+    // old x and y range
+    DDouble oxStart, oxEnd;
+    DDouble oyStart, oyEnd;
+
+    // get ![XY].CRANGE
+    gdlGetCurrentAxisRange("X", oxStart, oxEnd, FALSE); //ignore projection limits, convert to linear values if necessary.
+    gdlGetCurrentAxisRange("Y", oyStart, oyEnd, FALSE);
+
+    if ((oyStart == oyEnd) || (oxStart == oxEnd))
+    {
+      if (oyStart != 0.0 && oyStart == oyEnd){
+        oyStart = 0;
+        oyEnd = 1;
+      }
+      if (oxStart != 0.0 && oxStart == oxEnd){
+        oxStart = 0;
+        oxEnd = 1;
+      }
+    }
+
     PLFLT ovpSizeX, ovpSizeY;
     ovpSizeX=ovpXR-ovpXL;
     ovpSizeY=ovpYT-ovpYB;
-    //get wiewport window in world coordinates
-    PLFLT xmin, xmax, ymin, ymax;
-    actStream->gvpw(xmin, xmax, ymin, ymax);
-    xStart=oxStart=xmin;
-    xEnd=oxEnd=xmax;
-    yStart=oyStart=ymin;
-    yEnd=oyEnd=ymax;
-    //convert these values to real values if box was log
-    if (xAxisWasLog) {xStart=pow(10,xStart);xEnd=pow(10,xEnd);}
-    if (yAxisWasLog) {yStart=pow(10,yStart);yEnd=pow(10,yEnd);}
-
+    
+    // new x and y range, real values (not log)
+    DDouble xStart=oxStart;
+    DDouble xEnd=oxEnd;
+    DDouble yStart=oyStart;
+    DDouble yEnd=oyEnd;
 
     // handle Log options passing via Keywords
-    static int xTypeIx = e->KeywordIx("XTYPE");
-    static int yTypeIx = e->KeywordIx("YTYPE");
-    static int xLogIx = e->KeywordIx("XLOG");
-    static int yLogIx = e->KeywordIx("YLOG");
+    // note: undocumented keywords [xyz]type still exist and
+    // have priority on [xyz]log !
+    static int xTypeIx = e->KeywordIx( "XTYPE" );
+    static int yTypeIx = e->KeywordIx( "YTYPE" );
+    static int xLogIx = e->KeywordIx( "XLOG" );
+    static int yLogIx = e->KeywordIx( "YLOG" );
+    static int xTickunitsIx = e->KeywordIx( "XTICKUNITS" );
+    static int yTickunitsIx = e->KeywordIx( "YTICKUNITS" );
 
-    // if (e->KeywordPresent(xLogIx)) xLog = e->KeywordSet(xLogIx);
-    // if (e->KeywordPresent(yLogIx)) yLog = e->KeywordSet(yLogIx);
-    
-    if (e->KeywordPresent(xTypeIx )) {
-      xLog= ( xLog || e->KeywordSet (xTypeIx ));
-    } else {
-      xLog= ( xLog || e->KeywordSet (xLogIx ));
+    if ( e->KeywordPresent( xTypeIx ) ) xLog = e->KeywordSet( xTypeIx ); else xLog = e->KeywordSet( xLogIx );
+    if ( e->KeywordPresent( yTypeIx ) ) yLog = e->KeywordSet( yTypeIx ); else yLog = e->KeywordSet( yLogIx );
+
+    if ( xLog && e->KeywordSet( xTickunitsIx ) ) {
+      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
+      xLog = FALSE;
     }
-    
-    if (e->KeywordPresent(yTypeIx )) {
-      yLog= ( yLog || e->KeywordSet (yTypeIx ));
-    } else {
-      yLog= ( yLog ||e->KeywordSet (yLogIx));
+    if ( yLog && e->KeywordSet( yTickunitsIx ) ) {
+      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
+      yLog = FALSE;
     }
 
-    //YNOZERO corrects yStart
-    if ( e->KeywordSet( "YNOZERO") && yStart >0 && !yLog ) yStart=0.0;
-	bool setdummy;
-    setdummy=gdlGetDesiredAxisRange( e, "X", xStart, xEnd);
-	setdummy=gdlGetDesiredAxisRange( e, "Y", yStart, yEnd);
-    if (xStart == xEnd && yStart == yEnd) {
-      e->Throw("Invalid plotting ranges.  Set up a plot window first.");
+    //XRANGE and YRANGE overrides all that, but  Start/End should be recomputed accordingly
+    DDouble xAxisStart, xAxisEnd, yAxisStart, yAxisEnd;
+    bool setx=gdlGetDesiredAxisRange(e, "X", xAxisStart, xAxisEnd);
+    bool sety=gdlGetDesiredAxisRange(e, "Y", yAxisStart, yAxisEnd);
+    if (sety)
+    {
+      yStart=yAxisStart;
+      yEnd=yAxisEnd;
     }
+    if (setx)
+    {
+      xStart=xAxisStart;
+      xEnd=xAxisEnd;
+    }
+    //handle Nozero option after all that!
+    if(!gdlYaxisNoZero(e) && yStart >0 && !yLog ) yStart=0.0;
+
     gdlHandleUnwantedAxisValue(xStart, xEnd, xLog);
     gdlHandleUnwantedAxisValue(yStart, yEnd, yLog);
 
+    // [XY]STYLE
+    DLong xStyle=0, yStyle=0;
+    gdlGetDesiredAxisStyle(e, "X", xStyle);
+    gdlGetDesiredAxisStyle(e, "Y", yStyle);
+
+     //xStyle and yStyle apply on range values
+    if ((xStyle & 1) != 1) {
+      PLFLT intv = gdlAdjustAxisRange(xStart, xEnd, xLog);
+    }
+    if ((yStyle & 1) != 1) {
+      PLFLT intv = gdlAdjustAxisRange(yStart, yEnd, yLog);
+    }
+    
     DDouble yVal, xVal;
     //in absence of arguments we will have:
     yVal=(standardNumPos)?oyStart:oyEnd;
@@ -186,19 +226,52 @@ namespace lib {
       vpX=xVal;
       vpY=yVal;
     }
-    else
+    else //DATA
     {
+
+#ifdef USE_LIBPROJ4
+      // Map Stuff (xtype = 3)
+      LPTYPE idata;
+      XYTYPE odata;
+      bool mapSet=false;
+      get_mapset(mapSet);
+      mapSet=(mapSet && coordinateSystem==DATA);
+      if ( mapSet )
+      {
+        ref=map_init();
+        if ( ref==NULL )
+        {
+          e->Throw("Projection initialization failed.");
+        }
+        idata.u=xVal * DEG_TO_RAD;
+        idata.v=yVal * DEG_TO_RAD;
+        odata=PJ_FWD(idata, ref);
+        xVal=odata.u;
+        yVal=odata.v;
+        DDouble *sx, *sy;
+        GetSFromPlotStructs( &sx, &sy );
+
+        DFloat *wx, *wy;
+        GetWFromPlotStructs( &wx, &wy );
+
+        DDouble pxStart, pxEnd, pyStart, pyEnd;
+        DataCoordLimits( sx, sy, wx, wy, &pxStart, &pxEnd, &pyStart, &pyEnd, true );
+        actStream->vpor( wx[0], wx[1], wy[0], wy[1] );
+        actStream->wind( pxStart, pxEnd, pyStart, pyEnd );
+      }
+#endif
       if (xAxisWasLog) xVal=log10(xVal);
       if (yAxisWasLog) yVal=log10(yVal);
+      if ( !isfinite(xVal)|| !isfinite(yVal) ) return; //no plot
       actStream->WorldToNormedDevice(xVal, yVal, vpX, vpY);
     }
     //compute new temporary viewport in relative coords
 #define ADDEPSILON 0.1
     if ( standardNumPos )
     {
-      vpXL=(xAxis)?ovpXL:vpX;
-      vpXR=(xAxis)?ovpXR:vpX+ovpSizeY;
-      vpYB=(xAxis)?vpY:ovpYB;
+      vpXL=(xAxis)?ovpXL       :vpX;
+      vpXR=(xAxis)?ovpXR       :vpX+ovpSizeY;
+      vpYB=(xAxis)?vpY         :ovpYB;
       vpYT=(xAxis)?vpY+ovpSizeX:ovpYT;
     }
     else
@@ -208,7 +281,7 @@ namespace lib {
       vpYB=(xAxis)?vpY-ovpSizeX:ovpYB;
       vpYT=(xAxis)?vpY:ovpYT;
     }
-
+     
     actStream->OnePageSaveLayout(); // one page
 
     actStream->vpor(vpXL, vpXR, vpYB, vpYT);
@@ -220,7 +293,7 @@ namespace lib {
     { //special name "axisX" needed because we artificially changed size of box
       gdlAxis(e, actStream, "axisX", xStart, xEnd, xLog, standardNumPos?1:2, ovpSizeY);
 
-      if ( e->KeywordSet("SAVE") )
+      if ( e->KeywordSet(SAVEIx) )
       {
         gdlStoreAxisCRANGE("X", xStart, xEnd, xLog);
         gdlStoreAxisType("X", xLog);
@@ -232,7 +305,7 @@ namespace lib {
     {//special name "axisY" needed because we artificially changed size of box
       gdlAxis(e, actStream, "axisY", yStart, yEnd, yLog, standardNumPos?1:2, ovpSizeX);
 
-      if ( e->KeywordSet("SAVE") )
+      if ( e->KeywordSet(SAVEIx) )
       {
         gdlStoreAxisCRANGE("Y", yStart, yEnd, yLog);
         gdlStoreAxisType("Y", yLog);
@@ -241,6 +314,11 @@ namespace lib {
     }
     // reset the viewport and world coordinates to the original values
     actStream->RestoreLayout();
+    // if save, update world coordinates to the new values
+    if ( e->KeywordSet(SAVEIx) )
+    {
+      actStream->wind(xStart, xEnd, yStart, yEnd);
+    }
 
   }
 
