@@ -27,7 +27,7 @@ void empty(EnvT* e)
   GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
   if (actDevice->Name() == "X")
   {
-    GDLGStream *actStream = actDevice->GetStream();
+    GDLGStream *actStream = actDevice->GetStream(false);//this command SHOULD NOT open a window if none opened.
     if (actStream != NULL) actStream->Flush();
   }
 }
@@ -70,7 +70,10 @@ void tvcrs( EnvT* e)
 
   PLFLT ix,iy;
 
-  if (e->KeywordSet("DATA")) // /DATA
+  static int DATA = e->KeywordIx("DATA");
+  static int NORMAL = e->KeywordIx("NORMAL");
+  static int DEVICE = e->KeywordIx("DEVICE");
+  if (e->KeywordSet(DATA)) // /DATA
   {
     DDouble tempx,tempy;
     tempx=(*x)[0];
@@ -91,7 +94,9 @@ void tvcrs( EnvT* e)
       tempx= sx[0] +odata.u * sx[1];
       tempy= sy[0] +odata.v * sy[1]; //normed values
       actStream->NormedDeviceToDevice(tempx,tempy,ix,iy);
-      actStream->WarpPointer(ix,iy);
+      DLong iix=ix;
+      DLong iiy=iy;
+      actStream->WarpPointer(iix,iiy);
       actStream->Flush();
       actStream->UnsetFocus();
       return;
@@ -104,16 +109,18 @@ void tvcrs( EnvT* e)
      if(yLog) tempy=pow(10,tempy);
     actStream->WorldToDevice(tempx,tempy,ix,iy);
   }
-  else if (e->KeywordSet("NORMAL"))
+  else if (e->KeywordSet(NORMAL))
   {
     actStream->NormedDeviceToDevice((*x)[0],(*y)[0],ix,iy);
   }
-  else // (e->KeywordSet("DEVICE"))
+  else // (e->KeywordSet(DEVICE))
   {
     ix=(*x)[0];
     iy=(*y)[0];
   }
-  actStream->WarpPointer(ix,iy);
+  DLong iix=ix;
+  DLong iiy=iy;
+  actStream->WarpPointer(iix,iiy);
   actStream->Flush();
   actStream->UnsetFocus();
 
@@ -168,14 +175,19 @@ void cursor(EnvT* e){
   {
     e->AssureLongScalarPar(2, wait);
   }
-  if (e->KeywordSet("NOWAIT")) wait=NOWAIT;
-  if (e->KeywordSet("CHANGE")) wait=CHANGE;
-  if (e->KeywordSet("WAIT")) wait=WAIT;
-  if (e->KeywordSet("DOWN")) wait=DOWN;
-  if (e->KeywordSet("UP")) wait=UP;
-  PLFLT xp, yp;
-  PLINT xleng, yleng, xoff, yoff;
-  actStream->gpage(xp, yp, xleng, yleng, xoff, yoff);
+  static int NOWAITIx = e->KeywordIx("NOWAIT");
+  static int CHANGEIx = e->KeywordIx("CHANGE");
+  static int WAITIx   = e->KeywordIx("WAIT");
+  static int DOWNIx   = e->KeywordIx("DOWN");
+  static int UPIx     = e->KeywordIx("UP");
+  static int DEVICEIx = e->KeywordIx("DEVICE");
+  static int NORMALIx = e->KeywordIx("NORMAL");
+  
+  if (e->KeywordSet(NOWAITIx)) wait=NOWAIT;
+  if (e->KeywordSet(CHANGEIx)) wait=CHANGE;
+  if (e->KeywordSet(WAITIx)) wait=WAIT;
+  if (e->KeywordSet(DOWNIx)) wait=DOWN;
+  if (e->KeywordSet(UPIx)) wait=UP;
   if(actStream->GetGin(&gin, wait)==false) return;
   // outside window report -1 -1 at least for DEVICE values
   if (gin.pX < 0 || gin.pX > actStream->xPageSize() || gin.pY < 0 || gin.pY > actStream->yPageSize())
@@ -183,7 +195,7 @@ void cursor(EnvT* e){
     gin.pX = -1;
     gin.pY = -1;
   }
-  if (e->KeywordSet("DEVICE"))
+  if (e->KeywordSet(DEVICEIx))
   {
     DLongGDL* xLong;
     DLongGDL* yLong;
@@ -197,7 +209,7 @@ void cursor(EnvT* e){
   {
     DDoubleGDL* x;
     DDoubleGDL* y;
-    if (e->KeywordSet("NORMAL"))
+    if (e->KeywordSet(NORMALIx))
     {
       x = new DDoubleGDL(gin.dX);
       y = new DDoubleGDL(gin.dY);
