@@ -18,37 +18,55 @@
 #include "includefirst.hpp"
 #include "plotting.hpp"
 
+#ifdef _MSC_VER
+#define isnan _isnan
+#endif
+
 namespace lib {
 
   using namespace std;
+//  using std::isinf;
+#ifndef _MSC_VER
+  using std::isnan;
+#endif
 
   class oplot_call : public plotting_routine_call 
   {
-    DDoubleGDL *yVal, *xVal, *zVal, *xTemp, *yTemp;
+    DDoubleGDL *yVal, *xVal, *xTemp, *yTemp;
     SizeT xEl, yEl, zEl;
-    Guard<BaseGDL> xval_guard,yval_guard, zval_guard, xtempval_guard;
+    DDouble minVal, maxVal, xStart, xEnd, yStart, yEnd, zValue;
+    bool doMinMax;
+    bool xLog, yLog, wasBadxLog, wasBadyLog;
+    Guard<BaseGDL> xval_guard, yval_guard, xtemp_guard;
+    DLong iso;
     bool doT3d;
-    DDouble zValue;
 
-    private: bool handle_args( EnvT* e) // {{{
+private:
+
+  bool handle_args(EnvT* e) 
     {
+
       //T3D?
       static int t3dIx = e->KeywordIx( "T3D");
-      doT3d=(e->KeywordSet(t3dIx)|| T3Denabled(e));
+      doT3d=(e->KeywordSet(t3dIx)|| T3Denabled());
 
       //note: Z (VALUE) will be used uniquely if Z is not effectively defined.
-      zValue=0.0;
       static int zvIx = e->KeywordIx( "ZVALUE");
+      zValue=0.0;
       e->AssureDoubleScalarKWIfPresent ( zvIx, zValue );
+      zValue=min(zValue,0.999999); //to avoid problems with plplot
+      zValue=max(zValue,0.0);
 
-      bool polar=FALSE;
-      DLong nsum=1;
-      e->AssureLongScalarKWIfPresent( "NSUM", nsum);
-      if ( e->KeywordSet( "POLAR"))
-      {
-        polar=TRUE;
-      }
+    // system variable !P.NSUM first
+      DLong nsum=(*static_cast<DLongGDL*>(SysVar::P()-> GetTag(SysVar::P()->Desc()->TagIndex("NSUM"), 0)))[0];
+      static int NSUMIx = e->KeywordIx( "NSUM");
+      e->AssureLongScalarKWIfPresent( NSUMIx, nsum);
 
+      static int polarIx = e->KeywordIx( "POLAR");
+      bool polar = (e->KeywordSet(polarIx));
+
+//    DDoubleGDL *yValBis, *xValBis;
+//    Guard<BaseGDL> xvalBis_guard, yvalBis_guard;
       //test and transform eventually if POLAR and/or NSUM!
       if( nParam() == 1)
       {
@@ -58,7 +76,7 @@ namespace lib {
         yEl=yTemp->N_Elements();
         xEl=yEl;
         xTemp = new DDoubleGDL( dimension( xEl), BaseGDL::INDGEN);
-        xtempval_guard.Reset( xTemp); // delete upon exit
+        xtemp_guard.Reset( xTemp); // delete upon exit
       }
       else
       {
@@ -66,7 +84,6 @@ namespace lib {
         if (xTemp->Rank() == 0)
           e->Throw("Expression must be an array in this context: "+e->GetParString(0));
         xEl=xTemp->N_Elements();
-
         yTemp = e->GetParAs< DDoubleGDL>( 1);
         if (yTemp->Rank() == 0)
           e->Throw("Expression must be an array in this context: "+e->GetParString(1));
@@ -83,6 +100,7 @@ namespace lib {
       //check nsum validity
       nsum=max(1,nsum);
       nsum=min(nsum,(DLong)xEl);
+
       if (nsum == 1)
       {
         if (polar)
@@ -96,7 +114,7 @@ namespace lib {
         }
         else
         { //careful about previously set autopointers!
-          if (nParam() == 1) xval_guard.Init( xtempval_guard.release());
+          if (nParam() == 1) xval_guard.Init( xtemp_guard.release());
           xVal = xTemp;
           yVal = yTemp;
         }
@@ -104,7 +122,7 @@ namespace lib {
       else
       {
         int i, j, k;
-        DLong size = xEl / nsum;
+        DLong size = (DLong)xEl / nsum;
         xVal = new DDoubleGDL(size, BaseGDL::ZERO); //SHOULD BE ZERO, IS NOT!
         xval_guard.Reset(xVal); // delete upon exit
         yVal = new DDoubleGDL(size, BaseGDL::ZERO); //IDEM
@@ -134,25 +152,13 @@ namespace lib {
           }
         }
       }
-      if (doT3d)
-      {
-        //make zVal
-        zEl=xVal->N_Elements();
-        zVal=new DDoubleGDL(dimension(zEl), BaseGDL::NOZERO);
-        zval_guard.Reset(zVal); // delete upon exit
-        for (SizeT i=0; i< zEl ; ++i) (*zVal)[i]=zValue;
-      }
-	  return 0;
+    return false;
     }
 
   private: void old_body( EnvT* e, GDLGStream* actStream) 
   {
     DLong psym;
 
-    // get ![XY].CRANGE
-    DDouble xStart, xEnd, yStart, yEnd;
-    gdlGetCurrentAxisRange("X", xStart, xEnd);
-    gdlGetCurrentAxisRange("Y", yStart, yEnd);
     DDouble minVal, maxVal;
     bool doMinMax;
 
@@ -161,50 +167,63 @@ namespace lib {
     gdlGetAxisType("X", xLog);
     gdlGetAxisType("Y", yLog);
 
-    if ((yStart == yEnd) || (xStart == xEnd))
-    {
-      if (yStart != 0.0 && yStart == yEnd)
-        Message("OPLOT: !Y.CRANGE ERROR, setting to [0,1]");
-      yStart = 0; //yVal->min();
-      yEnd = 1; //yVal->max();
+   GetCurrentUserLimits(actStream, xStart, xEnd, yStart, yEnd);
 
-      if (xStart != 0.0 && xStart == xEnd)
-        Message("OPLOT: !X.CRANGE ERROR, setting to [0,1]");
-      xStart = 0; //xVal->min();
-      xEnd = 1; //xVal->max();
-    }
-    
     //now we can setup minVal and maxVal to defaults: Start-End and overload if KW present
 
     minVal = yStart; //to give a reasonable value...
     maxVal = yEnd;   //idem
     doMinMax = false; //although we will not use it...
-    if( e->KeywordSet( "MIN_VALUE") || e->KeywordSet( "MAX_VALUE"))
+    static int MIN_VALUEIx = e->KeywordIx("MIN_VALUE");
+    static int MAX_VALUEIx = e->KeywordIx("MAX_VALUE");
+    if( e->KeywordSet(MIN_VALUEIx) || e->KeywordSet(MAX_VALUEIx))
       doMinMax = true; //...unless explicitely required
-    e->AssureDoubleScalarKWIfPresent( "MIN_VALUE", minVal);
-    e->AssureDoubleScalarKWIfPresent( "MAX_VALUE", maxVal);
+    e->AssureDoubleScalarKWIfPresent( MIN_VALUEIx, minVal);
+    e->AssureDoubleScalarKWIfPresent( MAX_VALUEIx, maxVal);
 
-    int noclipvalue=0;
-    e->AssureLongScalarKWIfPresent( "NOCLIP", noclipvalue);
-    // Clipping is enabled by default for OPLOT.
-    // make all clipping computations BEFORE setting graphic properties (color, size)
-    bool doClip=(e->KeywordSet("CLIP")||noclipvalue==1);
+//    int noclipvalue=0;
+//    static int NOCLIPIx = e->KeywordIx("NOCLIP");
+//    e->AssureLongScalarKWIfPresent( NOCLIPIx, noclipvalue);
+//    // Clipping is enabled by default for OPLOT.
+//    // make all clipping computations BEFORE setting graphic properties (color, size)
+//    bool doClip=(e->KeywordSet(CLIPIx)||!(noclipvalue==0));
     bool stopClip=false;
-    if ( doClip )  if ( startClipping(e, actStream, false)==TRUE ) stopClip=true;
+//    if ( doClip )  
+      if ( startClipping(e, actStream)==true ) stopClip=true;
 
     // start drawing. Graphic Keywords accepted:CLIP(YES), COLOR(YES), LINESTYLE(YES), NOCLIP(YES),
     //                                          PSYM(YES), SYMSIZE(YES), T3D(YES), ZVALUE(YES)
     gdlSetGraphicsForegroundColorFromKw(e, actStream);
+    
+    // it is important to fix symsize before changing vpor or win 
     gdlGetPsym(e, psym);
     gdlSetPenThickness(e, actStream);
     gdlSetSymsize(e, actStream);
     gdlSetLineStyle(e, actStream);
 
-    static DDouble x0,y0,xs,ys; //conversion to normalized coords
-    x0=(xLog)?-log10(xStart):-xStart;
-    y0=(yLog)?-log10(yStart):-yStart;
-    xs=(xLog)?(log10(xEnd)-log10(xStart)):xEnd-xStart;xs=1.0/xs;
-    ys=(yLog)?(log10(yEnd)-log10(yStart)):yEnd-yStart;ys=1.0/ys;
+    bool mapSet=false;
+#ifdef USE_LIBPROJ4
+    get_mapset(mapSet);
+    if ( mapSet )
+    {
+      ref=map_init();
+      if ( ref==NULL )
+      {
+        e->Throw("Projection initialization failed.");
+      }
+        DDouble *sx, *sy;
+        GetSFromPlotStructs( &sx, &sy );
+
+        DFloat *wx, *wy;
+        GetWFromPlotStructs( &wx, &wy );
+
+        DDouble pxStart, pxEnd, pyStart, pyEnd;
+        DataCoordLimits( sx, sy, wx, wy, &pxStart, &pxEnd, &pyStart, &pyEnd, true );
+//        actStream->OnePageSaveLayout(); // one page
+        actStream->vpor( wx[0], wx[1], wy[0], wy[1] );
+        actStream->wind( pxStart, pxEnd, pyStart, pyEnd );
+    }
+#endif
 
     if ( doT3d ) //convert X,Y,Z in X',Y' as per T3D perspective.
     {
@@ -217,6 +236,13 @@ namespace lib {
       {
         e->Throw("Illegal 3D transformation. (FIXME)");
       }
+      
+      static DDouble x0,y0,xs,ys; //conversion to normalized coords
+      x0=(xLog)?-log10(xStart):-xStart;
+      y0=(yLog)?-log10(yStart):-yStart;
+      xs=(xLog)?(log10(xEnd)-log10(xStart)):xEnd-xStart;xs=1.0/xs;
+      ys=(yLog)?(log10(yEnd)-log10(yStart)):yEnd-yStart;ys=1.0/ys;
+      
       Data3d.zValue = zValue;
       Data3d.Matrix = plplot3d; //try to change for !P.T in future?
             Data3d.x0=x0;
@@ -224,7 +250,7 @@ namespace lib {
             Data3d.xs=xs;
             Data3d.ys=ys;
         switch (axisExchangeCode) {
-          case NORMAL: //X->X Y->Y plane XY
+          case NORMAL3D: //X->X Y->Y plane XY
             Data3d.code = code012;
             break;
           case XY: // X->Y Y->X plane XY
@@ -243,12 +269,18 @@ namespace lib {
             Data3d.code = code201;
             break;
         }
-
         actStream->stransform(gdl3dTo2dTransform, &Data3d);
     }
-
-      // TODO: handle "valid"!
-    bool valid=draw_polyline(e, actStream, xVal, yVal, minVal, maxVal, doMinMax, xLog, yLog, psym, FALSE);
+#ifdef USE_LIBPROJ4
+        if ( mapSet && psym < 1) {
+          GDLgrProjectedPolygonPlot(actStream, ref, NULL, xVal, yVal, false, false, NULL);
+          psym=-psym;
+          if (psym > 0) draw_polyline(actStream, xVal, yVal, minVal, maxVal, doMinMax, xLog, yLog, psym);
+        }
+        else draw_polyline(actStream, xVal, yVal, minVal, maxVal, doMinMax, xLog, yLog, psym);
+#else
+    draw_polyline(actStream, xVal, yVal, minVal, maxVal, doMinMax, xLog, yLog, psym);
+#endif
     if (stopClip) stopClipping(actStream);
   } 
 
@@ -259,6 +291,7 @@ namespace lib {
     private: void post_call(EnvT* e, GDLGStream* actStream)
     {
      if (doT3d) actStream->stransform(NULL,NULL);
+//      actStream->RestoreLayout();
       actStream->lsty(1);//reset linestyle
       actStream->sizeChar(1.0);
     } 
