@@ -32,15 +32,15 @@ namespace lib {
 
   class plot_call : public plotting_routine_call 
   {
-    DDoubleGDL *yVal, *xVal, *zVal, *xTemp, *yTemp;
+    DDoubleGDL *yVal, *xVal, *xTemp, *yTemp;
     SizeT xEl, yEl, zEl;
-    DDouble minVal, maxVal, xStart, xEnd, yStart, yEnd,
-            zValue;
+    DDouble minVal, maxVal, xStart, xEnd, yStart, yEnd, zValue;
     bool doMinMax;
     bool xLog, yLog, wasBadxLog, wasBadyLog;
-    Guard<BaseGDL> xval_guard, yval_guard, zval_guard, xtemp_guard;
+    Guard<BaseGDL> xval_guard, yval_guard, xtemp_guard;
     DLong iso;
     bool doT3d;
+    DFloat  UsymConvX, UsymConvY;
 
 private:
 
@@ -49,7 +49,7 @@ private:
 
     //T3D ?
     static int t3dIx = e->KeywordIx( "T3D");
-    doT3d=(e->KeywordSet(t3dIx)|| T3Denabled(e));
+    doT3d=(e->KeywordSet(t3dIx)|| T3Denabled());
 
     //note: Z (VALUE) will be used uniquely if Z is not effectively defined.
     static int zvIx = e->KeywordIx( "ZVALUE");
@@ -60,9 +60,11 @@ private:
 
     // system variable !P.NSUM first
     DLong nsum=(*static_cast<DLongGDL*>(SysVar::P()-> GetTag(SysVar::P()->Desc()->TagIndex("NSUM"), 0)))[0];
-    e->AssureLongScalarKWIfPresent("NSUM", nsum);
+      static int NSUMIx = e->KeywordIx( "NSUM");
+      e->AssureLongScalarKWIfPresent( NSUMIx, nsum);
 
-    bool polar = (e->KeywordSet("POLAR"));
+      static int polarIx = e->KeywordIx( "POLAR");
+      bool polar = (e->KeywordSet(polarIx));
 
     DDoubleGDL *yValBis, *xValBis;
     Guard<BaseGDL> xvalBis_guard, yvalBis_guard;
@@ -157,38 +159,53 @@ private:
 
     // handle Log options passing via Functions names PLOT_IO/OO/OI
     // the behavior can be superseed by [xy]log or [xy]type
-    string ProName=e->GetProName();
-    if (ProName != "PLOT") {
-      if (ProName == "PLOT_IO") yLog=TRUE;
-      if (ProName == "PLOT_OI") xLog=TRUE;
-      if (ProName == "PLOT_OO") {
-	xLog=TRUE;
-	yLog=TRUE;
+    string ProName = e->GetProName( );
+    if ( ProName != "PLOT" ) {
+      if ( ProName == "PLOT_IO" ) yLog = TRUE;
+      if ( ProName == "PLOT_OI" ) xLog = TRUE;
+      if ( ProName == "PLOT_OO" ) {
+        xLog = TRUE;
+        yLog = TRUE;
       }
     }
 
     // handle Log options passing via Keywords
-    static int xTypeIx = e->KeywordIx("XTYPE");
-    static int yTypeIx = e->KeywordIx("YTYPE");
-    static int xLogIx = e->KeywordIx("XLOG");
-    static int yLogIx = e->KeywordIx("YLOG");
-
+    // note: undocumented keywords [xyz]type still exist and
+    // have priority on [xyz]log !
+    static int xLogIx = e->KeywordIx( "XLOG" );
+    static int yLogIx = e->KeywordIx( "YLOG" );
     if (e->KeywordPresent(xLogIx)) xLog = e->KeywordSet(xLogIx);
     if (e->KeywordPresent(yLogIx)) yLog = e->KeywordSet(yLogIx);
 
-    if (e->KeywordPresent(xTypeIx )) {
-      xLog=e->KeywordSet (xTypeIx );
-    } else {
-      xLog=e->KeywordSet (xLogIx );
+    // note: undocumented keywords [xyz]type still exist and
+    // have priority on [xyz]log ! In fact, it is the modulo (1, 3, 5 ... --> /log)   
+    static int xTypeIx = e->KeywordIx( "XTYPE" );
+    static int yTypeIx = e->KeywordIx( "YTYPE" );
+    static int xType, yType;
+    if (e->KeywordPresent(xTypeIx)) {
+      e->AssureLongScalarKWIfPresent(xTypeIx, xType);
+      if ((xType % 2) == 1) xLog= TRUE; else xLog= FALSE;
+    }
+    if (e->KeywordPresent(yTypeIx)) {
+      e->AssureLongScalarKWIfPresent( yTypeIx, yType);
+      if ((yType % 2) == 1) yLog= TRUE; else yLog= FALSE;
     }
     
-    if (e->KeywordPresent(yTypeIx )) {
-      yLog=e->KeywordSet (yTypeIx );
-    } else {
-      yLog=e->KeywordSet (yLogIx);
+    //  cout << xType<< " " <<xLog << " "<<yType <<" " << yLog << endl;
+
+    static int xTickunitsIx = e->KeywordIx( "XTICKUNITS" );
+    static int yTickunitsIx = e->KeywordIx( "YTICKUNITS" );
+
+    if ( xLog && e->KeywordSet( xTickunitsIx ) ) {
+      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
+      xLog = FALSE;
     }
-    
-    //cout << xLog << " " << yLog << endl;
+    if ( yLog && e->KeywordSet( yTickunitsIx ) ) {
+      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
+      yLog = FALSE;
+    }
+
+    //    cout << xLog << " " << yLog << endl;
 
     // compute adequate values for log scale, warn adequately...
     wasBadxLog = FALSE;
@@ -235,41 +252,27 @@ private:
     minVal = yStart; //to give a reasonable value...
     maxVal = yEnd;   //idem
     doMinMax = false; //although we will not use it...
-    if( e->KeywordSet( "MIN_VALUE") || e->KeywordSet( "MAX_VALUE"))
+    static int MIN_VALUEIx = e->KeywordIx("MIN_VALUE");
+    static int MAX_VALUEIx = e->KeywordIx("MAX_VALUE");
+    if( e->KeywordSet(MIN_VALUEIx) || e->KeywordSet(MAX_VALUEIx))
       doMinMax = true; //...unless explicitely required
-    e->AssureDoubleScalarKWIfPresent( "MIN_VALUE", minVal);
-    e->AssureDoubleScalarKWIfPresent( "MAX_VALUE", maxVal);
+    e->AssureDoubleScalarKWIfPresent( MIN_VALUEIx, minVal);
+    e->AssureDoubleScalarKWIfPresent( MAX_VALUEIx, maxVal);
     yStart=gdlPlot_Max(yStart,minVal);
     yEnd=gdlPlot_Min(yEnd,maxVal);
     //XRANGE and YRANGE overrides all that, but  Start/End should be recomputed accordingly
     DDouble xAxisStart, xAxisEnd, yAxisStart, yAxisEnd;
     bool setx=gdlGetDesiredAxisRange(e, "X", xAxisStart, xAxisEnd);
     bool sety=gdlGetDesiredAxisRange(e, "Y", yAxisStart, yAxisEnd);
-    if(setx && sety)
+    if (setx)
     {
       xStart=xAxisStart;
       xEnd=xAxisEnd;
-      yStart=yAxisStart;
-      yEnd=yAxisEnd;
     }
-    else if (sety)
+    if (sety)
     {
       yStart=yAxisStart;
       yEnd=yAxisEnd;
-// wrong behaviour: x axis limits do not depend from Y values       
-//      //must compute min-max for other axis!
-//      {
-//        gdlDoRangeExtrema(yVal,xVal,xStart,xEnd,yStart,yEnd);
-//      }
-    }
-    else if (setx)
-    {
-      xStart=xAxisStart;
-      xEnd=xAxisEnd;
-      //must compute min-max for other axis!
-      {
-        gdlDoRangeExtrema(xVal,yVal,yStart,yEnd,xStart,xEnd,doMinMax,minVal,maxVal);
-      }
     }
     //handle Nozero option after all that!
     if(!gdlYaxisNoZero(e) && yStart >0 && !yLog ) yStart=0.0;
@@ -277,16 +280,8 @@ private:
 
      //ISOTROPIC
     iso=0;
-    e->AssureLongScalarKWIfPresent( "ISOTROPIC", iso);
-
-    if (doT3d)
-    {
-      //make zVal
-      zEl=xVal->N_Elements();
-      zVal=new DDoubleGDL(dimension(zEl), BaseGDL::NOZERO);
-      zval_guard.Reset(zVal); // delete upon exit
-      for (SizeT i=0; i< zEl ; ++i) (*zVal)[i]=zValue;
-    }
+    static int ISOTROPICIx = e->KeywordIx("ISOTROPIC");
+    e->AssureLongScalarKWIfPresent( ISOTROPICIx, iso);
 
     return false;
   }
@@ -306,10 +301,10 @@ private:
 
      //xStyle and yStyle apply on range values
     if ((xStyle & 1) != 1) {
-      PLFLT intv = AutoIntvAC(xStart, xEnd, xLog);
+      PLFLT intv = gdlAdjustAxisRange(xStart, xEnd, xLog);
     }
     if ((yStyle & 1) != 1) {
-      PLFLT intv = AutoIntvAC(yStart, yEnd, yLog);
+      PLFLT intv = gdlAdjustAxisRange(yStart, yEnd, yLog);
     }
 
     // MARGIN
@@ -318,21 +313,27 @@ private:
     gdlGetDesiredAxisMargin(e, "Y", yMarginB, yMarginT);
 
     // viewport and world coordinates
-    // use POSITION
-    int positionIx = e->KeywordIx( "POSITION");
-    DFloatGDL* boxPosition = e->IfDefGetKWAs<DFloatGDL>( positionIx);
-    if (boxPosition == NULL) boxPosition = (DFloatGDL*) 0xF;
     // set the PLOT charsize before setting viewport (margin depend on charsize)
     gdlSetPlotCharsize(e, actStream);
-
-    static DDouble x0,y0,xs,ys; //conversion to normalized coords
-    x0=(xLog)?-log10(xStart):-xStart;
-    y0=(yLog)?-log10(yStart):-yStart;
-    xs=(xLog)?(log10(xEnd)-log10(xStart)):xEnd-xStart;xs=1.0/xs;
-    ys=(yLog)?(log10(yEnd)-log10(yStart)):yEnd-yStart;ys=1.0/ys;
+    
+    //fix viewport and coordinates for non-3D box. this permits to have correct UsymConv values.
+    // it is important to fix simsize before!
+    gdlSetSymsize(e, actStream);
+    if (gdlSetViewPortAndWorldCoordinates(e, actStream,
+        xLog, yLog,
+        xMarginL, xMarginR, yMarginB, yMarginT,
+        xStart, xEnd, yStart, yEnd, iso)==FALSE) return; //no good: should catch an exception to get out of this mess.
+    actStream->setSymbolSizeConversionFactors();
 
     if (doT3d)
     {
+
+      static DDouble x0,y0,xs,ys; //conversion to normalized coords
+      x0=(xLog)?-log10(xStart):-xStart;
+      y0=(yLog)?-log10(yStart):-yStart;
+      xs=(xLog)?(log10(xEnd)-log10(xStart)):xEnd-xStart;xs=1.0/xs;
+      ys=(yLog)?(log10(yEnd)-log10(yStart)):yEnd-yStart;ys=1.0/ys;
+    
       DDoubleGDL* plplot3d;
       DDouble az, alt, ay, scale;
       ORIENTATION3D axisExchangeCode;
@@ -342,14 +343,13 @@ private:
       {
         e->Throw("Illegal 3D transformation. (FIXME)");
       }
-
       if (gdlSet3DViewPortAndWorldCoordinates(e, actStream, plplot3d, xLog, yLog,
         xStart, xEnd, yStart, yEnd) == FALSE) return;
       gdlSetGraphicsForegroundColorFromKw(e, actStream);
 
       DDouble  t3xStart, t3xEnd, t3yStart, t3yEnd, t3zStart, t3zEnd;
       switch (axisExchangeCode) {
-        case NORMAL: //X->X Y->Y plane XY
+        case NORMAL3D: //X->X Y->Y plane XY
           t3xStart=(xLog)?log10(xStart):xStart,
           t3xEnd=(xLog)?log10(xEnd):xEnd,
           t3yStart=(yLog)?log10(yStart):yStart,
@@ -385,7 +385,7 @@ private:
           actStream->w3d(scale, scale, scale,
           t3xStart,t3xEnd,t3yStart,t3yEnd,t3zStart,t3zEnd,
           alt, az);
-          gdlAxis3(e, actStream, "Z", xStart, xEnd, xLog, 0);
+          gdlAxis3(e, actStream, "Z", xStart, xEnd, xLog,0);
           gdlAxis3(e, actStream, "Y", yStart, yEnd, yLog);
           break;
         case YZ: // X->X Y->Z plane XZ
@@ -428,12 +428,52 @@ private:
           gdlAxis3(e, actStream, "X", yStart, yEnd, yLog);
           break;
       }
-      // title and sub title
-      gdlWriteTitleAndSubtitle(e, actStream);
-
+      //finish box: since there is no option in plplot for upper box 3d axes, do it ourselve:
+      Data3d.zValue = 0;
+      Data3d.Matrix = plplot3d; //try to change for !P.T in future?
+            Data3d.x0=x0;
+            Data3d.y0=y0;
+            Data3d.xs=xs;
+            Data3d.ys=ys;
+      switch (axisExchangeCode) {
+          case NORMAL3D: //X->X Y->Y plane XY
+            Data3d.code = code012;
+            actStream->stransform(gdl3dTo2dTransform, &Data3d);      
+            actStream->join(t3xStart,t3yEnd,t3xEnd,t3yEnd);
+            actStream->join(t3xEnd,t3yStart,t3xEnd,t3yEnd);
+            break;
+          case XY: // X->Y Y->X plane XY
+            Data3d.code = code012;
+            actStream->stransform(gdl3dTo2dTransform, &Data3d);      
+            actStream->join(t3yStart,t3xEnd,t3yEnd,t3xEnd);
+            actStream->join(t3yEnd,t3xStart,t3yEnd,t3xEnd);
+            break;
+          case XZ: // Y->Y X->Z plane YZ
+            Data3d.code = code210;
+            actStream->stransform(gdl3dTo2dTransform, &Data3d);      
+            actStream->join(t3zStart,t3yStart,t3zEnd,t3yStart);
+            actStream->join(t3zEnd,t3yStart,t3zEnd,t3yEnd);
+            break;
+//          case YZ: // X->X Y->Z plane XZ
+//            cerr<<"yz"<<endl;
+//            Data3d.code = code021;
+//            actStream->stransform(gdl3dTo2dTransform, &Data3d);      
+//            break;
+//          case XZXY: //X->Y Y->Z plane YZ
+//            cerr<<"xzxy"<<endl;
+//            Data3d.code = code120;
+//            actStream->stransform(gdl3dTo2dTransform, &Data3d);      
+//            break;
+//          case XZYZ: //X->Z Y->X plane XZ
+//            cerr<<"xzyz"<<endl;
+//            Data3d.code = code201;
+//            actStream->stransform(gdl3dTo2dTransform, &Data3d);      
+//            break;
+        }
+      
       //data: will plot using coordinates transform.
       //TODO: unless PSYM=0 (optimize)
-
+      
       Data3d.zValue = zValue;
       Data3d.Matrix = plplot3d; //try to change for !P.T in future?
             Data3d.x0=x0;
@@ -441,7 +481,7 @@ private:
             Data3d.xs=xs;
             Data3d.ys=ys;
       switch (axisExchangeCode) {
-          case NORMAL: //X->X Y->Y plane XY
+          case NORMAL3D: //X->X Y->Y plane XY
             Data3d.code = code012;
             break;
           case XY: // X->Y Y->X plane XY
@@ -462,17 +502,13 @@ private:
         }
 
       actStream->stransform(gdl3dTo2dTransform, &Data3d);
+      // title and sub title
+      gdlWriteTitleAndSubtitle(e, actStream);
 
     } else
     {
-      //fix viewport and coordinates for box
-      if (gdlSetViewPortAndWorldCoordinates(e, actStream, boxPosition,
-          xLog, yLog,
-          xMarginL, xMarginR, yMarginB, yMarginT,
-	      xStart, xEnd, yStart, yEnd, iso)==FALSE) return; //no good: should catch an exception to get out of this mess.
       //current pen color...
       gdlSetGraphicsForegroundColorFromKw(e, actStream);
-
       gdlBox(e, actStream, xStart, xEnd, yStart, yEnd, xLog, yLog);
     }
   } 
@@ -481,22 +517,16 @@ private:
     {
       DLong psym;
       // plot the data
-      int nodataIx = e->KeywordIx( "NODATA"); 
+      static int nodataIx = e->KeywordIx( "NODATA"); 
       if ( !e->KeywordSet(nodataIx) )
       {
-        //get psym
-        gdlGetPsym(e, psym); //PSYM
-        //handle clipping
-        bool doClip=(e->KeywordSet("CLIP")||e->KeywordSet("NOCLIP"));
-        // make all clipping computations BEFORE setting graphic properties (color, size)
         bool stopClip=false;
-        if ( doClip )  if ( startClipping(e, actStream, false)==TRUE ) stopClip=true;
+        if ( startClipping(e, actStream, false)==TRUE ) stopClip=true;
         // here graphic properties
         gdlSetPenThickness(e, actStream);
-        gdlSetSymsize(e, actStream);
         gdlSetLineStyle(e, actStream);
-        // TODO: handle "valid"!
-        bool valid=draw_polyline(e, actStream, xVal, yVal, minVal, maxVal, doMinMax, xLog, yLog, psym, FALSE);
+        gdlGetPsym(e, psym); //PSYM
+        draw_polyline(actStream, xVal, yVal, minVal, maxVal, doMinMax, xLog, yLog, psym);
         if (stopClip) stopClipping(actStream);
       }
     } 
