@@ -44,26 +44,35 @@ namespace lib
     bool handle_args (EnvT* e)
     {
 
-      // undocumented keywords [xyz]type still exist and
-      // had priority on [xyz]log !
-      
-      if (e->KeywordPresent( "XTYPE" )) {
-	xLog=e->KeywordSet ( "XTYPE" );
-      } else {
-	xLog=e->KeywordSet ( "XLOG" );
-      }
+    // handle Log options passing via Keywords
+    // note: undocumented keywords [xyz]type still exist and
+    // have priority on [xyz]log ! 
+    static int xTypeIx = e->KeywordIx( "XTYPE" );
+    static int yTypeIx = e->KeywordIx( "YTYPE" );
+    static int zTypeIx = e->KeywordIx( "ZTYPE" );
+    static int xLogIx = e->KeywordIx( "XLOG" );
+    static int yLogIx = e->KeywordIx( "YLOG" );
+    static int zLogIx = e->KeywordIx( "ZLOG" );
+    static int xTickunitsIx = e->KeywordIx( "XTICKUNITS" );
+    static int yTickunitsIx = e->KeywordIx( "YTICKUNITS" );
+    static int zTickunitsIx = e->KeywordIx( "ZTICKUNITS" );
 
-      if (e->KeywordPresent( "YTYPE" )) {
-	yLog=e->KeywordSet ( "YTYPE" );
-      } else {
-	yLog=e->KeywordSet ( "YLOG" );
-      }
+    if ( e->KeywordPresent( xTypeIx ) ) xLog = e->KeywordSet( xTypeIx ); else xLog = e->KeywordSet( xLogIx );
+    if ( e->KeywordPresent( yTypeIx ) ) yLog = e->KeywordSet( yTypeIx ); else yLog = e->KeywordSet( yLogIx );
+    if ( e->KeywordPresent( zTypeIx ) ) zLog = e->KeywordSet( zTypeIx ); else zLog = e->KeywordSet( zLogIx );
 
-      if (e->KeywordPresent( "ZTYPE" )) {
-	zLog=e->KeywordSet ( "ZTYPE" );
-      } else {
-	zLog=e->KeywordSet ( "ZLOG" );
-      }
+    if ( xLog && e->KeywordSet( xTickunitsIx ) ) {
+      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
+      xLog = FALSE;
+    }
+    if ( yLog && e->KeywordSet( yTickunitsIx ) ) {
+      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
+      yLog = FALSE;
+    }
+    if ( zLog && e->KeywordSet( zTickunitsIx ) ) {
+      Message( "PLOT: LOG setting ignored for Date/Time TICKUNITS." );
+      zLog = FALSE;
+    }
 
       if ( nParam ( )==1 )
       {
@@ -137,27 +146,16 @@ namespace lib
       DDouble xAxisStart, xAxisEnd, yAxisStart, yAxisEnd;
       bool setx=gdlGetDesiredAxisRange(e, "X", xAxisStart, xAxisEnd);
       bool sety=gdlGetDesiredAxisRange(e, "Y", yAxisStart, yAxisEnd);
-      if(setx && sety)
+      if (setx)
       {
         xStart=xAxisStart;
         xEnd=xAxisEnd;
-        yStart=yAxisStart;
-        yEnd=yAxisEnd;
       }
-      else if (sety)
+      if (sety)
       {
         yStart=yAxisStart;
         yEnd=yAxisEnd;
       }
-      else if (setx)
-      {
-        xStart=xAxisStart;
-        xEnd=xAxisEnd;
-        //must compute min-max for other axis!
-        {
-          gdlDoRangeExtrema(xVal,yVal,yStart,yEnd,xStart,xEnd);
-        }
-      }
   #undef UNDEF_RANGE_VALUE
       // z range
       datamax=0.0;
@@ -174,7 +172,7 @@ namespace lib
         shades=e->GetKWAs<DLongGDL>( shadesIx ); doShade=true;
       } else {
         // Get COLOR from PLOT system variable
-        static DStructGDL* pStruct=SysVar::P();
+        DStructGDL* pStruct=SysVar::P();   //MUST NOT BE STATIC, due to .reset 
         shades=new DLongGDL( 1, BaseGDL::NOZERO );
         shades_guard.Init ( shades ); // delete upon exit
         shades=static_cast<DLongGDL*>(pStruct->GetTag(pStruct->Desc()->TagIndex("COLOR"), 0)); doShade=false;
@@ -188,7 +186,7 @@ namespace lib
     {
       //T3D
       static int t3dIx = e->KeywordIx( "T3D");
-      bool doT3d=(e->KeywordSet(t3dIx)|| T3Denabled(e));
+      bool doT3d=(e->KeywordSet(t3dIx)|| T3Denabled());
       //ZVALUE
       static int zvIx = e->KeywordIx( "ZVALUE");
       DDouble zValue=0.0;
@@ -224,20 +222,23 @@ namespace lib
 
       if ( ( xStyle&1 )!=1 )
       {
-        PLFLT intv=AutoIntvAC ( xStart, xEnd, xLog );
+        PLFLT intv=gdlAdjustAxisRange ( xStart, xEnd, xLog );
       }
 
       if ( ( yStyle&1 )!=1 )
       {
-        PLFLT intv=AutoIntvAC ( yStart, yEnd, yLog );
+        PLFLT intv=gdlAdjustAxisRange ( yStart, yEnd, yLog );
       }
 
-      bool hasMinVal=e->KeywordPresent("MIN_VALUE");
-      bool hasMaxVal=e->KeywordPresent("MAX_VALUE");
+      static int MIN_VALUEIx = e->KeywordIx( "MIN_VALUE");
+      static int MAX_VALUEIx = e->KeywordIx( "MAX_VALUE");
+
+      bool hasMinVal=e->KeywordPresent(MIN_VALUEIx);
+      bool hasMaxVal=e->KeywordPresent(MAX_VALUEIx);
       DDouble minVal=datamin;
       DDouble maxVal=datamax;
-      e->AssureDoubleScalarKWIfPresent ( "MIN_VALUE", minVal );
-      e->AssureDoubleScalarKWIfPresent ( "MAX_VALUE", maxVal );
+      e->AssureDoubleScalarKWIfPresent ( MIN_VALUEIx, minVal );
+      e->AssureDoubleScalarKWIfPresent ( MAX_VALUEIx, maxVal );
 
       if (!setZrange) {
         zStart=max(minVal,zStart);
@@ -247,7 +248,7 @@ namespace lib
       // then only apply expansion  of axes:
       if ( ( zStyle&1 )!=1 )
       {
-        PLFLT intv=AutoIntvAC ( zStart, zEnd, zLog );
+        PLFLT intv=gdlAdjustAxisRange ( zStart, zEnd, zLog );
       }
 
       // background BEFORE next plot since it is the only place plplot may redraw the background...
@@ -263,7 +264,8 @@ namespace lib
       PLFLT az=30.0;
       //set az and ax (alt)
       DFloat alt_change=alt;
-      e->AssureFloatScalarKWIfPresent("AX", alt_change);
+      static int AXIx=e->KeywordIx("AX");
+      e->AssureFloatScalarKWIfPresent(AXIx, alt_change);
       alt=alt_change;
 
       alt=fmod(alt,360.0); //restrict between 0 and 90 for plplot!
@@ -272,7 +274,8 @@ namespace lib
         e->Throw ( "SHADE_SURF: AX restricted to [0-90] range by plplot (fix plplot!)" );
       }
       DFloat az_change=az;
-      e->AssureFloatScalarKWIfPresent("AZ", az_change);
+      static int AZIx=e->KeywordIx("AZ");
+      e->AssureFloatScalarKWIfPresent(AZIx, az_change);
       az=az_change;
 
       //now we are in plplot different kind of 3d
@@ -297,7 +300,7 @@ namespace lib
         {
           DDoubleGDL* t3dMatrix=plplot3d->Dup();
           SelfTranspose3d(t3dMatrix);
-          static DStructGDL* pStruct=SysVar::P();
+          DStructGDL* pStruct=SysVar::P();   //MUST NOT BE STATIC, due to .reset 
           static unsigned tTag=pStruct->Desc()->TagIndex("T");
           for (int i=0; i<t3dMatrix->N_Elements(); ++i )(*static_cast<DDoubleGDL*>(pStruct->GetTag(tTag, 0)))[i]=(*t3dMatrix)[i];
           GDLDelete(t3dMatrix);
@@ -368,15 +371,18 @@ namespace lib
         if (yLog) for ( SizeT i=0; i<cgrid1.ny; i++ ) cgrid1.yg[i] = cgrid1.yg[i]>0?log10(cgrid1.yg[i]):1E-12;
 
         // Important: make all clipping computations BEFORE setting graphic properties (color, size)
-        bool doClip=(e->KeywordSet("CLIP")||e->KeywordSet("NOCLIP"));
+        static int NOCLIPIx = e->KeywordIx("NOCLIP");
+        static int CLIPIx = e->KeywordIx("CLIP");
+        bool doClip=(e->KeywordSet(CLIPIx)||e->KeywordSet(NOCLIPIx));
         bool stopClip=false;
-        if ( doClip )  if ( startClipping(e, actStream, false)==TRUE ) stopClip=true;
+        if ( doClip )  if ( startClipping(e, actStream)==false ) stopClip=true;
 
         gdlSetGraphicsForegroundColorFromKw ( e, actStream );
         //mesh option
         PLINT meshOpt;
         meshOpt=(doShade)?MAG_COLOR:0;
-        if (e->KeywordSet ( "SKIRT" )) meshOpt+=DRAW_SIDES;
+        static int SKIRTIx = e->KeywordIx("SKIRT");
+        if (e->KeywordSet ( SKIRTIx )) meshOpt+=DRAW_SIDES;
 
         // Get decomposed value for shades
         DLong decomposed=GraphicsDevice::GetDevice()->GetDecomposed();
