@@ -32,7 +32,8 @@ void print_tree::pr_name( RefAST node )
 
     RefDNode dNode = static_cast<RefDNode>( node);
 
-    str = node->getText();
+//    str =  dNode->getText();
+    str = std::string("[") + i2s(dNode->getType()) + ":" + i2s(dNode->CData()) + "]" + node->getText();
 //     printf("%s ", str.c_str());
     printf("%s(%d) ", str.c_str(), dNode->getLine());
 } // pr_name
@@ -132,6 +133,11 @@ void print_tree::pr_leaves( ProgNodeP top )
 	pr_top( t );
       else
 	pr_name( t );
+      if( t->getNextSibling() != NULL && t->KeepRight())
+      {
+	printf("^");
+	break;
+      }
     }
 } // pr_leaves
 
@@ -180,6 +186,11 @@ void print_tree::pr_top( ProgNodeP top )
       {    
 	if (is_nonleaf( t ))
 	  first = false;
+	if( t->getNextSibling() != NULL && t->KeepRight())
+	{
+	  printf("^");
+	  break;
+	}
       }
     pr_leaves( top );
   }
@@ -214,6 +225,11 @@ void print_tree::pr_tree( ProgNodeP top )
     indent_level = 0;
     pr_top( t );
     printf("\n");
+    if( t->getNextSibling() != NULL && t->KeepRight())
+    {
+      printf("^");
+      break;
+    }
   }
 } // pr_tree
 
