@@ -329,7 +329,7 @@ PyObject *GDLSub( PyObject *self, PyObject *argTuple, PyObject *kwDict,
 		  }
 	      }
 	  
-	    sub = proList[ proIx];
+	    sub = funList[ proIx];
 	  }
       }
     else
@@ -373,7 +373,7 @@ PyObject *GDLSub( PyObject *self, PyObject *argTuple, PyObject *kwDict,
     if( libCall)
       e = new EnvT( NULL, sub);
     else
-      e = new EnvUDT( NULL, sub);
+      e = new EnvUDT( NULL, static_cast<DSubUD*>(sub));
 
     Guard< EnvBaseT> e_guard( e);
 
@@ -537,6 +537,15 @@ extern "C" {
     // instantiate the interpreter (creates $MAIN$ environment)
     interpreter = new DInterpreter();
 
+    // Ole: enable GDL PATH!
+    string gdlPath=GetEnvString("GDL_PATH");
+    if( gdlPath == "") gdlPath=GetEnvString("IDL_PATH");
+    if( gdlPath == "")
+      {
+        gdlPath = "+" GDLDATADIR "/lib";
+      }
+    SysVar::SetGDLPath( gdlPath);
+    
     PyObject* m = Py_InitModule("GDL", GDLMethods);
 
     gdlError = PyErr_NewException((char*)"GDL.error", NULL, NULL);
