@@ -168,14 +168,17 @@ typedef std::set< DPtr>               DPtrListT;
 //class ArrayIndexT;
 //typedef std::vector<ArrayIndexT*> ArrayIndexVectorT;
 
+// // for GUI (but also used in GDLEventHandler()
+// typedef DLong WidgetIDT;
+
 // to resolve include conflict (declared in gdlexception.hpp)
 void ThrowGDLException( const std::string& str);
 
 // for OpenMP (defined in objects.cpp - must be declared here)
 extern DLong CpuHW_NCPU;
 extern DLong CpuTPOOL_NTHREADS;
-extern DLong CpuTPOOL_MIN_ELTS;
-extern DLong CpuTPOOL_MAX_ELTS;
+extern DLong64 CpuTPOOL_MIN_ELTS;
+extern DLong64 CpuTPOOL_MAX_ELTS;
 
 // convert something to string
 template <typename T>
@@ -232,7 +235,7 @@ private:
   
   Guard& operator=( Guard& r)
   {
-    if( &r == this) return;
+    if( &r == this) return *this;
     delete guarded;
     guarded = r.guarded;
     r.guarded = NULL;
@@ -315,7 +318,7 @@ public:
   
   void Reset( T* newGuarded)
   {
-    delete guarded;
+    delete[] guarded;
     guarded = newGuarded;
   }  
   void Release()
